
/* Kirk Christiansen */
/* BOARDS.C          */

/* For TankGame      */
/* This 'C' module contains the code   */
/* for all the board drawing functions */
/* for Tankgame.                       */

#ifndef  _BOARDS
/*************************************************************************/
#include "kcgrphcs.c"
#include "tankvars.c"
#include "powerups.c"
/*************************************************************************/
/* Function Prototypes */
void drawboard(int boardnumber, int color);
void drawboardoutline(int clr);
void draw_statbox(void);
void draw_board_items (void);

void draw_void_board(int clr);
void draw_city_board(int clr);
void draw_city_board2(int clr);
void draw_fortress_board(int clr);
void draw_river_board(int clr);
void draw_pillar_board(int clr);
void draw_mountain_board(int clr);
void draw_palace_board(int clr);
void draw_split_exit_board(int clr);
void draw_snake_board(int clr);
void draw_multi_teleport_board(void);
void draw_tank_yard_board(void);
void draw_the_13th(void);
/*************************************************************************/
void drawboardoutline(int clr)
/* This small function simply draws the outer wall of the game */
/* board.  This wall is the boundary of the playing grid.      */
{
  /* DRAW MAIN WALL AROUND BOARD */
  /* horizontals */
  bar_blit(0,0,320,5,clr,screen_buffer);
  bar_blit(0,164,320,5,clr,screen_buffer);
  /*verticals */
  bar_blit(0,5,5,159,clr,screen_buffer);
  bar_blit(315,5,5,159,clr,screen_buffer);
}
/*************************************************************************/
void draw_statbox(void)
/* Blits the appropriate stat bar/box on the bottom of the screen */
/* depending on what type of game is being played. */
{
  switch (gametype)
  {
    case (FULLGAME) : blit (0,170,320,30,screen_buffer,stat_picts[13]);
		      break;
    case (TWOPLYR_FULLGAME) : blit (0,170,320,30,screen_buffer,
				    stat_picts[15]);
			      break;
    default : /* must be a MATCH, or NULL_MODEM_MATCH */
	      blit (0,170,320,30,screen_buffer,stat_picts[14]);
	      break;
  };
}
/*************************************************************************/
void drawboard(int boardnumber, int color)
/* This function draws the board pattern for the inputed board number */
{

  switch (boardnumber)
   {
     case VOID :  draw_void_board(color);
		  break;
     case CITY :  draw_city_board(color);
		  break;
     case FORTRESS : draw_fortress_board(color);
		     break;
     case RIVER : draw_river_board(color);
		  break;
     case PILLARS : draw_pillar_board(color);
		    break;
     case MOUNTAINS : draw_mountain_board(color);
		      break;
     case PALACE : draw_palace_board(color);
		   break;
     case CITY2 : draw_city_board2(color);
		  break;
     case SPLITEXIT : draw_split_exit_board(color);
		      break;
     case SNAKEBRD :  draw_snake_board(color);
		      break;
     case MULTI_TELEPORT_BRD : draw_multi_teleport_board();
			       break;
     case TANK_YARD : draw_tank_yard_board();
		      break;
     case THE_13TH  : draw_the_13th();
		      break;
   }

  /* Draw board items that can be draw ontop of */
  draw_board_items();
  /* Draw the board outline */
  drawboardoutline(color);
  /* Draw the stats box on bottom of screen */
  draw_statbox();
}
/*************************************************************************/
void draw_board_items (void)
/* This function draw certain items that can be drawn on top of by   */
/* tanks and projectiles.  A river could be fired over so you must   */
/* draw the river first before you draw the projectiles crossing it. */
{
  int x,y;
  static char  flow=0;
  static char  count=0;
  int clr1,clr2;

  /* First, draw any power_up pills if they exist */
  draw_power_ups ();

  /* Now, draw the misc picts if can be run over/fired over....etc */
  switch (boardnumber)
   {
     case RIVER :
	   {
	      /* put river parts */
	      blit (128,4,RIVERWIDTH,RIVERHEIGHT,
		    screen_buffer,board_picts[5]);

	      blit (131,95,RIVERWIDTH,RIVERHEIGHT,
		    screen_buffer,board_picts[5]);

	     /* put the current waves */
		if (flow++ < 10)
		 {
		  for (y=7;y<160;y+=15)
		  {
		   for (x=137;x<180;x+=32)
		     {
		      putpixel(x,y,AQUA,screen_buffer);
		     }
		      putpixel(153,y+3,AQUA,screen_buffer);
		   }
		 }

		else
		 {
		  for (y=13;y<160;y+=15)
		  {
		    for (x=137;x<180;x+=32)
		     {
		      putpixel(x,y,AQUA,screen_buffer);
		     }
		    putpixel(153,y+3,AQUA,screen_buffer);
		   }
		  if (flow++ > 20)
		    flow=0;
		 }

	      /* put the bridge */
		blit(130,75,BRIDGEWIDTH,BRIDGEHEIGHT,
		screen_buffer,board_picts[4]);
		break;
	   }
     case PALACE:   /* place the teleport box */
	     {
	      if (count++ < 8)
		 {
		 dual_color_blit(137,72,TELE_GROUND_WIDTH,TELE_GROUND_HEIGHT,
		       LIGHTMAGENTA,LIGHTBLUE,screen_buffer,board_picts[6]);
		 }
	      else
		 {
		dual_color_blit(137,72,TELE_GROUND_WIDTH,TELE_GROUND_HEIGHT,
		      LIGHTBLUE,LIGHTMAGENTA,screen_buffer,board_picts[6]);
		 if (count++ > 16) count=0;
		 }
	       break;
	      }
     case SPLITEXIT:
	      {
	      if (count++ < 8)
		 {
		 /* blit 'To level 11' */
		 dual_color_blit(230,115,51,24,LIGHTMAGENTA,LIGHTBLUE,
				 screen_buffer,board_picts[10]);
		 /* blit 'To level 9' */
		 dual_color_blit(230,47,51,24,LIGHTMAGENTA,LIGHTBLUE,
				 screen_buffer,board_picts[11]);
		 }
	      else
		 {
		 /* blit 'To level 11' */
		 dual_color_blit(230,115,51,24,LIGHTBLUE,LIGHTMAGENTA,
				 screen_buffer,board_picts[10]);
		 /* blit 'To level 9' */
		 dual_color_blit(230,47,51,24,LIGHTBLUE,LIGHTMAGENTA,
				 screen_buffer,board_picts[11]);
		 if (count++ > 16) count=0;
		 }
	       break;
	      }
     case SNAKEBRD:
	      {
	      if (count++ < 8)
		 {
		 /* blit 'To level 10' telebox */
		 dual_color_blit(288,9,27,20,LIGHTMAGENTA,LIGHTBLUE,
				 screen_buffer,board_picts[14]);
		 }
	      else
		 {
		 /* blit 'To level 10' telebox */
		 dual_color_blit(288,9,27,20,LIGHTBLUE,LIGHTMAGENTA,
				 screen_buffer,board_picts[14]);
		 if (count++ > 16) count=0;
		 }
	       break;
	      } /* end case SNAKEBRD */
     case MULTI_TELEPORT_BRD:   /* place the teleport boxes */
	     {
	      if (count++ < 8)
		 {  clr1=LIGHTBLUE; clr2=LIGHTMAGENTA; }
	      else
		 { clr1=LIGHTMAGENTA; clr2=LIGHTBLUE;
		   if(count++>16) count=0; }
	      /* NOW DRAW THE TELEPORTERS with the appropiate colors */
	       for (x=0;x<3;x++)
		{
		 /* upper 3 teleporters */
		 dual_color_blit(103+(x*40),54,TELE_GROUND_WIDTH,
		 TELE_GROUND_HEIGHT,clr1,clr2,screen_buffer,board_picts[6]);
		 /* lower  3 teleporters */
		 dual_color_blit(103+(x*40),87,TELE_GROUND_WIDTH,
		 TELE_GROUND_HEIGHT,clr2,clr1,screen_buffer,board_picts[6]);
		}
		break;
	      } /* end case MULTI_TELEPORTER_BRD */
   }; /* end switch*/
}
/**************************************************************************/
void draw_void_board(int clr)
{
  /* This board is exactly what is says....void. No obstacles :) */
  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};
}
/*************************************************************************/
void draw_city_board(int clr)
{
  int x, y;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

  for (x=47;x<250;x+=92)
   for (y=40;y<150;y+=60)
     {
      blit(x-10,y-5,60,40,screen_buffer,board_picts[0]);
     }
}
/*************************************************************************/
void draw_fortress_board(int clr)
{
  /* midleft vertical */
  solid_rect (40,67,55,92,clr,screen_buffer);
  /* midright vertical */
  solid_rect (240,63,255,105,clr,screen_buffer);
  /* horizontal lowleft */
  solid_rect (30,109,100,122,clr,screen_buffer);
  /* square lowerleft corner */
  solid_rect (30,139,45,149/**/,clr,screen_buffer);
  /* vertical lowerleft corner */
  solid_rect (65,122,80,149,clr,screen_buffer);
  /* horizontal upleft */
  solid_rect (45,34,80,46,clr,screen_buffer);
  /* vertical upleft */
  solid_rect (30,25,45,46,clr,screen_buffer);
  /* vertical midleft */
  solid_rect (80,34,95,77,clr,screen_buffer);
  /* box midupleft */
  solid_rect (80,5,95,13,clr,screen_buffer);
  /* midupper vertical */
  solid_rect (125,5,145,29,clr,screen_buffer);
  /* middle vertical */
  solid_rect (125,63,145,118,clr,screen_buffer);
  /* lowmiddle square */
  solid_rect (125,147,145,165,clr,screen_buffer);
  /* upright horizontal */
  solid_rect (170,21,275,34,clr,screen_buffer);
  /* midright square */
  solid_rect (175,63,200,84,clr,screen_buffer);
  /* lowmidright vertical */
  solid_rect (175,113,195,147,clr,screen_buffer);
  /* lowright horizontal */
  solid_rect (195,130,225,147,clr,screen_buffer);
  /* lowright square */
  solid_rect (250,126,275,147,clr,screen_buffer);
}
/*************************************************************************/
void draw_mountain_board(int clr)
{
  /* This board is not complete yet, but will consist of a bridge */
  /* over a river.                                                */

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};
}
/*************************************************************************/
void draw_pillar_board(int clr)
{
 int x,y;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

  for (x=28;x<275;x+=40)
   for (y=28;y<155;y+=34)
    {
      blit(x,y-2,20,15,screen_buffer,board_picts[2]);
    }
}
/*************************************************************************/
void draw_river_board(int clr)
{
 int x,y;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

  /* Draw some trees */
  blit (25,30,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (81,18,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (89,55,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (35,85,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (95,125,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);

  blit (225,31,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (250,131,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  blit (210,75,TREE_WIDTH,TREE_HEIGHT,screen_buffer,board_picts[7]);
  /* Draw some bushes */
  blit (30,155,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);
  blit (74,31,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);
  blit (122,105,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);

  blit (282,60,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);
  blit (300,155,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);
  blit (200,151,BUSH_WIDTH,BUSH_HEIGHT,screen_buffer,board_picts[9]);
  /* Draw some dead trees */
  blit (245,100,DEADTREE_WIDTH,DEADTREE_HEIGHT,screen_buffer,board_picts[8]);

}
/*************************************************************************/
void draw_palace_board(int clr)
/* This board consists of a teleportation cubical in the middle */
{
  int x, y;

  /* Middle teleport box */
  /* left bar on box */
  solid_rect (125,63,135,105,clr,screen_buffer);
  /* knob on bottom of left bar */
  solid_rect (135,93,140,105,clr,screen_buffer);
  /* right bar on box */
  solid_rect (165,63,175,105,clr,screen_buffer);
  /* knob on bottom of right bar */
  solid_rect (160,93,165,105,clr,screen_buffer);
  /* top bar on box */
  solid_rect (135,63,165,71,clr,screen_buffer);
  /* draw horizontal bars on the 4 corner L formations */
  for (x=55;x<=175;x+=120)
    for (y=25;y<=126;y+=101)
     {
       solid_rect (x,y,x+70,y+17,clr,screen_buffer);
     }

  /* draw vertical small squares on the 4 corner L formations */
  for (x=50;x<=225;x+=175)
    for (y=42;y<=97;y+=55)
     {
       solid_rect (x,y,x+25,y+29,clr,screen_buffer);
     }
}
/*************************************************************************/
void draw_city_board2(int clr)
{
 int i;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

    /* up left midsize bldg */
    blit(35,25,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
    /* left hand big bldg */
    blit (35,75,BIGBLDGWIDTH,BIGBLDGHEIGHT,screen_buffer,board_picts[0]);

    /* low left 3 small bldgs*/
    for (i=0;i<3;i++)
     {
      dual_color_blit(35+(i*45),133,SMALLBLDGWIDTH,SMALLBLDGHEIGHT,WALLCOLOR,
		      WALLCOLOR3,screen_buffer,board_picts[2]);
     }

   /* upper mid bldg (2 actually put together) */
   for (i=0;i<2;i++)
    {
     blit(90+(i*13),25,MIDBLDGWIDTH,MIDBLDGHEIGHT,
	  screen_buffer,board_picts[1]);
    }

    /* middle mid bldg */
     blit(110,83,MIDBLDGWIDTH,MIDBLDGHEIGHT,
	  screen_buffer,board_picts[1]);

     /* mid/right mid bldg */
     blit(172,70,MIDBLDGWIDTH,MIDBLDGHEIGHT,
	  screen_buffer,board_picts[1]);

   /* upper right mid bldg (2 actually put together) */
   for (i=0;i<2;i++)
    {
     blit(170+(i*13),25,MIDBLDGWIDTH,MIDBLDGHEIGHT,
	  screen_buffer,board_picts[1]);
    }

  /* right hand long bldg */
     blit(245,25,LONGBLDGWIDTH,LONGBLDGHEIGHT,
	  screen_buffer,board_picts[3]);
  /* low right big bldg */
    blit (235,100,BIGBLDGWIDTH,BIGBLDGHEIGHT,screen_buffer,board_picts[0]);

  /* low/right mid bldg */
     blit(172,122,MIDBLDGWIDTH,MIDBLDGHEIGHT,
	  screen_buffer,board_picts[1]);

}
/*************************************************************************/
void draw_split_exit_board(int clr)
{
 int i;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

  /* Draw upright hand wall */
    blit (286,5,29,24,screen_buffer,board_picts[12]);
    blit (286,25,29,24,screen_buffer,board_picts[12]);

  /* Draw midright hand wall */
    blit (286,68,29,24,screen_buffer,board_picts[12]);
    blit (286,75,29,24,screen_buffer,board_picts[12]);
    blit (286,93,29,24,screen_buffer,board_picts[12]);

  /* Draw lowright hand wall */
    blit (286,139,29,24,screen_buffer,board_picts[12]);

  /* Draw the top part of encompass wall */
  for (i=273;i>200;i-=12)
    blit (i,83,17,13,screen_buffer,board_picts[13]);
  /* Draw the left side part of encompass wall */
  for (i=83;i<157;i+=10)
    blit (195,i,17,13,screen_buffer,board_picts[13]);

  /* Blit some bldgs on Left side of board */
  for (i=28;i<200;i+=60)
    blit (i,25,29,24,screen_buffer,board_picts[12]);
  for (i=28;i<200;i+=60)
    blit (i,71,29,24,screen_buffer,board_picts[12]);
  for (i=28;i<200;i+=60)
    blit (i,119,29,24,screen_buffer,board_picts[12]);
}
/*************************************************************************/
void draw_snake_board(int clr)
{
 int i;

  /* Do something with clr to avoid compilier Warning :) */
  if (clr==BLACK) {};

  /* Draw lower thick wall */
  for (i=3;i<250;i+=50)
   {
     dual_color_blit(i,97,60,40,WALLCOLOR2,
	      WALLCOLOR3,screen_buffer,board_picts[0]);
   }

  /* Draw upper thick wall */
  for (i=62;i<280;i+=50)
   {
     dual_color_blit(i,31,60,40,WALLCOLOR2,
	      WALLCOLOR3,screen_buffer,board_picts[0]);
   }
}
/*************************************************************************/
void draw_multi_teleport_board(void)
/* This cool board has a structure in the middle of the board that */
/* has 6 teleporters!  You teleport from one to another.           */
{
  int i;

  /* FIRST, DRAW THE INNER MULTI-TELEPORT BOX WITH 6 CELLS */
  /* First we must draw the verticals one by one, all 4 of them */
  for (i=0;i<4;i++)
   {
    dual_color_blit (90+(i*40),53,13,34,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[17]);
    dual_color_blit (90+(i*40),81,13,34,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[17]);
   }
  /* Now the 3 horizontals */
  for (i=0;i<3;i++)
   {
    dual_color_blit (99+(i*40),75,33,13,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[16]);
   }

  /* NOW DRAW THE WALLS SURROUNDING THE TELEPORTER BOX */
  for (i=0;i<2;i++)
   {  /* First, the 4 horizontals */
    dual_color_blit (35+(i*150),24,100,11,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[18]);
    dual_color_blit (35+(i*150),134,100,11,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[18]);
    /* now the upper verticals */
    dual_color_blit (34+(i*238),32,13,34,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[17]);
    /* now the lower verticals */
    dual_color_blit (34+(i*238),101,13,29,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[15]);
   }

    /* 2 more lower vertical overlappers to match size of upper ones */
    dual_color_blit (34,106,13,29,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[15]);
    dual_color_blit (272,108,13,29,WALLCOLOR,WALLCOLOR3,
		     screen_buffer,board_picts[15]);
}
/*************************************************************************/
void draw_tank_yard_board(void)
/* This is a junkyard full of tanks! */
{
  int i;

  /* Draw the lower left 2 rows of tanks */
  for (i=0;i<8;i++)
   {
      dual_color_blit (27+(i*22),120,SQRTANKPICT_WIDTH,SQRTANKPICT_HEIGHT,
		       LIGHTGRAY,WALLCOLOR3,screen_buffer,tank_picts[0]);

      dual_color_blit (27+(i*22),83,SQRTANKPICT_WIDTH,SQRTANKPICT_HEIGHT,
		       LIGHTGRAY,WALLCOLOR3,screen_buffer,tank_picts[4]);
   }

  /* Draw the upper left 2 rows of tanks */
  for (i=0;i<8;i++)
   {
      dual_color_blit (27+(i*22),66,SQRTANKPICT_WIDTH,SQRTANKPICT_HEIGHT,
		       LIGHTGRAY,WALLCOLOR3,screen_buffer,tank_picts[24]);

      dual_color_blit (27+(i*22),23,SQRTANKPICT_WIDTH,SQRTANKPICT_HEIGHT,
		       LIGHTGRAY,WALLCOLOR3,screen_buffer,tank_picts[28]);
   }

  /* draw lower right building */
    blit(230,105,60,40,screen_buffer,board_picts[0]);
  /* draw upper right building */
    blit(230,25,60,40,screen_buffer,board_picts[0]);
}
/*************************************************************************/
void draw_the_13th(void)
/* The 13th Level! */
{
   /* Draw the '1' in 13 */
  blit(65,30,LONGBLDGWIDTH,LONGBLDGHEIGHT,screen_buffer,board_picts[3]);
  blit(65,81,LONGBLDGWIDTH,LONGBLDGHEIGHT,screen_buffer,board_picts[3]);

  /* Draw the '3' in 13 */
   /* first the left most 3 blocks on the '3' */
  blit(171,29,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
  blit(174,70,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
  blit(171,110,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
  /* now the rightmost 2 blocks on the '3' before the vertical */
  blit(196,29,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
  blit(196,110,MIDBLDGWIDTH,MIDBLDGHEIGHT,screen_buffer,board_picts[1]);
  /* draw the right vertical part of the '3' */
  blit(225,30,LONGBLDGWIDTH,LONGBLDGHEIGHT,screen_buffer,board_picts[3]);
  blit(225,81,LONGBLDGWIDTH,LONGBLDGHEIGHT,screen_buffer,board_picts[3]);

  /* Draw the left vertical that runs along the left border */
    blit (1,5,20,77,screen_buffer,board_picts[19]);
    blit (1,99,20,77,screen_buffer,board_picts[19]);
  /* Draw the right vertical that runs along the left border */
    blit (299,5,20,77,screen_buffer,board_picts[19]);
    blit (299,99,20,77,screen_buffer,board_picts[19]);
}
/*************************************************************************/
#define _BOARDS
#endif