
/* Kirk Christiansen              */
/* DRAWTANK.C                     */

/* Functions for drawing the      */
/* tank Pieces in the Tank Game.  */
/* Tank Types:  1) Triangular     */
/*              2) Square         */
/*              3) Hunter Killer  */

#ifndef  _DRAWTANK
/*************************************************************************/
#include "kcgrphcs.c"
#include "tankvars.c"
/*************************************************************************/
/* Function Prototypes */
void drawtanks(void);
void drawtank(int tanknum, int clr);
void drawsqrtank (int tanknum, int clr);
void draw_hunter_killer (int tanknum, int clr);
void draw_guntank (int tanknum, int clr);
void draw_circletank(int tanknum,int clr);
void erasetank(int tanknum);
/*************************************************************************/
void drawtanks(void)
/* This function takes the entire tank array, and draws out each */
/* tank if it exists on the game grid.                           */
{
  int i;

    for (i=0;i<MAXTANKS;i++)
    {
      /* If tank is alive and visible */
      if ((tanks[i].armor > 0) &&
	  (tanks[i].tankstats[VISIBILITYSTATUS]==VISIBLE))
       {
	  drawtank(i,tanks[i].colors[0]);
       }
    }
}
/*************************************************************************/
void drawtank(int tanknum, int clr)
/*  This function takes in as input the array number of the tank.  */
/*  It then calls the appropiate tank drawing function according to */
/*  the tank type.                                                  */
{
 if (tanks[tanknum].tanktype==SQRTANK)
 {
  drawsqrtank(tanknum,clr);
 }
 else if (tanks[tanknum].tanktype==HUNTERKILLER)  /* terminator like */
 {
   draw_hunter_killer(tanknum,clr);
 }
 else if (tanks[tanknum].tanktype==GUNTANK) /* smart mode like */
 {
   draw_guntank(tanknum,clr);
 }
 else if (tanks[tanknum].tanktype==CIRCLETANK) /* wierd lookin */
 {
   draw_circletank(tanknum,clr);
 }
 else if (tanks[tanknum].tanktype==DUMMYTANK) /* decoy */
 {
   /* do nothing, don't draw it....cuz you will never see it */
 }
}
/*************************************************************************/
void drawsqrtank (int tanknum, int clr)
/* This function draws a tank of type SQUARE */
{
 int x,y,pos;

 x=tanks[tanknum].xcor;
 y=tanks[tanknum].ycor;
 pos=tanks[tanknum].pos;

    if (clr==GREEN)
     {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[pos-1]);
     }
    else if (clr==DARKGRAY) /* wrong, this is for red */
     {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[7+pos]);
     }
    else if (clr==RED)
     {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[15+pos]);
     }
    else if (clr==BLACK)   /* an erase is requested */
     {
       erase_blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
		 SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[pos-1]);
     }
 }
/*************************************************************************/
void draw_hunter_killer (int tanknum, int clr)
/* This function draws a hunter killer type of tank, which has  */
/* a triangular type body and a rectangular tred on each flank. */
{
 int x,y,pos;

 x=tanks[tanknum].xcor;
 y=tanks[tanknum].ycor;
 pos=tanks[tanknum].pos;

   if (clr==BLACK)
    {
     erase_blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	 SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[23+pos]);
    }
   else if (clr==OFFGRAY)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[23+pos]);
    }
   else if (clr==GREEN)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[31+pos]);
    }
   else if (clr==RED)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[39+pos]);
    }
}
/*************************************************************************/
void draw_guntank (int tanknum, int clr)
/* This function draws a gun tank, which has a small barrel, and large */
/* treds, and a cool looking muzzle. */
{
 int x,y,pos;

 x=tanks[tanknum].xcor;
 y=tanks[tanknum].ycor;
 pos=tanks[tanknum].pos;

   if (clr==BLACK)
    {
     erase_blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	 SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[47+pos]);
    }
   else if (clr==OFFGRAY)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[63+pos]);
    }
   else if (clr==GREEN)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[47+pos]);
    }
   else if (clr==RED)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[55+pos]);
    }
}
/*************************************************************************/
void draw_circletank (int tanknum, int clr)
/* This function draws the wierd looking Circle Floating Tank */
{
 int x,y;

 x=tanks[tanknum].xcor;
 y=tanks[tanknum].ycor;

   if (clr==BLACK)
    {
     erase_blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	 SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[72]);
    }
   else if (clr==CIRCLE_CLR)
    {
       blit(x-SQRTANK_X_OFFSET,y-SQRTANK_Y_OFFSET,SQRTANKPICT_WIDTH,
	    SQRTANKPICT_HEIGHT,screen_buffer,tank_picts[72]);
    }
}
/*************************************************************************/
void erasetank(int tanknum)
/* This function simply calls drawtank with a color which will */
/* erase the tank by drawing back over its current position.   */
{
 drawtank(tanknum,GROUNDCOLOR);
}
/*************************************************************************/
#define _DRAWTANK
#endif
