
/* Kirk Christiansen */
/* EFFECTS.C         */

/* Special Effects for */
/* computer games.     */

#ifndef _EFFECTS
#define _EFFECTS
/************************************************************************/
#include <stdlib.h>
#include "kcgrphcs.c"
/************************************************************************/
#define NUM_FADE_PATTERNS  6

enum fade_patterns {QUICK_FADE,CHUNK_FADE,VERT_FADE,HORIZ_FADE,HATCH_FADE,
		    CLOSE_FADE};
/************************************************************************/
/* Function Prototypes */
void select_random_fade (char far *screen_is);
void fade_screen (char far *screen_is);
void quick_fade (char far *screen_is);
void chunk_fade (char far *screen_is);
void vertical_sweep_fade (char far *screen_is);
void horizontal_sweep_fade(char far *screen_is);
void hatch_fade(char far *screen_is);
void close_fade(char far *screen_is);
void morph_fade (char far *screen_is, char far *next_image);
/************************************************************************/
void select_random_fade (char far *screen_is)
{
  randomize();

  switch (rand()%NUM_FADE_PATTERNS)
   {
     case (QUICK_FADE) :  quick_fade(screen_is);
			  break;
     case (CHUNK_FADE) :  chunk_fade(screen_is);
			  break;
     case (VERT_FADE)  :  vertical_sweep_fade(screen_is);
			  break;
     case (HORIZ_FADE)  : horizontal_sweep_fade(screen_is);
			  break;
     case (HATCH_FADE)  : hatch_fade(screen_is);
			  break;
     case (CLOSE_FADE)  : close_fade(screen_is);
			  break;
   };

   /* Make sure the screen is cleared */
   clear_screen(screen_is);
}
/************************************************************************/
void fade_screen (char far *screen_is)
{
 long i;

 randomize();

 for (i=0;i<300000; i++)
   {
     putpixel(rand()%SCREEN_WIDTH,rand()%SCREEN_HEIGHT,BLACK,screen_is);
   }
}
/************************************************************************/
void quick_fade (char far *screen_is)
{
 long i;

 randomize();

 for (i=0;i<150000; i++)
   {
     putpixel(rand()%SCREEN_WIDTH,rand()%SCREEN_HEIGHT,BLACK,screen_is);
   }
}
/************************************************************************/
void chunk_fade (char far *screen_is)
{
 int i,j;
 int x,y;

 randomize();

 for (i=0;i<3000;i++)
  {
   /* pick a random point */
   x=(rand()%(SCREEN_WIDTH-10));
   y=(rand()%(SCREEN_HEIGHT-10));

   /* draw 10 lines all 10 pixels long, to create a 10x10 square */
    for (j=0;j<10;j++)
     {
      line (x,y+j,x+10,y+j,BLACK,screen_is);
     }
  }
}
/************************************************************************/
void vertical_sweep_fade (char far *screen_is)
/* This fade is like a curtain closing....it goes from top of the */
/* screen down to the bottom. */
{
 int i;

 for (i=0;i<(SCREEN_HEIGHT);i++)
  {
    bar_blit (0,i,SCREEN_WIDTH,1,BLACK,screen_is);
    delay(3);
  }

  /* Turn off sound */
}
/************************************************************************/
void horizontal_sweep_fade (char far *screen_is)
/* This fade sweep across the screen.....not down, across  */
{
 int i;

 for (i=0;i<(SCREEN_WIDTH);i++)
  {
    line (i,0,i,(SCREEN_HEIGHT-1),BLACK,screen_is);
    delay(2);
  }
}
/************************************************************************/
void hatch_fade(char far *screen_is)
{
 int i;

 for (i=0;i<(SCREEN_HEIGHT);i+=3)
  {
    line (0,i,(SCREEN_WIDTH-1),i,BLACK,screen_is);
    delay(4);
  }
 for (i=0;i<(SCREEN_WIDTH);i+=3)
  {
    line (i,0,i,(SCREEN_HEIGHT-1),BLACK,screen_is);
    delay(4);
  }
 for (i=(SCREEN_HEIGHT-1);i>-1;i-=3)
  {
    line (0,i,(SCREEN_WIDTH-1),i,BLACK,screen_is);
    delay(4);
  }
 for (i=(SCREEN_WIDTH-1);i>-1;i-=3)
  {
    line (i,0,i,(SCREEN_HEIGHT-1),BLACK,screen_is);
    delay(4);
  }

  delay (25);
}
/************************************************************************/
void close_fade(char far *screen_is)
/* This fade pattern closes the board from both the left and right */
/* sides simultaneously.  Like in a trash compactor style.  */
{
  int x1=0;
  int x2=SCREEN_WIDTH-1;

   do
   {
     bar_blit(x1,0,1,200,BLACK,screen_is);
     bar_blit(x2,0,1,200,BLACK,screen_is);
     /* update the fade locations */
     x1++;
     x2--;
     delay(4);
   }
   while ((x1<159)&&(x2>158));
}
/************************************************************************/
void morph_fade (char far *screen_is, char far *next_image)
/* This fade pattern morphs a current screen into a new screen picture */
/* I came up with this algorithm 10/7/95.  It has a cool look to it.   */
/* I use a global *320 multiplication lookup table from kcgrphcs.c     */
/* I also access the global var cpu_speed to determine for loop length */
{
 long i,j;
 long num;  /* num of pixels to loop through */

 /* set num according to cpu speed */
 num = 80000 + (cpu_speed * 120000);  /* so either 80000 or 200000 */

 randomize();  /* make sure we get a random morph look */

 for (i=0;i<num;i++)
   {
     /* get a random pixel on the screen (ie 64k area) */
     j=mul320_tbl[(rand()%200)]+(rand()%320);
     /* put a pixel from the new screen onto the old screen */
     screen_is[j]=next_image[j];
   }
}
/************************************************************************/
#endif