
/* Kirk Christiansen   */
/* KCGRPHCS.C          */

/* My Graphics library for video mode 13h.  */
/* Functions I created for drawing,erasing, */
/* and manipulating pictures and images.    */
/*************************************************************************/
#ifndef _KCGRPHCS
/*************************************************************************/
#include <stdio.h>
#include <string.h>     /* for strlen() */
#include <dos.h>        /* for FP_SEG() */
#include <ctype.h>      /* for toupper() */
#include "tankvars.c"
/*************************************************************************/
#define CHARPICTWIDTH   7
#define CHARPICTHEIGHT  9
#define CLCHARPICTWIDTH   9
#define CLCHARPICTHEIGHT  9
#define SCREEN_WIDTH  320
#define SCREEN_HEIGHT 200
/*************************************************************************/
/* Function Prototypes */
void initialize_mul320_table (void);
void line(int x1,int y1,int x2,int y2,int color,char far *screenis);
void thickline(int x1,int y1,int x2,int y2,int color,char far *screenis);
void rect(int x1, int y1, int x2, int y2, int color, char far *screenis);
void solid_rect(int x1,int y1,int x2,int y2,int color,char far *screenis);
int  getpixel(int x, int y, char far *screenis);
void putpixel(int x, int y, int color, char far *screenis);
void out_text (char *stringis, int x, int y,int color,char far *screenis);
void move_image(int x1,int y1,int width,int height,
		unsigned char far *source_buffer,
		unsigned char far *destination_buffer);
void blit(int x1,int y1,int width,int height,unsigned char far *screenis,
	   unsigned char far *sprite);
void fullblit(int x1,int y1,int width,int height,unsigned char far *screenis,
	   unsigned char far *sprite);
void color_blit(int x1,int y1,int width,int height,int color,
	       unsigned char far *screenis,  unsigned char far *sprite);
void bar_blit(int x1,int y1,int width,int height,int color,
	       unsigned char far *screenis);
void dual_color_blit(int x1,int y1,int width,int height,int clr1,int clr2,
		 unsigned char far *screenis,  unsigned char far *sprite);
void erase_blit(int x1,int y1,int width,int height,
		unsigned char far *screenis,unsigned char far *sprite);
/*************************************************************************/
/* GLOBAL LOOKUP ARRAYS */
unsigned int mul320_tbl[200];  /* This table contains the pre-calculated */
		      /* results of multiplication of 320 up through 199. */
/*************************************************************************/
void initialize_mul320_table (void)
/* This function simply fills the look up multiplication table */
/* This function MUST be called before any other library functions */
{
 int i;

  for (i=0;i<200;i++)
    mul320_tbl[i] = i * 320;
}
/*************************************************************************/
void line(int x1,int y1,int x2,int y2,int color,char far *screenis)
/* This function uses Breshenham's Algorithm to draw a line  */
/* from (x1,y1) to (x2,y2) in 'color' on *screenis.          */
/* This method avoids CPU time-consuming division            */
{
  int i;
  int length;
  int y_unit,x_unit; /* Variables for amount of change in x and y */
  int offset;        /* Basically, a pointer of where to draw to */
  int xdiff,ydiff;
  int error_term;

  /* Set the offset to the first point to be drawn */
   offset=y1*SCREENWIDTH+x1;

   ydiff=y2-y1;   /* Calculate difference between y coordinates */
   xdiff=x2-x1;	   /* Calculate difference between x coordinates */
   error_term=0;   /* Initialize error term */


  if (ydiff<0)      /* If the line moves in the negative direction */
    {
     ydiff=-ydiff;         /* ...get absolute value of difference */
     y_unit=-SCREENWIDTH;  /* ...and set negative unit in y dimension */
    }
  else
     y_unit=SCREENWIDTH;   /* Else set positive unit in  y dimension */


  if (xdiff<0) 	 /* If the line moves in the negative direction */
    {
       xdiff=-xdiff;  /* ...get absolute value of difference */
       x_unit=-1;     /* ...and set negative unit in x dimension */
    }
  else
     x_unit=1;	  /* Else set positive unit in y dimension */


  if (xdiff>ydiff)    /* If difference is bigger in x dimension */
    {
       length=xdiff+1;	/* ...prepare to count off in x direction */

	for (i=0; i<length; i++)
	  {
	   screenis[offset]=color; /* Set the next pixel in line to color */
	   offset+=x_unit;	  /* Move offset to next pixel in x dir. */
	   error_term+=ydiff;

	    /* Check to see if move is required in y direction */
	     if (error_term>xdiff)   /* If so...  */
	       {
		 error_term-=xdiff;   /* ...reset error term */
		 offset+=y_unit;   /*..move offset to next pixel in y dir.*/
	       }
	  }
   }

  else   /* If difference is bigger in y dimension */
    {
     length=ydiff+1;  /* ...prepare to count off in y direction */

       for (i=0; i<length; i++)
	 {
	  screenis[offset]=color;  /* Set next pixel in the line to COLOR */
	  offset+=y_unit;   /* Move offset to next pixel in y direction */
	  error_term+=xdiff;

	   /* Check to see if move is required in x direction */
	      if (error_term>0)   /* If so...*/
		{
		 error_term-=ydiff;	/* ...reset error term */
		 offset+=x_unit;   /* move offset to next pixel in x dir.*/
		}
	  }
     }
}
/*************************************************************************/
void rect(int x1, int y1, int x2, int y2, int color, char far *screenis)
/* I created this function to draw a rectangle with an upper left hand */
/* coordinate at (x1,y1) and a lower right hand coordinate at (x2,y2). */
/* I could transfer this to straight pixel plotting instead of calling */
/* the function line 4 times.                                          */
{
 line (x1,y1,x2,y1,color,screenis);
 line (x2,y1,x2,y2,color,screenis);
 line (x2,y2,x1,y2,color,screenis);
 line (x1,y2,x1,y1,color,screenis);
}
/*************************************************************************/
void solid_rect(int x1,int y1,int x2,int y2,int color,char far *screenis)
/* Same as function rect, except it fills in the rectangle completely */
{
 int width;
 int height;

 if (x2 >= x1)
   width = x2-x1;
 else
   width = x1-x2;

 if (y2 >= y1)
   height = y2-y1;
 else
   height = y1-y2;

 /* Now draw that rect :) */
 bar_blit (x1,y1,width,height,color,screenis);
}
/*************************************************************************/
void thickline(int x1,int y1,int x2,int y2,int color,char far *screenis)
/* This function simply draw a line 3 pixels wide. NOTE: only works */
/* for horizontal and vertical lines!!!                             */
{
 /* Draw original line */
 line(x1,y1,x2,y2,color,screenis);

 /* Draw lines on each side of original to make it width 3 */
 if (x1-x2)  /* x1 - x2 is not 0, so its a horizontal line */
   {
    line(x1,y1-1,x2,y2-1,color,screenis);
    line(x1,y1+1,x2,y2+1,color,screenis);
   }
 else /* its a vertical line, so adjust x coordinates only */
   {
    line(x1-1,y1,x2-1,y2,color,screenis);
    line(x1+1,y1,x2+1,y2,color,screenis);
   }
}
/*************************************************************************/
int getpixel(int x, int y, char far *screenis)
/* This function returns the color value of the pixel at (x,y) */
{
 return screenis[(mul320_tbl[y]+x)];
}
/*************************************************************************/
void putpixel(int x, int y, int color, char far *screenis)
/* This function simply plots a pixel of 'color' at (x,y) */
{
 screenis[(mul320_tbl[y]+x)]=color;
}
/*************************************************************************/
void out_text (char *stringis,int x, int y,int color, char far *screenis)
/* This function uses the input PCX file of character pictures to  */
/* display a string to the screen at (x,y) with the inputed color. */
/* NOTE: this function needs adjusted if I change my character     */
/*  picture file.                                                  */
{
 int i;
 char c;

 for (i=0;i<strlen(stringis);i++)
  {
    c=(toupper(stringis[i]));

    if ((c>=65) && (c<=90))
      c-=55;    /* This will align characters A-Z with my pictures */
    else if ((c>=48) && (c<=57))
      c-=48;    /* This will align character 0-9 with my pictures */
    else
      c=-1; /* -1 is flag for bad character */

    /* Check for legal character */
     if (c!=-1)
      {
       color_blit(x+(i*CHARPICTWIDTH),y,CHARPICTWIDTH,CHARPICTHEIGHT,color,
		 screenis,character_picts[c]);
      }
  }
}
/*************************************************************************/
void move_image(int x1,int y1,int width,int height,
		unsigned char far *source_buffer,
		unsigned char far *destination_buffer)
/* This function takes in as input an (x,y) coordinate of an image */
/* from a source_buffer, and writes it to a destination_buffer by  */
/* copying each pixel and using the image's width and height as    */
/* boundaries of when to stop copying. NOTE: This function assumes */
/* that the destination_buffer is >= the source_buffer!            */
{
  int x,y;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      destination_buffer[y*width+x]=source_buffer[mul320_tbl[y1+y]+x1+x];
}
/************************************************************************/
void blit(int x1,int y1,int width,int height,unsigned char far *screenis,
	   unsigned char far *sprite)
/* Another appropiate name for this function would be put_sprite, because */
/* it takes a sprite (image) with width & height and places it at (x1,y1) */
/* to *screenis, while checking for transparent (BLACK) pixels.           */
{
  int x,y;
  unsigned char byte;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	byte=sprite[y*width+x];

	 if (byte)  /* if not transparent, put it on the screen */
	    screenis[mul320_tbl[y1+y]+x1+x]=byte;
      }
}
/************************************************************************/
void fullblit(int x1,int y1,int width,int height,unsigned char far *screenis,
	   unsigned char far *sprite)
/* same as blit...except it blits black too */
{
  int x,y;
  unsigned char byte;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	screenis[mul320_tbl[y1+y]+x1+x]=sprite[y*width+x];
      }
}
/************************************************************************/
void color_blit(int x1,int y1,int width,int height,int color,
	       unsigned char far *screenis,  unsigned char far *sprite)
/* Same function as blit except you pass in a color you want the sprite */
/* to be. So you can draw the same picture in different colors.         */
{
  int x,y;
  unsigned char byte;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	byte=sprite[y*width+x];
	 if (byte)
	    screenis[mul320_tbl[y1+y]+x1+x]=color;
      }
}
/************************************************************************/
void bar_blit(int x1,int y1,int width,int height,int color,
	       unsigned char far *screenis)
/* This function blits a color bar (thick bar) on the screen */
/* Good for drawing solid rectangles. */
{
  int x,y;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	screenis[mul320_tbl[y1+y]+x1+x]=color;
      }
}
/************************************************************************/
void dual_color_blit(int x1,int y1,int width,int height,int clr1,int clr2,
		 unsigned char far *screenis,  unsigned char far *sprite)
/* Same function as blit except you pass in a color you want the sprite */
/* to be. So you can draw the same picture in different colors.         */
{
  int x,y;
  int first_clr=0;
  int second_clr=0;
  unsigned char byte;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	byte=sprite[y*width+x];

	 if (byte)
	    {
	      if (!first_clr)
		 first_clr=byte;
	      else if ((!second_clr) && (byte!=first_clr))
		 second_clr=byte;

	      if (byte==first_clr)
		screenis[mul320_tbl[y1+y]+x1+x]=clr1;
	      else if (byte==second_clr)
		screenis[mul320_tbl[y1+y]+x1+x]=clr2;
	     }
      }
}
/************************************************************************/
void erase_blit(int x1,int y1,int width,int height,
		unsigned char far *screenis,unsigned char far *sprite)
/* Erases a sprite that was previously put at (x1,y1).   */
{
  int x,y;
  unsigned char byte;

  for (y=0; y<height; y++)
    for (x=0; x<width; x++)
      {
	byte=sprite[y*width+x];

	 if (byte)
	    screenis[mul320_tbl[y1+y]+x1+x]=0; /* 0 is BLACK */
      }
}
/************************************************************************/
#define _KCGRPHCS
#endif
