
; Kirk Christiansen
; KCTNKASM.ASM

; Assembly language functions for
; tankgame, maze programs, and other games that
; run in video mode 13h.

;========================================================================
.MODEL large
; why not large???, I get fixup overflows on cmmnd line

.CODE

; Function Prototypes
PUBLIC _clear_screen, _set_mode, _get_current_mode, _putwindow
PUBLIC _setpalette,  _wait_for_vert_retrace;
;========================================================================
SCREEN_WIDTH   EQU   320      ; Screen width in pixels in mode 13h
SCREEN_HEIGHT  EQU   200      ; Screen height in pixels in mode 13h
;========================================================================
; clear_screen (char far *screenis)
; Function for clearing video memory or offscreen buffer at
; screenis address to all zeroes (for Black).

_clear_screen       PROC
		    ARG  screenis:DWORD   ;4bytes 2 for seg, 2 for offset
		    push bp          ; save base pointer
		    mov  bp, sp      ; sets up stack pointer
		    push di          ; save di
		    les  di, screenis  ;point ES:DI at screenis
		    mov  cx, SCREEN_WIDTH/2 * SCREEN_HEIGHT  ;32,000
		    mov  ax,0       ; 0 for black
		    rep  stosw      ; Takes value in ax and repeats cx times
		    pop  di         ;   where es:di is the destination
		    pop  bp
		    ret
_clear_screen       ENDP             ;end procedure
;========================================================================
; setmode(int mode)
; This function sets the VGA adapter to BIOS mode 'modeis'

_set_mode     PROC
	       ARG  modeis:WORD    ;2 bytes
	       push bp           ;save base pointer
	       mov  bp,sp        ;set up stack pointer
	       mov  ax,modeis    ;AX = video function (like 13h)
	       mov  ah,0         ;function number (0=set video mode)
	       int  10h          ;invoke the interupt
	       pop  bp
	       ret
_set_mode     ENDP               ;end of procedure
;========================================================================
;get_current_mode(void)
;This function gets the current video mode via interupt 10h/function 0fh
;It returns the current mode in the AL register.  So in order to retrive
;this value you must set a char variable in calling fnct to this fnct.

_get_current_mode    PROC
		     push bp        ;save bp
		     mov  bp,sp     ;set up stack pointer
		     mov  ah,0fh    ;function number for get current mode
		     int  10h       ;invoke the interupt
		     pop  bp
		     ret
_get_current_mode    ENDP           ;end of procedure
;========================================================================
;putwindow(x,y,wdth_is,hght_is,offset,segment)
; Move rectangular area of screen buffer at offset, segment
; with upper left corner at (x,y) by using the area's width(wdth_is) and
; height(hght_is). This transfer is to the screen, which has address 0a000h.
; The heart of this function is the use of the REP MOVSW instruction,
; MOVSW will move words of information from a location addressed at DS:SI
; to a location at ES:DI. This is similiar to a lightning quick For Loop
; transfer from one array to another in RAM. NOTE: if the direction flag
; is zero, then MOVSW works the data transfer from left to right.

_putwindow  PROC

      ARG x:WORD,y:WORD,wdth_is:WORD,hght_is:WORD,buf_off:WORD,buf_seg:WORD
      push  bp
      mov   bp,sp
      push  ds
      push  di
      push  si
      mov   ax,y         ;THESE NEXT LINES CALCULATE WHERE WINDOW BEGINS
      mov   dx,320       ;put SCREENWIDTH in dx, and multiply by Y and
      mul   dx           ;add X to result to get proper start of window.
      add   ax,x         ;then move the proper offset into di.
      mov   di,ax
      add   ax,buf_off
      mov   si,ax
      mov   dx,0a000h	   ; Get screen segment in ES
      mov   es,dx
      mov   dx,buf_seg     ; Get screen buffer segment in DS
      mov   ds,dx
      mov   dx,hght_is     ; Get line count into DX
      cld                  ; clear direction flag
ploop1:
      mov   cx,wdth_is     ; Get pixel width of window into CX
      shr   cx,1           ; shift cx right 1 bit
      push  di             ; Save screen and buffer addresses
      push  si
      rep   movsw          ; Move one line of window to screen
      pop   si             ; Restore screen and buffer addresses
      pop   di
      add   si,320         ; ...and advance them to next line
      add   di,320
      dec   dx             ; Count off one line
      jnz   ploop1         ; If more lines in window,
			   ;   loop back and draw them
      pop   si             ; Restore all registers etc...
      pop   di
      pop   ds
      pop   bp
      ret
_putwindow	ENDP
;========================================================================
; setpalette(char far *color_regs,int firstreg,int numregs)
;   Set VGA color registers, beginning with FIRSTREG and
;	  continuing for NUMREGS to the color values in COLOR_REGS

_setpalette	PROC

    ARG   regs:DWORD
    push  bp              ; Save BP
    mov   bp,sp           ; Set up stack pointer
    les   dx,regs         ; Point ES:SX at palette registers
    mov   ah,10h          ; Specify BIOS function 10h
    mov   al,12h          ; ...subfunction 12h
    mov   bx,0            ; Start with first register
    mov   cx,100h         ; Set all 256 (100h) registers
    int   10h             ; Call video BIOS
    pop   bp              ; Restore BP
    ret
_setpalette	ENDP
;========================================================================
; wait_for_vert_retrace(void)
; I got this function from some kid off the net, and he told me it
; delays the computer a little bit but gets rid of the screen glitches
; that occur during the vertical retrace.  It worked excellent for
; Tankgame and got rid of the jitters on moving tanks and projectiles
; The kids name was Prasanth and he told me you could get rid of the
; small delay by using triple buffering.

_wait_for_vert_retrace     PROC
			   push bp           ;save base pointer
			   mov  bp,sp        ;set up stack pointer

		       ; retrace code starts here
		       ; I have no idea what its doing :)

			   mov  dx,3DAh
		    loop1:
			   in   al,dx
			   and  al,08h
			   jnz  loop1
		    loop2:
			   in   al,dx
			   and  al,08h
			   jz   loop2

			   pop  bp
			   ret
_wait_for_vert_retrace     ENDP               ;end of procedure
;========================================================================
END