
/* Kirk Christiansen */
/* KC_KEYBD.C        */

/* This is the module that contains the */
/* functions and code for my keyboard   */
/* handler which will replace the BIOS  */
/* Int 9 handler.                       */

#ifndef _KC_KEYBD  /* Insulator */
#define _KC_KEYBD
/***********************************************************************/
#include <stdio.h>
#include <dos.h>
/***********************************************************************/
/* Keyboard Constants */
#define NUM_SCANCODES   128    /* Maximum number of scancodes allowed */

#define TRUE  1
#define FALSE 0

/* Keyboard scan codes in hex form */
#define ESC        0x01
#define ENTER      0x1C
#define SPACE      0x39
#define BACKSPACE  0x0E
#define TILDE      0x29
#define PAUSE      0x1D
#define PAGEUP     0x49
#define PAGEDOWN   0x51
#define PLUS       0x0D
#define MINUS      0x0C
#define SLASH      0x2B
#define BACKSLASH  0x35

#define LEFTARROW  0x4B
#define RIGHTARROW 0x4D
#define UPARROW    0x48
#define DOWNARROW  0x50
#define CTRL       0x1D
#define ALT        0x38
#define TAB        0x0F
#define CAPSLOCK   0x3A
#define LEFTSHIFT  0x2A

#define F1         0x3B
#define F2         0x3C
#define F3         0x3D
#define F4         0x3E
#define F5         0x3F
#define F6         0x40
#define F7         0x41
#define F8         0x42
#define F9         0x43
#define F10        0x44
#define F11        0x85
#define F12        0x86

/* Number keys */
#define KEY_1      0x02
#define KEY_2      0x03
#define KEY_3      0x04
#define KEY_4      0x05
#define KEY_5      0x06
#define KEY_6      0x07
#define KEY_7      0x08
#define KEY_8      0x09
#define KEY_9      0x0A
#define KEY_0      0x0B
/* Alphabet keys */
#define KEY_A      0x1E
#define KEY_B      0x30
#define KEY_C      0x2E
#define KEY_D      0x20
#define KEY_E      0x12
#define KEY_F      0x21
#define KEY_G      0x22
#define KEY_H      0x23
#define KEY_I      0x17
#define KEY_J      0x24
#define KEY_K      0x25
#define KEY_L      0x26
#define KEY_M      0x32
#define KEY_N      0x31
#define KEY_O      0x18
#define KEY_P      0x19
#define KEY_Q      0x10
#define KEY_R      0x13
#define KEY_S      0x1F
#define KEY_T      0x14
#define KEY_U      0x16
#define KEY_V      0x2F
#define KEY_W      0x11
#define KEY_X      0x2D
#define KEY_Y      0x15
#define KEY_Z      0x2C
/***********************************************************************/
/* Module Globals */
void far interrupt (*old_keyboard_handler)();
int keys_pressed_array[NUM_SCANCODES]={0};
int keys_were_pressed_array[NUM_SCANCODES]={0};
/***********************************************************************/
/* Function Prototypes */
void far interrupt my_keyboard_handler(void);
void remove_old_keyboard_handler(void);
void restore_old_keyboard_handler(void);
int get_key_state (int scancode);
int get_and_reset_key_state (int scancode);
int get_current_key_state (int scancode);
int keypress(void);
void reset_keys_array(void);
/***********************************************************************/
void far interrupt my_keyboard_handler(void)
/* This interrupt handler function contains the code to implement */
/* my own keyboard handler.  */
{
  int scan_code;
  unsigned char control_port;


  /* First, read in scan code */
  scan_code = inportb(0x60);


     /* NOTE : this next section of code is not necessary if */
     /* this program is running on AT machines.  But if the  */
     /* program is running on an XT machine port 61h is read */
     /* to determine if a scan code has been received.       */
	/* Read in control port */
	control_port = inportb(0x61);
       /* Toggle 7th bit to signal a scan code has been read in */
	outportb(0x61, control_port | 0x80);
	outportb(0x61, control_port);


  /* If key is released then */
  if (scan_code & 0x80)        /* Check the 7th bit for a 1 (release) */
   {
     scan_code &= 0x7f;               /* Toggle the 7th (last) bit off */
     keys_pressed_array[scan_code]=FALSE;  /* Set it to FALSE */
   }

  /* Else, if key is currently being pressed */
  else
   {
    keys_pressed_array[scan_code]=TRUE;     /* Set the appropiate cells */
    keys_were_pressed_array[scan_code]=TRUE;
   }

  /* Inform the PIC that there has been an EOI */
  outportb(0x20, 0x20);
}
/***********************************************************************/
void remove_old_keyboard_handler(void)
/* This function simply removes the default keyboard handler and */
/* installs mine in its place.  It does this by changing vectors */
{
  /* First, get the vector for the default_handler */
  old_keyboard_handler = getvect(9);

  /* Now, put my keyboard handler into its place */
  setvect(9, my_keyboard_handler);
}
/***********************************************************************/
void restore_old_keyboard_handler(void)
/* This function just re-installs the default keyboard handler */
{
 setvect (9, old_keyboard_handler);
}
/***********************************************************************/
int get_key_state (int scancode)
/* Returns TRUE if the key (scancode) has been pressed or is */
/* currently being pressed.                                  */
{
 int result;

  /* Check to see if a keypress is happening or has happened */
  result =
     (keys_pressed_array[scancode] || keys_were_pressed_array[scancode]);

  /* Reset and old keypress once its been checked for */
  keys_were_pressed_array[scancode]=0;

  /* Return the result */
  return result;
}
/**************************************************************************/
int get_and_reset_key_state (int scancode)
/* Returns TRUE if the key (scancode) has been pressed or is  */
/* currently being pressed, and if it has or is it is reset.  */
{
 int result;

  /* Check to see if a keypress is happening or has happened */
  result =
     (keys_pressed_array[scancode] || keys_were_pressed_array[scancode]);

  /* Reset the old keypress once its been checked for */
  keys_were_pressed_array[scancode]=0;
  keys_pressed_array[scancode]=0;

  /* Return the result */
  return result;
}
/**************************************************************************/

int get_current_key_state (int scancode)
/* Returns TRUE if a key is currently being held down. */
{
  return   keys_pressed_array[scancode];
}
/**************************************************************************/
int keypress(void)
/* Returns TRUE if any key is currently being held down. */
{
  int i;

   for (i=0;i<NUM_SCANCODES;i++)
    {
     if (keys_pressed_array[i])
	 return TRUE;
    }

 return FALSE;
}
/**************************************************************************/
void reset_keys_array(void)
/* This function just resets both key_press arrays.  This is usually */
/* called after a keypress() function call, just to reset everything. */
{
  int i;

    for (i=0;i<NUM_SCANCODES;i++)
     {
       keys_pressed_array[i]=FALSE;
       keys_were_pressed_array[i]=FALSE;
     }
}
/**************************************************************************/
#endif
