
/* Kirk Christiansen */
/* MISCFNCS.C        */

#ifndef _MISCFNCS
/*************************************************************************/
#include <time.h>
#include <conio.h>
#include "tankvars.c"
#include "kc_keybd.c"
#include "kcgrphcs.c"
#include "ser_lib.c"
#include "tnkpicts.c"
/*************************************************************************/
/* Function Prototypes */
void get_computer_speed(void);
void set_up_tank_game (void);
void initialize_player_tank(void);
void initialize_second_player_tank(void);
void reset_tanks(int clear_what_tanks);
void reset_tank(int tanknum);
void set_explosion(int x, int y, int type);
void reset_explosions(void);
void draw_explosions(void);
void draw_blownuptank(int tank_x, int tank_y, int color);
void do_pause_routine(void);
int  establish_link (void);
void do_quit_game_sequence(int abort_program);
/*************************************************************************/
void get_computer_speed (void)
/* This function does a quick for loop to check the CPU's speed of */
/* the current computer that the game is running on.               */
/* This function sets the global variable cpu_speed.               */
{
 long unsigned i;
 unsigned long test_time;

 if (report_status)
  {
   printf ("CPU speed test");
   delay(100);printf(".");delay(100);printf(" .");delay(100);printf(" .\n");
  }
 /* save start time (using global variable current_time) */
 current_time = clock();

 for (i=0;i<1000000;i++)
  {
   /* do nothing */
  }

 /* Get the time of the test */
 test_time = clock() - current_time;

 /* Convert it to a enumerated type */
 if (test_time <= 5)
    cpu_speed = FASTSPEED;
 else /* test_time was greater than 5 ticks */
    cpu_speed = SLOWSPEED;

 if (report_status)
  {
   printf("  Test time was : %lu\n",test_time);
   printf("  CPU rating is : %d ",cpu_speed);
   /* print out the mnemonic of the speed */
   if (cpu_speed==FASTSPEED)
     printf ("(FAST)\n");
   else
     printf ("(AVG)\n"); /* say avg, so we don't know if its slow or avg */
  }
 /* small delay */
  delay (200);
}
/*************************************************************************/
void set_up_tank_game (void)
/* This function is called before the game enters into the main loop. */
/* It sets up TANKGAME according to the gametype the user selected.   */
{
 /* If user wants to play complete game mode, setup the game */
   if (gametype==FULLGAME)
    {
     initialize_player_tank();
     level=1;
    }

   else if (gametype==TWOPLYR_FULLGAME)
    {
     initialize_player_tank();
     initialize_second_player_tank();
     level=1;
    }

   else if (gametype==MATCH)  /* dual between two humans */
    {
     level=0; /* this shouldn't be used */
     initialize_player_tank();
     initialize_second_player_tank();
    }

   else if (gametype==NULL_MODEM_MATCH)
    {
     /* Initialize current communcations port / install com port ISR */
     initialize_com_port (current_com_port,BAUD_9600,
			  PARITY_NONE | BITS_8 | STOP_1);

     /* Try to establish link */
      if (establish_link()==1) /* link didn't work :( */
       {
	 /* release pcx file memory */
	 release_tankgame_sprites();
	 /* If error during linking */
	 set_mode(oldmode);
	 printf ("\n ERROR trying to establish link with remote pc! \n");
	 sound (300); delay (200); nosound();
	 /* restore com port / isr */
	 close_com_port(current_com_port);
	 /* Restore old keyboard handler */
	 restore_old_keyboard_handler();
	 exit(1);
       }
      else /* Link was successful */
       {
	/* initialize the game */
	 initialize_player_tank();
	 initialize_second_player_tank();
	 boardnumber=CITY;  /* use CITY BOARD for default */
       }
    }
}
/*************************************************************************/
void initialize_player_tank(void)
/* This function simply resets a human player's tank stats   */
{
  tanks[0].playertype  = HUMAN;
  if (tanks[0].tanktype==NOTANK) /* player did not select a tank */
     tanks[0].tanktype    = PLAYERTANK; /* set default type */
  tanks[0].tanknumber = 0;
  tanks[0].tankside = 1; /* Enemy computer tanks are on side 0 */
  tanks[0].tankstats[GAMESTATUS]  = PLAYING;
  if (difficulty_is == EASYDIFF)
   {
    tanks[0].armor       = STARTARMOR + 10;
    tanks[0].previousarmor = STARTARMOR + 10;
   }
  else /* hard or average */
   {
    tanks[0].armor       = STARTARMOR;
    tanks[0].previousarmor = STARTARMOR;
   }
  tanks[0].colors[0]  = PLAYERCOLOR;
  tanks[0].colors[1]  = PLAYERCOLOR_2;
  tanks[0].projcolor   = PLAYERPROJCOLOR;

   if ((gametype==NULL_MODEM_MATCH) && (current_local_pc_is==RECEIVER))
    {
     tanks[0].colors[0]  = PLAYERCOLOR2;
     tanks[0].colors[1]  = PLAYERCOLOR2_2;
     tanks[0].projcolor   = COMPUTERPROJCOLOR;
    }

  tanks[0].current_weapon = NORMAL_PROJ;
  tanks[0].attackmode  = NOATTACK;  /* None because player controls tank */
}
/*************************************************************************/
void initialize_second_player_tank(void)
/* This function simply resets a second human player's tank stats  */
/* Used for Null Modem Matches  or MATCHES                         */
{
  tanks[1].playertype  = HUMAN;

  if (tanks[1].tanktype==NOTANK) /* player did not select a tank */
     tanks[1].tanktype = PLAYERTANK; /* set default type */

  tanks[1].tanknumber = 1;

   if (gametype==TWOPLYR_FULLGAME)
      tanks[1].tankside    = 1;      /* Put tank on the same side as*/
   else                              /* player 1. */
     tanks[1].tankside    = 0;   /* Player 1 is on side 1 */

  tanks[1].tankstats[GAMESTATUS]  = PLAYING;
  if (difficulty_is == EASYDIFF)
   {
    tanks[1].armor       = STARTARMOR + 10;
    tanks[1].previousarmor = STARTARMOR + 10;
   }
  else /* hard or average */
   {
    tanks[1].armor       = STARTARMOR;
    tanks[1].previousarmor = STARTARMOR;
   }
  tanks[1].colors[0]  = PLAYERCOLOR2;
  tanks[1].colors[1]  = PLAYERCOLOR2_2;
   if (gametype==MATCH)
     tanks[1].projcolor  = COMPUTERPROJCOLOR;
   else
     tanks[1].projcolor  = PLAYERPROJCOLOR2;

   if ((gametype==NULL_MODEM_MATCH) && (current_local_pc_is==RECEIVER))
    {
     tanks[1].colors[0]  = PLAYERCOLOR;
     tanks[1].colors[1]  = PLAYERCOLOR_2;
     tanks[1].projcolor   = PLAYERPROJCOLOR;
    }

  tanks[1].current_weapon = NORMAL_PROJ;
  tanks[1].attackmode  = NOATTACK;  /* None because player controls tank */
}
/**************************************************************************/
void reset_tanks(int clear_what_tanks)
/* This function initializes the tank stats before the game starts */
/* The variable clear_what_tanks stores either a 1 or 0.  If it is */
/* 0 (or the enumtype ALLTANKS), then all the tanks in the tank    */
/* array will be reset. Otherwise, just the computer tanks from    */
/* subscript 2 and up.                                             */
{
 int i,x,y;

 for (i=clear_what_tanks;i<MAXTANKS;i++)
  {
    reset_tank (i);
  }
}
/***************************************************************************/
void reset_tank(int tanknum)
/* This function resets a tanks attributes. This function is usually */
/* called if a tank has been killed, or a new level has started.     */
{
 int x,y;

     tanks[tanknum].xcor      = 0;
     tanks[tanknum].ycor      = 0;
     tanks[tanknum].pos       = 0;
     tanks[tanknum].cruise    =  NOCRUISE;
     tanks[tanknum].playertype = NOONE;
     tanks[tanknum].tanktype   = NOTANK;
     tanks[tanknum].tanknumber = 0;
     tanks[tanknum].tankside   = 0;

     /* Set the tankstats array */
     tanks[tanknum].tankstats[GAMESTATUS] = NOTPLAYING;
     tanks[tanknum].tankstats[VISIBILITYSTATUS] = VISIBLE;

     tanks[tanknum].previousarmor = 0;
     tanks[tanknum].armor         = 0;
     tanks[tanknum].colors[0]     = 0;
     tanks[tanknum].colors[1]     = 0;
     tanks[tanknum].projcolor     = 0;
     tanks[tanknum].current_weapon = NORMAL_PROJ;
     tanks[tanknum].attackmode    = NOATTACK;
     tanks[tanknum].currentinput  = 0;
     tanks[tanknum].lastinput     = 0;
     tanks[tanknum].thrustcount   = 0;
     tanks[tanknum].previous_ammo = 0;

 /* Reset the ammo array */
 for (x=0;x<MAXWEAPONS;x++)
  {
   tanks[tanknum].weapon_ammo[x]=0;
  }

 /* Reset the projectile arrays to 0 */
   for (x=0;x<MAXPROJS;x++)
      {
	 tanks[tanknum].projs[x].dir=0;
	 tanks[tanknum].projs[x].x=0;
	 tanks[tanknum].projs[x].y=0;
	 tanks[tanknum].projs[x].type=0;
	 tanks[tanknum].projs[x].time=0;
      }

  /* Whipe out the input buffer */
  for (x=0;x<MAXBUFFERSIZE;x++)
     tanks[tanknum].inputbuffer[x] = 0;
}
/*************************************************************************/
void reset_explosions(void)
/* Simply resets the whole structure, called before a new game. */
{
 int i;

   for (i=0;i<MAX_EXPLOSIONS;i++)
      {
       explosion_array[i].x=0;
       explosion_array[i].y=0;
       explosion_array[i].type=0;
       explosion_array[i].time=0;
      }
}
/*************************************************************************/
void set_explosion(int x, int y, int type)
{
 int i;

   for (i=0;i<MAX_EXPLOSIONS;i++)
      {
	/* If there is an empty space to set an explosion */
       if(!(explosion_array[i].x))
	 {
	  explosion_array[i].x=x;
	  explosion_array[i].y=y;
	  explosion_array[i].type=type;
	  explosion_array[i].time=current_time;
	  return;
	 }
      }
}
/*************************************************************************/
void draw_explosions(void)
/* Checks the global explosion array to see if explosions exist. If */
/* they do, then the function draws them into the screen_buffer.    */
{
 int i;

 for (i=0;i<MAX_EXPLOSIONS;i++)
  {
    if (explosion_array[i].x) /* if and x coordinate exists, then */
      {                        /* an explosion must exist.         */
       switch (explosion_array[i].type)
	{
	 case (PROJ_EXPLOSION) :
		      if ((current_time - explosion_array[i].time)
			   < PROJ_EXPLOSION_EXIST_TIME)
			 {
			     /* draw it using the x & y coordinate */
			 blit(explosion_array[i].x-4,explosion_array[i].y-4,
			 /*wd*/7,/*hght*/7,screen_buffer,misc_picts[0]);
			 }
		      else /* NO TIME LEFT */
			 {
			  /* erase it using the x & y coordinate */
			  color_blit (explosion_array[i].x-4,
				      explosion_array[i].y-4,/*width*/7,
			       /*hght*/7,BLACK,screen_buffer,misc_picts[0]);
			/* RESET IT */
			explosion_array[i].x=0;explosion_array[i].y=0;
			explosion_array[i].type=0;explosion_array[i].time=0;
			 }
		      break;
	 case (TANK_EXPLOSION) :
		      if ((current_time - explosion_array[i].time)
			   < TANK_EXPLOSION_EXIST_TIME)
			{
			 /* draw it using the x & y coordinate */
			 blit(explosion_array[i].x-7,explosion_array[i].y-7,
			 /*wd*/15,/*hght*/13,screen_buffer,misc_picts[1]);
			}
		      else  /* NO TIME LEFT */
			{
			/* erase it using the x & y coordinate */
			 color_blit (explosion_array[i].x-7,
				     explosion_array[i].y-7,/*wid*/15,
			      /*hght*/13,BLACK,screen_buffer,misc_picts[1]);
			/* RESET IT */
			explosion_array[i].x=0;explosion_array[i].y=0;
			explosion_array[i].type=0;explosion_array[i].time=0;
			}
		      break;
	} /* end switch (explosion_array[i].type) */
      } /* end if an explosion exists */
  } /* end for (i=0;i<MAX_EXPLOSIONS;i++) */
}
/**************************************************************************/
void draw_blownuptank(int tank_x, int tank_y, int color)
/* Draws a bunch of junk looking pieces to show tank is destroyed */
{
 int i;
 int left_side = tank_x - (TS+5);
 int top_side  = tank_y - (TS+5);

 for (i=0;i<20;i++)
  {
    putpixel(left_side + rand() % 20, top_side + rand() % 20,
	     color, screen_buffer);
  }
 /* For this function, I need to added a blow-up tank picture and    */
 /* put it on the screen after a tank has been destroyed.            */
 /* Like blown up tank pieces scattered about the tanks coordinates. */

}
/***************************************************************************/
void do_pause_routine(void)
/* This function is invoked when the user hits the pause button */
/* and the game is not already paused.   It must save the states */
/* of the current projectiles..etc so their times don't expire */
{
 int i,j;

     /* update time */
      current_time = clock();

       /* First save all the projs REMAINING AMOUNT OF times */
       /* Then store that time back into the proj structure */
       for (i=0;i<MAXTANKS;i++)
	 {
	  for (j=0;j<MAXPROJS;j++)
	   {
	     if (tanks[i].projs[j].dir) /* if a proj exists in that cell */
	      tanks[i].projs[j].time =
		PROJ_EXIST_TIME-(current_time-tanks[i].projs[j].time);
	   }
	 }


       /* NOW WAIT FOR THE KEYPRESS */
	do
	{
	   current_time=clock();
	}
	while (!keypress());

	reset_keys_array();

     /* update time */
      current_time = clock();


       /* Now take out those remaining times we saved before in the */
       /* proj structure and recalculate the times those projs would */
       /* have been fired based on the new current_time. */
       for (i=0;i<MAXTANKS;i++)
	 {
	  for (j=0;j<MAXPROJS;j++)
	   {
	     if (tanks[i].projs[j].dir) /* if a proj exists in that cell */
	      tanks[i].projs[j].time =
		  current_time - (PROJ_EXIST_TIME - tanks[i].projs[j].time);
	   }
	 }
}
/*************************************************************************/
int  establish_link (void)
/* This function tries to establish a link with the remote PC so */
/* that a modem game can be played.                              */
{
  int in_char;

  /* Display some instructions to user */
  out_text("NOTE  RECEIVER SHOULD INITIATE LINKUP",20,30,YELLOW,screen);
  putpixel(50,32,YELLOW,screen);
  putpixel(50,36,YELLOW,screen);


  if (current_local_pc_is == SENDER)
     out_text("Trying to contact receiver",50,60,YELLOW,screen);
  else
     out_text("Waiting for SENDER to call",50,60,YELLOW,screen);

  /* First test if local PC is Sender or the Receiver */
  if (current_local_pc_is == SENDER)
   {
     do
      {
	if (soundon)
	 {sound(500);delay(1);nosound();}
	delay(3);
	/* Send a character to remote telling it your the Sender */
	send_a_character('S');  /* send the ascii character 'S' */
	/* Try to receive a 'R' character (Receiver signal) from remote */
	in_char=get_character_from_in_buffer();


	if (in_char=='R')
	  {
	   out_text(" LINK ESTABLISHED ",100,120,YELLOW,screen);
	   if (soundon)
	    {sound (500); delay(300); nosound();}
	   delay(1000);
	  };
      }
     while ((in_char != 'R') && (!keypress()));

     /* Test to see if user wanted to abort */
     if (keypress())
      {
	/* Reset the keys array because of keypress */
	reset_keys_array();
	out_text(" GAME ABORTED ON KEY HIT ",50,120,YELLOW,screen);
	delay(1000);
	return 1;
      }
   }
 else if (current_local_pc_is == RECEIVER)
  {
     do
      {
	/* Send a character to remote telling it your the Receiver */
	send_a_character('R');  /* send the ascii character 'R' */
	/* Try to receive a 'S' character (Sender signal) from remote */
	in_char=get_character_from_in_buffer();

	if (in_char=='S')
	  {
	   out_text(" LINK ESTABLISHED ",100,120,YELLOW,screen);
	   if (soundon)
	    {sound (500); delay(300); nosound();}
	   delay(1000);
	  };
      }
     while ((in_char != 'S') && (!keypress()));

     /* Test to see if user wanted to abort */
     if (keypress())
      {
       /* Reset the keys array because of keypress */
	reset_keys_array();
	out_text(" GAME ABORTED ON KEY HIT ",50,120,YELLOW,screen);
	delay(1000);
	return 1;
      }
  }

 /* RETURN 0 for good link */
 return 0;
}
/*************************************************************************/
void do_quit_game_sequence(int abort_program)
/* This function simply restores the user's system back to the way it */
/* was before Tankgame started.....including memory release/isr's etc */
/* The flag abort_program is set if it is an early exit from Tankgame */
{
 /* Release memory */
  farfree(screen_buffer);
  release_tankgame_sprites();
  release_menu_files_allocated_memory(); /* release all menu sprites */

 /* Restore old video mode */
  set_mode(oldmode);

 /* Restore com port / isr */
 close_com_port(current_com_port);

 /* Restore old keyboard handler */
 restore_old_keyboard_handler();

 /* See if early termination of program is needed */
  if (abort_program)
    exit(0);
}
/*************************************************************************/
#define _MISCFNCS
#endif
