
/* Kirk Christiansen   */
/* MOVETANK.C          */

/* Functions for tank  */
/* movement around the */
/* screen.             */

#ifndef _MOVETANK
/***************************************************************************/
#include <stdlib.h>     /* for abs value macro */
#include "drawtank.c"
#include "tankvars.c"
/***************************************************************************/
/* Function Prototypes */
int iswrongcolor(int color, int tanknum);
int islegalmove(int newx, int newy, int mvtank);
void rotatetankleft(int tanknum);
void rotatetankright(int tanknum);
int movetankbackward(int who);
int movetankforward(int who);
/***************************************************************************/
int iswrongcolor(int color, int tanknum)
 /* This function simply returns whether or not a tank can */
 /* roll over a certain color. Such as a wall.             */
{
  /* All tanks can move over the ground and themselves */
  if ((color==GROUNDCOLOR)     || (color==tanks[tanknum].colors[0]) ||
      (color==tanks[tanknum].colors[1]))
    {
      return 0;
    }

  /* a tank can also run over these */
  if ( (color==POWER_UP_COLOR_1) ||
      (color==POWER_UP_COLOR_2) || (color==PROJEXPLOSIONCOLOR1) ||
      (color==PROJEXPLOSIONCOLOR2) ||  (color==PROJEXPLOSIONCOLOR3))
   {
     return 0;   /* tank can move over these */
   }

  /* All tanks can move through projectiles, so they can run if */
  /* they are getting ambushed! */
  if ((color==PLAYERPROJCOLOR) || (color==COMPUTERPROJCOLOR) ||
      (color==PLAYERPROJCOLOR2))
      {
	return 0;
      }

  if ((color==BRIDGE_COLOR_1) || (color==BRIDGE_COLOR_2 ) ||
       (color==LIGHTBLUE))
      {
	return 0;
      }

 return 1;   /* tank cannot go over other colors */
}
/***************************************************************************/
int islegalmove(int newx, int newy, int mvtank)
{
 /*  This function checks to see if the tank can move to     */
 /* coordinates (newx,newy) by calculating the boundary      */
 /* of the tank itself.  A '1' is returned for a legal move. */

 int i,x,y;

 int upxbound = newx+TANKBOUND;        /* set the tank boundary.    */
 int lowxbound = newx-TANKBOUND;       /* i.e.  an invisible square */
 int upybound = newy + TANKBOUND;      /* around the tank to help   */
 int lowybound = newy - TANKBOUND;     /* detect collisions.        */

    /* First, check to see if any other tanks are in the move path */
  for (i=0;i<MAXTANKS;i++)
  {
     /* If moving tank isn't i and obstacle tank is playing */
    if ((tanks[mvtank].tanknumber != i) &&
	(tanks[i].tankstats[GAMESTATUS] == PLAYING))
     {
       /* Now take the absolute values of distance from the middle     */
       /* of the moving tank to the middle of the obstacle tank. if    */
       /* these x and y values are both less than twice the tank size, */
       /* then the obstacle tank is in the way.                        */
       if ( ((abs(newx-tanks[i].xcor)) < TSMUL2) &&
	    ((abs(newy-tanks[i].ycor)) < TSMUL2) )
	  {
	    return 0;  /* move is illegal */
	  }
     }
  }

    /* Now, check for any obstacles in the new move path */
    /* NOTE: take into account if tank is facing diagonally(i.e even #) */
    /* NOTE: might want to change below x+=2, to x+=TANKBOUND */

  if ((tanks[mvtank].pos)%2)  /* odd position or up,right,down,left */
   {
    for (x=lowxbound;x<=upxbound;x+=TANKBOUND)    /* These for loops check */
     for (y=lowybound;y<=upybound;y+=TANKBOUND)/* the majority of pixels */
      {                                  /* in the new desired location. */
	 if(iswrongcolor(getpixel(x,y,screen_buffer),mvtank))
	   return 0;  /* move is illegal */
      }
   }

  else  /* tank is facing diagonally */
   {
      /* Check upmost and lowest point on tank sprite */
      for (x=newx-1;x<=newx+1;x++)
       {
	if(iswrongcolor(getpixel(x,lowybound-1,screen_buffer),mvtank))
	  return 0;  /* move is illegal */
       if(iswrongcolor(getpixel(x,upybound+1,screen_buffer),mvtank))
	  return 0;  /* move is illegal */
       }
      /* Check, next upmost and lowest points on tank sprite */
      for (x=newx-2;x<=newx+2;x++)
       {
	if(iswrongcolor(getpixel(x,lowybound,screen_buffer),mvtank))
	  return 0;  /* move is illegal */
       if(iswrongcolor(getpixel(x,upybound,screen_buffer),mvtank))
	  return 0;  /* move is illegal */
       }
     /* Finally, check rest of body. NOTE +=2 on one of these can be */
     /* improved to +=TS or something.                              */
     for (x=lowxbound;x<=upxbound;x+=2)
      for (y=lowybound+1;y<=upybound-1;y+=2)
      {
       if(iswrongcolor(getpixel(x,y,screen_buffer),mvtank))
	  return 0;  /* move is illegal */
      }

   } /* end else tank is facing diagonally */

 return 1;   /* move is legal */
}
/***************************************************************************/
void rotatetankleft(int tanknum)
/* This function simply rotates the tank to its left one turn */
{
 erasetank(tanknum);

 if (tanks[tanknum].pos != UP)       /* tank not pointing up */
   {
    tanks[tanknum].pos = (tanks[tanknum].pos - 1);
   }
 else
   {
    tanks[tanknum].pos = UPLEFT;  /* tank was pointing up, now goes to upleft */
   }
}
/***************************************************************************/
void rotatetankright(int tanknum)
/* This function simply rotates the tank to its right one turn */
{
 erasetank(tanknum);

 if (tanks[tanknum].pos != UPLEFT)       /* tank not pointing upleft */
   {
    tanks[tanknum].pos = (tanks[tanknum].pos + 1);
   }
 else
   {
    tanks[tanknum].pos = UP;     /* tank was pointing upleft, now goes to up */
   }
}
/***************************************************************************/
int movetankbackward(int who)
/* This function moves a tank backwards one position */
{
   int newx=0, newy=0;
   int tankhasmoved = FALSE;
   int i = 0;  /* i represents the movement space to be subtracted  */
	       /* from TANKMOVERATE to check for a valid move space */
	       /* this is so you will back right up to a wall if    */
	       /* it is in your path. */
  do
   {
    switch (tanks[who].pos)
       {
	  case UP :   newx = tanks[who].xcor;
		      newy = (tanks[who].ycor + (TANKMOVERATE - i));
		      break;
	  case UPRIGHT :   newx = tanks[who].xcor - (TANKMOVERATE - i);
			   newy = (tanks[who].ycor + (TANKMOVERATE - i));
			   break;

	  case RIGHT : newx = (tanks[who].xcor - (TANKMOVERATE - i));
		       newy = tanks[who].ycor;
		       break;

	  case DOWNRIGHT :   newx = tanks[who].xcor - (TANKMOVERATE - i);
			   newy = (tanks[who].ycor - (TANKMOVERATE - i));
			   break;

	  case DOWN  : newx = tanks[who].xcor;
		       newy = (tanks[who].ycor - (TANKMOVERATE - i));
		       break;

	  case DOWNLEFT :   newx = tanks[who].xcor + (TANKMOVERATE - i);
			   newy = (tanks[who].ycor - (TANKMOVERATE - i));
			   break;

	  case LEFT  : newx = (tanks[who].xcor + (TANKMOVERATE - i));
		       newy = tanks[who].ycor;
		       break;
	  case UPLEFT :   newx = tanks[who].xcor + (TANKMOVERATE - i);
			   newy = (tanks[who].ycor + (TANKMOVERATE - i));
			   break;

       }

	    if (islegalmove(newx,newy,who))  /* 1=True */
	       {
		 erasetank(who); /* erase the old position */
		 tanks[who].xcor=newx;  /* reset tank's coordinates */
		 tanks[who].ycor=newy;
		 tankhasmoved=TRUE;
	       }

    i++;
   }
  while ((!tankhasmoved) && (i < TANKMOVERATE));

 return tankhasmoved;
}
/***************************************************************************/
int movetankforward(int who)
/* This function moves a tank forward one position */
{
   int newx=0, newy=0;
   int tankhasmoved = FALSE;
   int i = 0;  /* i represents the movement space to be subtracted  */
	       /* from TANKMOVERATE to check for a valid move space */
	       /* this is so you will move right next to a wall if  */
	       /* it is in your path. */
  do
   {
    switch (tanks[who].pos)
       {
	  case UP :   newx = tanks[who].xcor;
		      newy = (tanks[who].ycor - (TANKMOVERATE - i));
		      break;
	  case UPRIGHT :   newx = tanks[who].xcor + (TANKMOVERATE - i);
			   newy = (tanks[who].ycor - (TANKMOVERATE - i));
			   break;

	  case RIGHT : newx = (tanks[who].xcor + (TANKMOVERATE - i));
		       newy = tanks[who].ycor;
		       break;

	  case DOWNRIGHT :   newx = tanks[who].xcor + (TANKMOVERATE - i);
			   newy = (tanks[who].ycor + (TANKMOVERATE - i));
			   break;

	  case DOWN  : newx = tanks[who].xcor;
		       newy = (tanks[who].ycor + (TANKMOVERATE - i));
		       break;
	  case DOWNLEFT :   newx = tanks[who].xcor - (TANKMOVERATE - i);
			   newy = (tanks[who].ycor + (TANKMOVERATE - i));
			   break;
	  case LEFT  : newx = (tanks[who].xcor - (TANKMOVERATE - i));
		       newy = tanks[who].ycor;
		       break;
	  case UPLEFT :   newx = tanks[who].xcor - (TANKMOVERATE - i);
			   newy = (tanks[who].ycor - (TANKMOVERATE - i));
			   break;
       }

	    if (islegalmove(newx,newy,who))  /* 1=True */
	       {
		 erasetank(who); /* erase the old tank position */
		 tanks[who].xcor=newx;  /* reset tank's coordinates */
		 tanks[who].ycor=newy;
		 tankhasmoved=TRUE;
	       }
    i++;
   }
  while ((!tankhasmoved) && (i < TANKMOVERATE));

 return tankhasmoved;
}
/***************************************************************************/
#define _MOVETANK
#endif

