
/* Kirk Christiansen */
/* PCXFUNCS.C           */

/*  A set of functions for loading */
/* and compressing PCX images.     */
/**********************************************************************/
#include  <stdio.h>
#include  <dos.h>
#include  <io.h>
#include  <fcntl.h>
#include  <mem.h>
#include  <alloc.h>
/**********************************************************************/
#ifndef _PCXFUNCS
/**********************************************************************/
#define	NO_ERROR 0
#define FILE_NOT_OPENED 1
#define INSUFFICIENT_MEMORY 2
#define TOO_MANY_ARGUMENTS 3

#define BUFLEN 5*1024
/**********************************************************************/
struct pcx_header {
  char manufacturer;
  char version;
  char encoding;
  char bits_per_pixel;
  int  xmin;
  int  ymin;
  int  xmax;
  int  ymax;
  int  hres;
  int  vres;
  char palette16[48];
  char reserved;
  char color_planes;
  int  bytes_per_line;
  int  palette_type;
  char filler[58];
};
/**********************************************************************/
struct pcx_struct {
		   struct pcx_header header;
		   unsigned char far *image;
		   unsigned char palette[3*256];
		  };
/**********************************************************************/
/* Function Prototypes */
int  loadPCX(char *filename,struct pcx_struct *pcx);
int  loadPCX_palette_only(char *filename,struct pcx_struct *pcx);
int  load_image(FILE *pcxfile,struct pcx_struct *pcx);
void load_palette(FILE *pcxfile,struct pcx_struct *pcx);
/**********************************************************************/
const int  MINIMUM_RUN=4;
/* constants for load image function */
const int BYTEMODE=0, RUNMODE=1;
/**********************************************************************/
/**********************************************************************/
int loadPCX(char *filename,struct pcx_struct *pcx)
{
  FILE *infile;
  int x;

	 /* Open file, abort if not found */
	if ((infile=fopen(filename,"rb"))==NULL) /* rb for 'read binary' */
	  {
	    printf(" Open .DAT file error. \n");
	    return(-1);
	  }


       /* Set file pointer to beginning of the file */
	 fseek(infile,0L,SEEK_SET);

       /* Read header information into header field */
       /* The complete header is 6 integers(WORDS),6 chars(BYTES), */
       /* and two char arrays (BYTE ARRAY) of 58 & 48.             */
       /* Total bytes = (8*2)+6+48+58=128 bytes.                   */
	fread(&pcx->header,/*sizeof(struct pcx_header)*/128,1,infile);


	if ((pcx->header.xmax - pcx->header.xmin)+1 > 320)
	  {
	   printf(" Error. X size is greater than 320. \n");
	   printf(" X size is %d\n",pcx->header.xmax-pcx->header.xmin+1);
	   return(-1);
	  }

	if (pcx->header.ymax - pcx->header.ymin+1 > 200)
	  {
	   printf(" Error. Y size is greater than 200. \n");
	   printf(" Y size is %d\n",pcx->header.ymax-pcx->header.ymin+1);
	   return(-1);
	  }

	if (pcx->header.version != 5)
	  {
	   printf(" Version Error. Need version 5 (256 clr version.");
	   printf(" Version is %d\n",pcx->header.version);
	   return(-1);
	  }

	/* Load in the actual picture */
	if (load_image(infile,pcx))
	 {
	  return(-1);
	 }

	/* Load in the picture's color palette */
	load_palette(infile,pcx);

	fclose(infile);

   return(0);
}
/**********************************************************************/
int load_image(FILE *pcxfile,struct pcx_struct *pcx)
/* This function loads in the 64,000 byte image in a PCX file */
{
	int image_width, image_height;
	unsigned char far *image_ptr;
	unsigned int i;
	unsigned int image_size;

	long count; /* number of bytes read in so far from image */
	unsigned char data;
	int num_bytes;  /* for run count of an RLE */

      /* Calculate image_width */
	image_width = pcx->header.xmax - pcx->header.xmin + 1;
      /* Calculate image_height */
	image_height = pcx->header.ymax - pcx->header.ymin + 1;
      /* Calculate image_size  */
	image_size = (long)image_width * image_height;


      /* Allocate memory for image, abort if not available */
      pcx->image = (char far *) farcalloc(image_size,sizeof(unsigned char));


      if ((pcx->image)==NULL)
	{
	 sound(1000);delay(500);nosound();
	 printf (" Memory not available \n for image_size %lu\n",image_size);
	 return(-1);
	}


	count=0;   /* Number of characters read in so far */

	while (count <= image_size)
	 {
	   /* Get the first piece of data */
	   data=(unsigned char)getc(pcxfile);

	   /* Is this an RLE? (a run of same colored pixels) */
	   if (data > 191)
	    {
	     /* How many bytes in the run */
	      num_bytes = data-192;
	     /* Get the actual data for the run */
	      data = (unsigned char)getc(pcxfile);
	     /* Now, replicate the data in the buffer num_bytes times */
	      while ((num_bytes--) > 0)
		{
		  pcx->image[count++] = data;
		}
	    } /* end if RLE */

	   else
	    {
	      /* Actual data, just copy it into the buffer */
	      pcx->image[count++] = data;
	    }
	 } /* end while (count <= image_size) */



  return 0;  /* no errors */
}
/**********************************************************************/
void load_palette(FILE *pcxfile,struct pcx_struct *pcx)
{
  int i,j;

	/* Goto end of file, then back up 768 bytes because that */
	/* is the size of the palette which is always attached   */
	/* to the end of the pcx file.                           */
	fseek(pcxfile,-768L,SEEK_END);

	/* read in the entire palette */
	fread(pcx->palette,3*256,1,pcxfile);


	/* Shift off the two left bits on the colors because */
	/* they are not used. Just the rightmost 6 bits are  */
	/* used for the whole palette.                       */
	for (i=0; i<256; i++)
	  {
	   for (j=0; j<3; j++)
	    {
	     pcx->palette[i*3+j] = pcx->palette[i*3+j]>>2;
	    }
	  }
}
/**********************************************************************/
/* compress function is commented out here for tankgame to save mem */
//int compress(struct pcx_struct *pcx)
/* NOTE: this function needs some more code to work completely. */
/* For now, I am not using it.                                  */
//{
//	long i;
//	int value,runlength,runptr;

//	int image_width;
//	int image_height;
//	long image_size;
//	long ptr=0;
//	long cptr=0;
 //	unsigned char far *cbuffer;

      /* Calculate image_width */
//	image_width = pcx->header.xmax - pcx->header.xmin + 1;
      /* Calculate image_height */
//	image_height = pcx->header.ymax - pcx->header.ymin + 1;
      /* Calculate image_size  */
//	image_size = (long)image_width * image_height;

      /* Allocate memory for cbuffer */
//     cbuffer = (unsigned char far *) farcalloc(65530,sizeof(unsigned char));


/*	while (ptr<image_size)
	   {
		if (pcx->image[ptr]==0)
		   {
		    runlength=0;
		    value=pcx->image[ptr];

			while (pcx->image[ptr]==0)
			 {
			   ptr++;
			   runlength++;

			   if (runlength>=127)
			     break;
			   if (ptr >= image_size)
			     break;
			  }

			cbuffer[cptr] = (unsigned char) (runlength + 128);
			cptr++;
			cbuffer[cptr] = (unsigned char) value;
			cptr++;
		   }
		else
		 {
		  runlength=0;
		  runptr=cptr++;
		   while (pcx->image[ptr]!=0)
		    {
		     cbuffer[cptr]=pcx->image[ptr];
		     cptr++;
		     ptr++;
		     runlength++;

			if (runlength>=127)
			  break;
			if (ptr >= image_size)
			  break;
		    }

		   cbuffer[runptr]=runlength;
		 }
	   }

	  cptr = 0;

  return(cptr);
}
  */
/**********************************************************************/
int loadPCX_palette_only(char *filename,struct pcx_struct *pcx)
/* this function just loads a PCX's palette not its image */
{
  FILE *infile;
  int x;

	 /* Open file, abort if not found */
	if ((infile=fopen(filename,"rb"))==NULL) /* rb for 'read binary' */
	  {
	    printf(" Open PCX file error. \n");
	    return(-1);
	  }


       /* Set file pointer to beginning of the file */
	 fseek(infile,0L,SEEK_SET);

       /* Read header information into header field */
       /* The complete header is 6 integers(WORDS),6 chars(BYTES), */
       /* and two char arrays (BYTE ARRAY) of 58 & 48.             */
       /* Total bytes = (8*2)+6+48+58=128 bytes.                   */
	fread(&pcx->header,/*sizeof(struct pcx_header)*/128,1,infile);


	if ((pcx->header.xmax - pcx->header.xmin)+1 > 320)
	  {
	   printf(" Error. X size is greater than 320. \n");
	   printf(" X size is %d\n",pcx->header.xmax-pcx->header.xmin+1);
	   return(-1);
	  }

	if (pcx->header.ymax - pcx->header.ymin+1 > 200)
	  {
	   printf(" Error. Y size is greater than 200. \n");
	   printf(" Y size is %d\n",pcx->header.ymax-pcx->header.ymin+1);
	   return(-1);
	  }

	if (pcx->header.version != 5)
	  {
	   printf(" Version Error. Need version 5 (256 clr version.");
	   printf(" Version is %d\n",pcx->header.version);
	   return(-1);
	  }

	/* Load in the picture's color palette */
	load_palette(infile,pcx);

	fclose(infile);

   return(0);
}
/**********************************************************************/
#define _PCXFUNCS
#endif