
/* Kirk Christiansen */
/* POWERUPS.C        */

/* This module contains functions */
/* and code for maintaining the   */
/* whereabouts and data of power  */
/* up pills in Tankgame.          */

/************************************************************************/
#ifndef _POWERUPS
/************************************************************************/
#include "kc_pcsnd.c"
#include "kcgrphcs.c"
#include "tankvars.c"
/************************************************************************/
#define PWR_UP_X_OFFSET  5
#define PWR_UP_Y_OFFSET  6
#define POWER_UP_PICT_WIDTH   10
#define POWER_UP_PICT_HEIGHT  13
/************************************************************************/
/* Function Prototypes */
void reset_power_ups (void);
void reset_power_up (int pwr_up_num);
void set_power_up (int x,int y,int type,int value);
void draw_power_ups (void);
void erase_power_up(int num);
void check_for_power_up_pickups(void);
void give_tank_power_up_stats(int tanknum, int power_up_num);
/************************************************************************/
void reset_power_ups (void)
{
 int i;

  for (i=0;i<MAX_POWER_UPS;i++)
   {
     power_ups[i].x=0;
     power_ups[i].y=0;
     power_ups[i].type=0;
     power_ups[i].value=0;
   }
}
/************************************************************************/
void reset_power_up (int pwr_up_num)
{
     power_ups[pwr_up_num].x=0;
     power_ups[pwr_up_num].y=0;
     power_ups[pwr_up_num].type=0;
     power_ups[pwr_up_num].value=0;
}
/************************************************************************/
void set_power_up (int x,int y,int type,int value)
/* This function simply checks the power-up array for an empty space */
/* so the inputed power-up pill data can be stored.                  */
{
 int i;

  for (i=0;i<MAX_POWER_UPS;i++)
   {
     if (!power_ups[i].x)   /* A empty place is open in array */
       {
	 power_ups[i].x=x;
	 power_ups[i].y=y;
	 power_ups[i].type=type;
	 power_ups[i].value=value;
	 return;
       }
   }
}
/************************************************************************/
void draw_power_ups (void)
/* This function simply puts a power pill or pills on the current */
/* board if they exist.                                           */
{
 int i;

   for (i=0;i<MAX_POWER_UPS;i++)
    {
     if (power_ups[i].x) /* If a power-up pill exists */
      {
       blit(power_ups[i].x-PWR_UP_X_OFFSET,power_ups[i].y-PWR_UP_Y_OFFSET,
	    POWER_UP_PICT_WIDTH,POWER_UP_PICT_HEIGHT,screen_buffer,
	    misc_picts[1+power_ups[i].type]);
       }
     }
}
/************************************************************************/
void erase_power_up(int num)
{
  color_blit(power_ups[num].x-PWR_UP_X_OFFSET,
	     power_ups[num].y-PWR_UP_Y_OFFSET,
	     POWER_UP_PICT_WIDTH,POWER_UP_PICT_HEIGHT,BLACK,screen_buffer,
	     misc_picts[1+power_ups[num].type]);

}
/************************************************************************/
void check_for_power_up_pickups(void)
{
  int i,j;

  for (i=0;i<MAX_POWER_UPS;i++)
   {
    for (j=0; j<MAXTANKS; j++)
     {
      if (power_ups[i].x) /* power pill exists */
       {
	if (tanks[j].tankstats[GAMESTATUS])  /* tank is alive */
	 {
	  /* On below if statement I had < TS + POWER_UP_WIDTH, */
	  /* but you ran over them too easy that way.           */
	  if ( ((abs(power_ups[i].x-tanks[j].xcor)) < TS) &&
	       ((abs(power_ups[i].y-tanks[j].ycor)) < TS) )
	    {
		/* delete pill, give tank the goods */
	       if (soundon)
		make_sound(POWER_UP_PICKUP);
	       give_tank_power_up_stats(j,i);
	       erase_power_up(i);
	       reset_power_up(i);
	    }
	 }
       }
     }
   }
}
/************************************************************************/
void give_tank_power_up_stats(int tanknum, int power_up_num)
/* This function simply transfers a power_up pill's stats to the */
/* tank that ran over it.                                        */
{
  int weapon_pill=FALSE;  /* was the power pill of weapon type? */
  int prev_weapon_ammo;
  int weapon_is;

  switch (power_ups[power_up_num].type)
  {
     case ARMOR_PILL :
	     tanks[tanknum].armor += power_ups[power_up_num].value;
	     break;
     case REBOUND_PILL :
	     weapon_pill=TRUE;
	     weapon_is=REBOUNDING_PROJ;
	     prev_weapon_ammo=tanks[tanknum].weapon_ammo[weapon_is];

	     tanks[tanknum].weapon_ammo[REBOUNDING_PROJ]+=
				     power_ups[power_up_num].value;
	     break;
     case PIERCING_PILL :
	     weapon_pill=TRUE;
	     weapon_is=PIERCING_PROJ;
	     prev_weapon_ammo=tanks[tanknum].weapon_ammo[weapon_is];

	     tanks[tanknum].weapon_ammo[PIERCING_PROJ]+=
				     power_ups[power_up_num].value;

  };

      /* check to see if tank has weapon already, if not */
      /* write out the abbreviation to screen.           */
      if ((weapon_pill) && (prev_weapon_ammo==0) && (tanknum==0))
	{
	  //display_abbreviation(weapon_is,WEAPON_ABRV_COLOR);
	}

      /* If pill was a weapon, switch current_weapon to the new weapon */
      if (weapon_pill)
	{
	  change_to_a_specific_weapon(weapon_is,tanknum);
	}

}
/************************************************************************/
#define _POWERUPS
#endif



