
/* Kirk Christiansen */
/* SER_LIB.C         */

/* Communications module */
/* Com port/ Serial port */
/* functions.            */

#ifndef _SER_LIB
#define _SER_LIB
/**************************************************************************/
#include <stdio.h>
#include <dos.h>
#include <conio.h>
//#include "tankvars.c"
#include "kc_keybd.c"
/**************************************************************************/
/* CONSTANTS */
#define TRUE  1
#define FALSE 0

#define COM_PORT_1        0x00
#define COM_PORT_2        0x01

#define COM_1  0x3f8  /* base address of com port 1 */
#define COM_2  0x2f8  /* base address of com port 2 */

#define INT_SER_PORT_0  0x0c  /* port 0 interrupt com1 and com3 */
#define INT_SER_PORT_1  0x0b  /* port 0 interrupt com2 and com4 */

#define BAUD_1200    96   /* or 01100000 */
#define BAUD_2400    48   /* or 00110000 */
#define BAUD_9600    12   /* or 00001100 */
#define BAUD_19200    6   /* or 00000110 */

#define GP02  8     /* enable interrupt  -- binary=00001000 */

#define DLL 0        /* low byte of the baud rate divisor. */
#define DLH 1        /* high byte of divisor latch  */

#define STOP_1  0  /* 1 stop bit per char */
#define STOP_2  4  /* 2 stop bits per char */

#define BITS_5  0  /* send five bit chars */
#define BITS_6  1  /* send six bit chars */
#define BITS_7  2  /* send seven bit chars */
#define BITS_8  3  /* send eight bit chars */

#define PARITY_NONE 0
#define PARITY_ODD  8
#define PARITY_EVEN 24

#define DIV_LATCH_ON 128   /* used to turn reg 0,1 into divisor latch */

#define PIC_IMR  0x21   /* PIC's interrupt mask register */
#define PIC_ICR  0x20   /* PIC's interrupt control register */

/* UART registers */
/* The below numbers are offsets in the UART, this way you can */
/* choose which COM port to use and just add these offsets to  */
/* the base address of the COM port you are using.             */
#define RBR 0	 /* Receiver Buffer Register          */
#define THR 0    /* Transmitter Holding Register      */
#define IER 1	 /* Interrupt Enable Register         */
#define IIR 2	 /* Interrupt Identification Register */
#define LCR 3	 /* Line Control Register             */
#define MCR 4	 /* Modem Control Register            */
#define LSR 5	 /* Line Status Register              */
#define MSR 6	 /* Modem Status Register             */

#define PIC_IMR  0x21     /* PIC's Interrupt Masking Register  */
#define PIC_ICR  0x20     /* PIC's Interrupt Control Register  */

#define TOKEN_CHAR 126

/* Globals */
void interrupt far (*old_com_handler) ();

/* Pointers to circular buffer positions */
#define MAX_CIRCULAR_BUFFER_SIZE 128  /* cirular buffer size */

char in_buffer[MAX_CIRCULAR_BUFFER_SIZE];  /* receiving buffer */

int in_start= -1;
int in_end= -1;
int in_count=0;   /* Count of current number of characters in in_buffer */

int old_int_mask;    /* stores the old interrupt mask on the PIC */

int current_port;  /* the current open communications port */

int serial_lock = FALSE; /* Serial ISR semaphore so the buffer */
			 /* isn't altered while it is being manipulated */



/**************************************************************************/
/* Function Prototypes */
void interrupt far my_com_handler(void);
void initialize_com_port(int port_base, int baud, int configuration);
void close_com_port (int port_base);
void send_a_character(char ch);
int  get_character_from_in_buffer (void);

unsigned char convert_scan_to_ascii (int scan_is);
int convert_ascii_to_scan (unsigned char ascii_is);
/**************************************************************************/
void interrupt far my_com_handler(void)
/* This is my ISR for assignment #2.  When invoked it gets the next */
/* character out of the RBR and places it in my circular in_buffer. */
/* Notice the lock to prevent buffer corruption.                    */
{
  int in_char;  /* character received from RBR */

  /* Lock out any other functions to avoid messing up buffer pointers */
    serial_lock=TRUE;

  /* Get the character out of ther RBR */
  in_char = inportb(current_port+RBR);

   /* Enqueue it */
       /* Test for wrap */
	if (++in_end > MAX_CIRCULAR_BUFFER_SIZE-1)
	   in_end=0;
       /* Put char in the buffer */
	in_buffer[in_end] = in_char;
       /* Increase the count of characters in in_buffer */
	 in_count++;

  /* Restore PIC */
  outportb(PIC_ICR,0x20);

  /* Undo Lock */
   serial_lock=FALSE;
}
/**************************************************************************/
void initialize_com_port(int port_num, int baud, int configuration)
/* This func opens up the serial(com) port, sets the desired config, */
/* turns on all the needed flags & bits to make interrupts happen, */
/* and loads in the ISR (interrupt service routine)        */
{
  int port_base;     /* used for base address */

  if (port_num==1)  /* com port 1 */
    port_base=COM_1;
  else
    port_base=COM_2;

  /* Save the port globally for other funcs */
   current_port = port_base;

  /* Set the baud */
  /* Turn on divisor latch regs */
    outportb(port_base+LCR, DIV_LATCH_ON);

  /* Send low & high bytes to divisor latches */
    outportb(port_base + DLL, baud);
    outportb(port_base + DLH, 0);

  /* Set the port configuration based on what user wanted */
    outportb(port_base + LCR, configuration);

  /* Enable interrupts */
    outportb(port_base + MCR, GP02);
    outportb(port_base + IER, 1);

  /* Hold off enabling PIC until we have the com ISR installed */
   if (port_base == COM_1)
    {
     old_com_handler = getvect(INT_SER_PORT_0);
     setvect(INT_SER_PORT_0, my_com_handler);
    }
   else
    {
     old_com_handler = getvect(INT_SER_PORT_1);
     setvect(INT_SER_PORT_1, my_com_handler);
    }


   /* Enable interrupt on PIC */
   old_int_mask = inportb(PIC_IMR);
   outportb (PIC_IMR,(port_base==COM_1) ? (old_int_mask & 0xef) :
		     (old_int_mask & 0xf7));
}
/**************************************************************************/
void close_com_port (int port_num)
/* Closes the port, which entails turning off ints and restoring the  */
/* old ISR.  This function is called during shut down of program.     */
{
  int port_base;     /* used for base address */

  if (port_num==1)  /* com port 1 */
    port_base=COM_1;
  else
    port_base=COM_2;

 /* Disable ints */
 outportb(port_base + MCR, 0);
 outportb(port_base + IER, 0);
 outportb(PIC_IMR, old_int_mask);

 /* Reset old isr */
 if (port_base == COM_1)
  {
   setvect(INT_SER_PORT_0, old_com_handler);
  }
 else
  {
   setvect(INT_SER_PORT_1, old_com_handler);
  }
}
/**************************************************************************/
void send_a_character(char ch)
/* This function tries to transmit a character to the remote computer */
/* by testing to see if transmit buffer is empty. It turns of interrupts */
/* during the process. */
{
  /* First, wait till transmit buffer is empty */
  /* This done by checking the 5th bit in the line status register */
  while (!(inportb(current_port + LSR) & 0x20)) {}

  /* Now turn off the interrupts */
   disable();

  /* Send the character */
   outportb(current_port + THR, ch);

  /* Turn interrupts back on */
   enable();
}
/**************************************************************************/
int get_character_from_in_buffer (void)
/* This func reads a char from the circ buffer and returns it to caller */
{
 int ch;

 /* Wait for isr to end */
  while (serial_lock) {}

 /* Check to see if char there */
 if (in_end != in_start)
   {
      /* Do wrap if necc */
      if (++in_start > MAX_CIRCULAR_BUFFER_SIZE-1)
	 in_start=0;
      /* Get the char out of buff */
       ch = in_buffer[in_start];

      /* There is one less character in the buffer now */
       if (in_count > 0)
	  --in_count;

       /* Send character back to caller */
	return ch;
   }
 else
   {
     /* buff empty */
     return 0;
   }
}
/**************************************************************************/
unsigned char convert_scan_to_ascii (int scan_is)
/* This function takes in as input a scan_code and converts the code */
/* into an ascii value. */
{
  unsigned char c=0;

   switch (scan_is)
    {
      case UPARROW : c=24; break;  /* ascii char up arrow */
      case LEFTARROW : c=27; break;
      case RIGHTARROW : c=26; break;
      case DOWNARROW : c=25; break;
      case SPACE  : c=32; break; /* ascii space */
      case ENTER : c=13; break;  /* ascii enter ^M */
      case ESC : c=127; break;   /* use 0 as a quit character */
      case TAB : c=16; break; /* use 16 as a TAB character */
      case PLUS : c=43; break;
      case MINUS : c=45; break;
      case KEY_1 : c=49; break;
      case KEY_2 : c=50; break;
      case KEY_3 : c=51; break;
      case KEY_4 : c=52; break;
      case KEY_5 : c=53; break;
      case KEY_6 : c=54; break;
      case KEY_7 : c=55; break;
      case KEY_8 : c=56; break;
      case KEY_9 : c=57; break;
      case KEY_0 : c=48; break;
      case KEY_A : c=65; break;
      case KEY_F : c=70; break;
      case KEY_L : c=76; break;
      case KEY_P : c=80; break;
      case KEY_Q : c=81; break;
      case KEY_R : c=82; break;
      case KEY_S : c=83; break;
      case KEY_T : c=84; break;
    };
 return c;
}
/**************************************************************************/
int convert_ascii_to_scan (unsigned char ascii_is)
/* This function takes in as input an ascii value and converts it */
/* to a scan code */
{
  int s=0;

  switch (ascii_is)
   {
     case 24 : s=UPARROW; break;
     case 27 : s=LEFTARROW; break;
     case 26 : s=RIGHTARROW; break;
     case 25 : s=DOWNARROW; break;
     case 32 : s=SPACE; break;
     case 13 : s=ENTER; break;
     case 127 : s=ESC; break;
     case 16 : s=TAB; break;
     case 43 : s=PLUS; break;
     case 45 : s=MINUS; break;
     //case 49 : s=KEY_1; break;
     //case 50 : s=KEY_2; break;
     //case 51 : s=KEY_3; break;
     //case 52 : s=KEY_4; break;
     //case 53 : s=KEY_5; break;
     //case 54 : s=KEY_6; break;
     //case 55 : s=KEY_7; break;
    // case 56 : s=KEY_8; break;
     //case 57 : s=KEY_9; break;
     //case 48 : s=KEY_0; break;
     //case 65 : s=KEY_A; break;
     //case 70 : s=KEY_F; break;
     //case 76 : s=KEY_L; break;
     //case 80 : s=KEY_P; break;
     case 81 : s=KEY_Q; break;
     //case 82 : s=KEY_R; break;
     //case 83 : s=KEY_S; break;
     //case 84 : s=KEY_T; break;
    // case 126: s=TOKEN_CHAR;
   };
  return s;
}
/**************************************************************************/
#endif