
/* Kirk Christiansen */
/* SETBOARD.C        */

/* This module contains the functions   */
/* for updating the level in Tank Game. */

#ifndef _SETBOARD
/***************************************************************************/
#include "tankvars.c"
#include "powerups.c"
#include "pcxfuncs.c"
/* NOTE: this module also calls some fncs from "tankfncs.c" */
/***************************************************************************/
/* Function Prototypes */
void init_comp_tank(int num, int amode, int x, int y, int pos);
void set_up_level (void);
void set_up_board (void);
void set_default_coordinates (int tanknum);
void reset_player_tank (void);
void reset_second_player_tank (void); /* used for 2plyr Fullgame */
void set_tank_types (void);
void reset_explosion_array(void);
void load_in_night_fight_palette(void);
/***************************************************************************/
void init_comp_tank(int num, int amode, int x, int y, int pos)
/* This function sets up a computer tank so it can play */
{
     tanks[num].playertype = COMPUTER;
     tanks[num].tanktype   = COMPUTERTANK;
     tanks[num].tanknumber = num;
     tanks[num].tankstats[GAMESTATUS] = PLAYING;
     tanks[num].previousarmor = STARTARMOR;
     tanks[num].armor        = STARTARMOR;

    if (amode==TERMINATOR)  /* if terminator set appropiate colors + type */
     {
      tanks[num].tanktype = HUNTERKILLER;
      tanks[num].colors[0]  = HKCOLOR;
      tanks[num].colors[1]  = HKCOLOR_2;
     }
    else if (amode==DUMMY)  /* if terminator set appropiate colors + type */
     {
      tanks[num].tanktype = DUMMYTANK;
     }
    else /* defaults */
     {
      tanks[num].colors[0]    = COMPUTERCOLOR;
      tanks[num].colors[1]    = COMPUTERCOLOR_2;
     }

     tanks[num].projcolor    = COMPUTERPROJCOLOR;
     tanks[num].current_weapon = NORMAL_PROJ;

   if (!amode)  /* if (the tank is not specifically programmed) */
     {
      tanks[num].attackmode=NAIVE2; /* Set it to default NAIVE2       */
     }
   else
     {
      tanks[num].attackmode=amode;
     }

   /* Now set the other parameters that were passed in */
   /* If x & y & pos are passed in as all 0 then we know the caller */
   /* wants the default coordinates. */
   if ((x==0)&&(y==0)&&(pos==0))
    {
      set_default_coordinates(num);
    }
   else /* caller has passed in some different coordinates */
    {
     tanks[num].xcor=x;
     tanks[num].ycor=y;
     tanks[num].pos=pos;
    }
}
/**************************************************************************/
void set_up_level (void)
/* This function contains all the code for changing levels. */
/* This includes setting up tank coordinates, board types & colors, */
/* tank heuristics, and initializations.                            */
{
  if (gametype==FULLGAME)
   {
    reset_player_tank ();  /* erase data from last level */
    reset_tanks (COMPTANKS); /* reset all the computer tanks */
    reset_explosion_array();
    reset_power_ups();
    display_level();
   }
  else if (gametype==TWOPLYR_FULLGAME)
   {
    reset_player_tank ();  /* erase data from last level */
    reset_second_player_tank ();  /* erase data from last level */
    reset_tanks (COMPTANKS); /* reset all the computer tanks */
    reset_explosion_array();
    reset_power_ups();
    display_level();
   }

  /* SET THE PLAYER TANKS TO DEFAULT COORDINATES */
  /* these can be changed below if needed.       */
	set_default_coordinates(0);
	set_default_coordinates(1);

  switch (level)
   {
    case 0:  /* One match game */ /* Obsolete in NEW TANKGAME? */
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     break;
    case 1:  /* The first stage */
	     boardnumber=CITY; boardcolor=LIGHTGRAY;
	     set_power_up(290,20,REBOUND_PILL,10);
	     set_power_up(30,150,ARMOR_PILL,3);
	     set_power_up(290,150,REBOUND_PILL,10);
	     set_power_up(30,20,PIERCING_PILL,30);
	     init_comp_tank(2,PATROL,0,0,0);
	     tanks[2].armor=tanks[2].previousarmor=6; /* weaken armor */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(160,150,ARMOR_PILL,3);
	       init_comp_tank(3,PATROL,300,35,DOWN);
	       tanks[3].armor=tanks[3].previousarmor=6; /* weaken armor */
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=8;
		if (tanks[3].armor) /* if tank is initialized */
		  tanks[3].armor=tanks[3].previousarmor=8;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
		if (tanks[3].armor) /* if tank is initialized */
		  tanks[3].armor=tanks[3].previousarmor=4;
	      }
	     break;

    case 2:  /* The second level */
	     boardnumber=PILLARS; boardcolor=BROWN;
	     set_power_up(77,34,ARMOR_PILL,5);
	     set_power_up(290,20,PIERCING_PILL,15);
	     set_power_up(155,157,REBOUND_PILL,10);
	     init_comp_tank(2,NAIVE,0,0,0);
	     tanks[2].armor=tanks[2].previousarmor=6; /* lessen armor */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(15,15,ARMOR_PILL,2);
	       set_power_up(15,159,ARMOR_PILL,2);
	       init_comp_tank(3,PATROL,300,35,DOWN);
	       tanks[3].armor=tanks[3].previousarmor=6;
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=10;
		if (tanks[3].armor) /* if tank is initialized */
		 tanks[3].armor=tanks[3].previousarmor=8;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
		if (tanks[3].armor) /* if tank is initialized */
		  tanks[3].armor=tanks[3].previousarmor=4;
	      }
	     break;

    case 3:  /* Level 3 */
	     boardnumber=FORTRESS;  boardcolor=OLIVEDRAB;
	     set_power_up(290,100,REBOUND_PILL,10);
	     init_comp_tank(2,TERMINATOR,0,0,0);
	     init_comp_tank(3,PATROL,0,0,0);
	     /* Make the enemy tanks have weak armor */
	     tanks[2].armor=tanks[2].previousarmor=2;
	     tanks[3].armor=tanks[3].previousarmor=5;

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       init_comp_tank(4,PATROL,200,50,DOWN);
	       tanks[4].armor=tanks[4].previousarmor=5; /* lessen armor */
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=3;
	       tanks[3].armor=tanks[3].previousarmor=7;
		if (tanks[4].armor) /* if tank is initialized */
		 tanks[4].armor=tanks[4].previousarmor=6;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[3].armor=tanks[3].previousarmor=3;
	       set_power_up(30,20,ARMOR_PILL,3);
		if (tanks[4].armor) /* if tank is initialized */
		 tanks[4].armor=tanks[4].previousarmor=3;
	      }
	     break;

    case 4:  /* Level 4  vs. 2 tanks */
	     boardnumber=RIVER;   boardcolor=BROWN;
	     set_power_up(30,20,ARMOR_PILL,5);
	     set_power_up(290,20,PIERCING_PILL,25);
	     set_power_up(290,150,REBOUND_PILL,5);
	     init_comp_tank(2,NAIVE,0,0,0);
	     init_comp_tank(3,NAIVE,0,0,0);
	     /* Make the enemy tanks have weak armor */
	     tanks[2].armor=tanks[2].previousarmor=7;
	     tanks[3].armor=tanks[3].previousarmor=7;

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       init_comp_tank(4,PATROL,300,100,LEFT);
	       set_power_up(30,150,ARMOR_PILL,5);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=10;
	       tanks[3].armor=tanks[3].previousarmor=10;
		if (tanks[4].armor) /* if tank is initialized */
		 tanks[4].armor=tanks[4].previousarmor=12;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=5;
	       set_power_up(70,80,ARMOR_PILL,3);
	      }
	     break;

    case 5:  /* Level 5 */
	     boardnumber=CITY2; boardcolor=LIGHTGRAY;
	     set_power_up(220,100,ARMOR_PILL,3);
	     set_power_up(290,20,REBOUND_PILL,15);
	     set_power_up(275,155,PIERCING_PILL,20);
	     set_power_up(255,155,ARMOR_PILL,1);
	     init_comp_tank(2,NAIVE,300,21,DOWN);
	     init_comp_tank(3,NAIVE,300,147,UP);
	     /* Make the enemy tanks have weak armor */
	     tanks[2].armor=tanks[2].previousarmor=5;
	     tanks[3].armor=tanks[3].previousarmor=5;

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       init_comp_tank(4,PATROL,305,100,LEFT);
	       tanks[4].armor=tanks[4].previousarmor=6; /* weaken armor */
	       set_power_up(15,15,ARMOR_PILL,4);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=10;
	       tanks[3].armor=tanks[3].previousarmor=10;
		if (tanks[4].armor) /* if tank is initialized */
		 tanks[4].armor=tanks[4].previousarmor=10;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=4;
		if (tanks[4].armor) /* if tank is initialized */
		  tanks[4].armor=tanks[4].previousarmor=4;
	      }
	     break;

    case 6:  /* Level 6---Invisible tank level */
	     boardnumber=PILLARS;  boardcolor=BROWN;
	     tanks[0].xcor=14; tanks[0].ycor=103; tanks[0].pos=UP;
	     tanks[1].xcor=14; tanks[1].ycor=63; tanks[1].pos=DOWN;
	     init_comp_tank(2,PATROL,300,15,DOWN);
	     tanks[2].armor=tanks[2].previousarmor=5; /* weaken armor */
	     set_power_up(15,150,REBOUND_PILL,8);
	     set_power_up(15,15,ARMOR_PILL,2);

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(100,15,REBOUND_PILL,7);
	       init_comp_tank(3,PATROL,300,150,UP);
	       tanks[3].armor=tanks[3].previousarmor=5;
	       set_power_up(305,83,ARMOR_PILL,2);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=8;
		if (tanks[3].armor) /* if tank is initialized */
		 tanks[3].armor=tanks[3].previousarmor=8;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
		if (tanks[3].armor) /* if tank is initialized */
		 tanks[3].armor=tanks[3].previousarmor=4;
	      }
	     break;

    case 7:  boardnumber=PALACE;    boardcolor=LIGHTGRAY;
	     set_power_up(290,20,REBOUND_PILL,10);
	     set_power_up(290,150,ARMOR_PILL,5);
	     set_power_up(15,15,PIERCING_PILL,30);
	     init_comp_tank(2,NAIVE,0,0,0);
	     init_comp_tank(3,PATROL,150,21,DOWN);
	     init_comp_tank(4,PATROL,150,147,UP);
	     tanks[2].armor=tanks[2].previousarmor=4; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	     tanks[4].armor=tanks[4].previousarmor=6; /* lessen armor */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(15,150,ARMOR_PILL,4);
	       init_comp_tank(5,PATROL,305,150,LEFT);
	       tanks[5].armor=tanks[5].previousarmor=6; /* lessen armor */
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
		if (!tanks[5].armor) /* if tank is notinitialized */
		 {
		 init_comp_tank(5,PATROL,305,150,LEFT);
		 tanks[5].armor=tanks[5].previousarmor=6; /* lessen armor */
		 }
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=3;
	       tanks[3].armor=tanks[3].previousarmor=4;
	       tanks[4].armor=tanks[4].previousarmor=4;
		if (tanks[5].armor) /* if tank is notinitialized */
		  tanks[5].armor=tanks[5].previousarmor=4;
	      }
	     break;

    case 8:  boardnumber=SPLITEXIT;    boardcolor=OLIVEDRAB;
	     set_power_up(250,20,PIERCING_PILL,10);
	     set_power_up(50,105,PIERCING_PILL,9);
	     set_power_up(299,128,ARMOR_PILL,2); /* inside level box */
	     set_power_up(15,15,ARMOR_PILL,3);
	     tanks[0].xcor=12; tanks[0].ycor=83; tanks[0].pos=RIGHT;
	     tanks[1].xcor=12; tanks[1].ycor=63; tanks[1].pos=RIGHT;
	     init_comp_tank(2,PATROL,250,150,LEFT);
	     init_comp_tank(3,NAIVE,150,18,DOWN);
	     init_comp_tank(4,PATROL,150,150,UP);
	     tanks[2].armor=tanks[2].previousarmor=6; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	     tanks[4].armor=tanks[4].previousarmor=7; /* lessen armor */

	     /* This next tank is hidden in upper left corner of board */
	     /* you can hit him with PIERCING rounds! (or w/normals) */
	      init_comp_tank(5,DUMMY,0,0,LEFT); /* a decoy */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(15,150,ARMOR_PILL,3);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       /* Make tanks stronger armor power */
	       tanks[2].armor=tanks[2].previousarmor=10;
	       tanks[3].armor=tanks[3].previousarmor=10;
	       tanks[4].armor=tanks[4].previousarmor=10;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=4;
	       tanks[4].armor=tanks[4].previousarmor=5;
	      }
	     break;

    case 9:  /* The Snake run booby trap stage */
	     boardnumber=SNAKEBRD; boardcolor=OLIVEDRAB;
	     tanks[0].xcor=12; tanks[0].ycor=144; tanks[0].pos=RIGHT;
	     tanks[1].xcor=12; tanks[1].ycor=157; tanks[1].pos=RIGHT;
	     set_power_up(300,150,PIERCING_PILL,18);
	     set_power_up(300,100,PIERCING_PILL,18);
	     set_power_up(240,143,ARMOR_PILL,5);
	     init_comp_tank(2,NAIVE,40,75,DOWN);
	     init_comp_tank(3,PATROL,40,35,DOWN);
	     tanks[2].armor=tanks[2].previousarmor=6; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	     /* make next two dummies cuz they don't get activated */
	     /* till a player hits the booby trap */
	     init_comp_tank(4,DUMMY,210,115,UP);
	     init_comp_tank(5,DUMMY,240,115,UP);
	     tanks[4].armor=tanks[4].previousarmor=4; /* lessen armor */
	     tanks[5].armor=tanks[5].previousarmor=4; /* lessen armor */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       /* reinit the comptank to NAIVE */
	       init_comp_tank(3,NAIVE,40,35,DOWN);
	       tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	       set_power_up(220,157,ARMOR_PILL,5);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       /* Make tanks stronger armor power */
	       tanks[2].armor=tanks[2].previousarmor=8;
	       tanks[3].armor=tanks[3].previousarmor=8;
	       tanks[4].armor=tanks[4].previousarmor=6;
	       tanks[5].armor=tanks[5].previousarmor=6;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=4;
	       tanks[4].armor=tanks[4].previousarmor=3;
	       tanks[5].armor=tanks[5].previousarmor=3;
	      }
	     break;

    case 10:  /* The Multi-teleport board with invisi tanks */
	     boardnumber=MULTI_TELEPORT_BRD; boardcolor=LIGHTGRAY;
	     tanks[0].xcor=12; tanks[0].ycor=40; tanks[0].pos=RIGHT;
	     tanks[1].xcor=12; tanks[1].ycor=133; tanks[1].pos=RIGHT;
	     set_power_up(266,40,ARMOR_PILL,4);
	     set_power_up(266,129,ARMOR_PILL,4);
	     set_power_up(15,15,ARMOR_PILL,3);
	     set_power_up(227,80,REBOUND_PILL,22);
	     init_comp_tank(2,PATROL,300,25,DOWN);
	     init_comp_tank(3,PATROL,200,50,DOWN);
	     init_comp_tank(4,PATROL,200,100,UP);
	     tanks[2].armor=tanks[2].previousarmor=5; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=4; /* lessen armor */
	     tanks[4].armor=tanks[4].previousarmor=4; /* lessen armor */

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(15,152,ARMOR_PILL,3);
	       init_comp_tank(5,PATROL,300,150,UP);
	       tanks[5].armor=tanks[5].previousarmor=5; /* lessen armor */
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       tanks[3].armor=tanks[3].previousarmor=6;
	       tanks[4].armor=tanks[4].previousarmor=6;
		if (tanks[5].armor) /* if tank is initialized */
		  tanks[5].armor=tanks[5].previousarmor=7;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       set_power_up(15,83,ARMOR_PILL,3);
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=3;
	       tanks[4].armor=tanks[4].previousarmor=3;
		if (tanks[5].armor) /* if tank is initialized */
		  tanks[5].armor=tanks[5].previousarmor=3;
	      }
	     break;

     case 11 :  /* tank yard board */
	     boardnumber=TANK_YARD; boardcolor=BROWN;
	     tanks[0].xcor=12; tanks[0].ycor=144; tanks[0].pos=RIGHT;
	     tanks[1].xcor=12; tanks[1].ycor=157; tanks[1].pos=RIGHT;
	     set_power_up(310,10,ARMOR_PILL,4);
	     set_power_up(310,159,ARMOR_PILL,4);
	     set_power_up(110,52,PIERCING_PILL,5);
	     set_power_up(100,110,ARMOR_PILL,4);
	     set_power_up(267,80,ARMOR_PILL,5);
	     /* make tanks 2&3 DUMMIES that come alive later */
	     init_comp_tank(2,DUMMY,144,128,UP);
	     init_comp_tank(3,DUMMY,122,31,DOWN);
	     init_comp_tank(4,PATROL,300,25,DOWN);
	     tanks[2].armor=tanks[2].previousarmor=6; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	     tanks[4].armor=tanks[4].previousarmor=6; /* lessen armor */
	     /* Make 5 a hidden tank! in upper right building */
	     init_comp_tank(5,NAIVE2,267,56,DOWN);
	     /* set tank[5] to NOTANK cuz it is hidden in the wall */
	     tanks[5].tanktype=NOTANK;

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       set_power_up(15,15,ARMOR_PILL,3);
	       /* Make tanks stronger armor power */
	       tanks[2].armor=tanks[2].previousarmor=8;
	       tanks[3].armor=tanks[3].previousarmor=8;
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       /* Make tanks stronger armor power */
	       tanks[2].armor=tanks[2].previousarmor=8;
	       tanks[3].armor=tanks[3].previousarmor=8;
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4; /* lessen armor */
	       tanks[3].armor=tanks[3].previousarmor=4; /* lessen armor */
	       tanks[4].armor=tanks[4].previousarmor=4; /* lessen armor */
	      }
	     break;

     case 12 :  /* Hard as hell fortress board */
	     boardnumber=FORTRESS; boardcolor=BROWN;
	     set_power_up(310,10,ARMOR_PILL,4);
	     set_power_up(112,150,ARMOR_PILL,4);
	     set_power_up(110,52,PIERCING_PILL,5);
	     set_power_up(80,100,ARMOR_PILL,4);
	     set_power_up(267,80,ARMOR_PILL,5);

	     init_comp_tank(2,NAIVE2,240,12,DOWN);
	     init_comp_tank(3,NAIVE,300,150,UP);
	     init_comp_tank(4,PATROL,285,83,LEFT);
	     tanks[2].armor=tanks[2].previousarmor=4; /* lessen armor */
	     tanks[3].armor=tanks[3].previousarmor=6; /* lessen armor */
	     tanks[4].armor=tanks[4].previousarmor=6; /* lessen armor */
	     /* now make circle tank w/right colors */
	     tanks[4].tanktype=CIRCLETANK;
	     tanks[4].colors[0]=tanks[5].colors[1]=CIRCLE_CLR;

	     /* Check to see if 2 players are playing */
	     if (two_players_playing())
	      {
	       init_comp_tank(5,PATROL,300,140,UP);
	       tanks[5].armor=tanks[5].previousarmor=6; /* lessen armor */
	       /* Make tank a circle tank */
	       tanks[5].tanktype=CIRCLETANK;
	       tanks[5].colors[0]=tanks[5].colors[1]=CIRCLE_CLR;
	       set_power_up(15,23,ARMOR_PILL,3);
	      }
	     /* Now check difficulty level, adjust accordingly */
	     if (difficulty_is==HARDDIFF)
	      {
	       /* reinit the 3rd comptank to TERMINATOR */
	       init_comp_tank(3,TERMINATOR,40,35,DOWN);
	       tanks[3].armor=tanks[3].previousarmor=4; /* make weak */
	      }
	     else if (difficulty_is==EASYDIFF)
	      {
		set_power_up(15,150,ARMOR_PILL,4);
	      }
	     break;

     case 13 :  /* THE FINAL BOARD! */
	     boardnumber=THE_13TH; boardcolor=LIGHTGRAY;

	     /* Set the number of regenerating tanks */
	     if (difficulty_is==EASYDIFF)
	       foes_on_13th=2;
	     else if (two_players_playing())
	       foes_on_13th=6;
	     else  /* average difficulty or hard w/one player */
	       foes_on_13th=4;

	     tanks[0].xcor=27; tanks[0].ycor=91; tanks[0].pos=RIGHT;
	     tanks[1].xcor=27; tanks[1].ycor=75; tanks[1].pos=RIGHT;
	     set_power_up(298,10,ARMOR_PILL,4);
	     set_power_up(298,158,ARMOR_PILL,4);
	     init_comp_tank(2,NAIVE,290,25,LEFT);
	     init_comp_tank(3,PATROL,290,60,LEFT);
	     init_comp_tank(4,TERMINATOR,290,100,LEFT);
	     init_comp_tank(5,NAIVE,290,145,LEFT);

	     /* Make their armor a little lesser cuz of the */
	     /* many foes the player(s) will be facing */
	     if (difficulty_is < HARDDIFF)
	      {
	       tanks[2].armor=tanks[2].previousarmor=4;
	       tanks[3].armor=tanks[3].previousarmor=4;
	       tanks[4].armor=tanks[4].previousarmor=3;
	       tanks[5].armor=tanks[5].previousarmor=4;
	      }
	     break;
   }
}
/**************************************************************************/
void set_up_board (void)
/* This function is called if the current gametype is a dual or a Null */
/* Modem Match.  It does not initialize any computer tanks.            */
{

 if (gametype==MATCH) /* We must check some stuff */
   {
     /* Check to see if RANDOM_BOARD was picked */
     if (boardnumber==RANDOM_BOARD)
      {
       /* Check for secret code to play duel on 13th level!!!! */
       if (get_key_state(KEY_T))
	boardnumber=THE_13TH;
       else
	boardnumber = rand()%7;
      }
     /* SEE if Night Fight was picked */
     if (nightfight==TRUE)
      {
       /* Load in NIGHTFIGHT palette  :) */
       load_in_night_fight_palette();
       /* Set the palette */
       setpalette(pcx[NIGHT_PCXPAL].palette);
      }
   }

  /* Place tanks at default coordinates */
   set_default_coordinates(0);
   set_default_coordinates(1);

  /* Now set up the board for a old fashion shoot em up! */
  switch (boardnumber)
   {
     case VOID :
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	       set_power_up(160,83,ARMOR_PILL,2);
	     break;
     case CITY :
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(240,20,REBOUND_PILL,10);
	       set_power_up(240,150,ARMOR_PILL,4);
	       set_power_up(80,20,ARMOR_PILL,4);
	       set_power_up(80,150,REBOUND_PILL,10);
	       set_power_up(160,83,PIERCING_PILL,30);
	      }
	     break;
     case PILLARS :
	     boardcolor=BROWN; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(20,20,PIERCING_PILL,15);
	       set_power_up(300,150,PIERCING_PILL,15);
	       set_power_up(300,20,ARMOR_PILL,2);
	       set_power_up(20,150,ARMOR_PILL,2);
	       set_power_up(160,120,REBOUND_PILL,10);
	       set_power_up(160,50,REBOUND_PILL,10);
	       set_power_up(160,83,ARMOR_PILL,4);
	      }
	     break;
     case FORTRESS :
	     boardcolor=OLIVEDRAB; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(135,83,ARMOR_PILL,8); /* embedded */
	       set_power_up(115,13,REBOUND_PILL,10);
	       set_power_up(175,157,REBOUND_PILL,10);
	       set_power_up(159,13,PIERCING_PILL,30);
	       set_power_up(115,157,PIERCING_PILL,30);
	      }
	     break;
     case RIVER :
	     boardcolor=BROWN; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(160,83,ARMOR_PILL,2);
	       set_power_up(240,20,PIERCING_PILL,10);
	       set_power_up(240,150,ARMOR_PILL,10);
	       set_power_up(80,20,ARMOR_PILL,10);
	       set_power_up(80,150,PIERCING_PILL,10);
	       set_power_up(13,13,REBOUND_PILL,12);
	       set_power_up(298,157,REBOUND_PILL,12);
	      }
	     break;
     case MOUNTAINS :
	     boardcolor=OLIVEDRAB; /* user picks the board */
	     break;
     case CITY2 :
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(15,15,REBOUND_PILL,25);
	       set_power_up(15,154,ARMOR_PILL,5);
	       set_power_up(305,15,ARMOR_PILL,5);
	       set_power_up(305,154,REBOUND_PILL,25);
	       set_power_up(160,83,REBOUND_PILL,20);
	      }
	     break;
     case PALACE :
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(87,53,REBOUND_PILL,18);
	       set_power_up(87,117,ARMOR_PILL,3);
	       set_power_up(216,53,ARMOR_PILL,3);
	       set_power_up(216,117,REBOUND_PILL,18);
	       set_power_up(150,100,PIERCING_PILL,40);
	      }
	     break;
     case MULTI_TELEPORT_BRD:
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(52,40,REBOUND_PILL,20);
	       set_power_up(52,128,ARMOR_PILL,3);
	       set_power_up(266,40,ARMOR_PILL,3);
	       set_power_up(266,128,REBOUND_PILL,20);
	      }
	     break;

     /* This is a hidden duel board - must press 'T' during RANDOM */
     case THE_13TH:
	     boardcolor=LIGHTGRAY; /* user picks the board */
	     if (powerups_on)
	      {
	       set_power_up(217,83,PIERCING_PILL,20);
	       set_power_up(30,20,REBOUND_PILL,20);
	       set_power_up(30,148,ARMOR_PILL,3);
	       set_power_up(290,20,ARMOR_PILL,3);
	       set_power_up(290,148,REBOUND_PILL,20);
	       set_power_up(145,40,ARMOR_PILL,2);
	       set_power_up(145,120,ARMOR_PILL,2);
	      }
	     break;

     default   :
	     boardnumber=VOID;
	     boardcolor=BROWN; /* user picks the board */
	     break;
   }
}
/**************************************************************************/
void set_default_coordinates (int tanknum)
/* This function sets a tank up to default coordinates and position */
{
 switch (tanknum)
 {
  case 0: if ((gametype==NULL_MODEM_MATCH)&&(current_local_pc_is==RECEIVER))
	   {
	     tanks[tanknum].xcor = STARTXTANK1;
	     tanks[tanknum].ycor = STARTYTANK1;
	     tanks[tanknum].pos  = STARTPOSTANK1;
	   }
	  else
	   {
	    tanks[tanknum].xcor = STARTXTANK0;
	    tanks[tanknum].ycor = STARTYTANK0;
	    tanks[tanknum].pos  = STARTPOSTANK0;
	   }
	  break;

  case 1: if((gametype==NULL_MODEM_MATCH)&&(current_local_pc_is==RECEIVER))
	   {
	     tanks[tanknum].xcor = STARTXTANK0;
	     tanks[tanknum].ycor = STARTYTANK0;
	     tanks[tanknum].pos  = STARTPOSTANK0;
	   }
	  else if (gametype==TWOPLYR_FULLGAME) /* put tank next to plyr 0 */
	   {
	     tanks[tanknum].xcor = STARTXTANK3;
	     tanks[tanknum].ycor = STARTYTANK3;
	     tanks[tanknum].pos  = STARTPOSTANK3;
	   }
	  else
	   {
	     tanks[tanknum].xcor = STARTXTANK1;
	     tanks[tanknum].ycor = STARTYTANK1;
	     tanks[tanknum].pos  = STARTPOSTANK1;
	   }
	  break;

  case 2: tanks[tanknum].xcor = STARTXTANK1;
	  tanks[tanknum].ycor = STARTYTANK1;
	  tanks[tanknum].pos  = STARTPOSTANK1;
	  break;
  case 3: tanks[tanknum].xcor = STARTXTANK2;
	  tanks[tanknum].ycor = STARTYTANK2;
	  tanks[tanknum].pos  = STARTPOSTANK2;
	  break;
 }
}
/**************************************************************************/
void reset_player_tank (void)
/* This function simply clears some of the remaining stats stored in the */
/* player from the previous level. Such as projectile coordinates.       */
{
  int x,y;

  tanks[0].xcor=0;
  tanks[0].ycor=0;
  tanks[0].pos=0;
  tanks[0].cruise=NOCRUISE;
  tanks[0].currentinput=0;
  tanks[0].lastinput=0;
  tanks[0].thrustcount=0;

   for (x=0;x<MAXPROJS;x++)
       {
	  tanks[0].projs[x].dir=0;
	  tanks[0].projs[x].x=0;
	  tanks[0].projs[x].y=0;
	  tanks[0].projs[x].type=0;
	  tanks[0].projs[x].time=0;
       }
}
/**************************************************************************/
void reset_second_player_tank (void)
/* This function simply clears some of the remaining stats stored in the */
/* 2nd player from the previous level. Such as projectile coordinates.   */
{
  int x,y;

  tanks[1].xcor=0;
  tanks[1].ycor=0;
  tanks[1].pos=0;
  tanks[1].cruise=NOCRUISE;
  tanks[1].currentinput=0;
  tanks[1].lastinput=0;
  tanks[1].thrustcount=0;

   for (x=0;x<MAXPROJS;x++)
       {
	  tanks[1].projs[x].dir=0;
	  tanks[1].projs[x].x=0;
	  tanks[1].projs[x].y=0;
	  tanks[1].projs[x].type=0;
	  tanks[1].projs[x].time=0;
       }
}
/**************************************************************************/
void reset_explosion_array(void)
{
 int i;

 for (i=0;i<MAX_EXPLOSIONS;i++)
  {
    explosion_array[i].x=0;
    explosion_array[i].y=0;
    explosion_array[i].type=0;
    explosion_array[i].time=0;
  }
}
/**************************************************************************/
void load_in_night_fight_palette(void)
/* This just makes sure Tankgame has access to night fight's palette */
/* because we will use this dark palette for NIGHT FIGHTS */
{
    /* Load the palette in to memory */
 if (loadPCX_palette_only(pcx_file_names[NIGHT_PCXPAL],&pcx[NIGHT_PCXPAL]))
     {
	printf(" ERROR loading .PCX file #%d\n",NIGHT_PCXPAL);
	pcx_files_loaded=FALSE;
     }
}
/**************************************************************************/
#define _SETBOARD
#endif

