
/* Kirk Christiansen                      */
/* TANKFNCS.C                             */

/* Tank Game Functions                    */
/* These are some miscellaneous functions */
/* used within Tankgame.                  */

#ifndef _TANKFNCS
/***************************************************************************/
#include "tankvars.c"
#include "kc_keybd.c"
#include "tnkprojs.c"
#include "kc_pcsnd.c"
#include "powerups.c"
#include "kcgrphcs.c"   /* for drawboard */
#include "weapons.c"
#include "miscfncs.c"
/**************************************************************************/
/* Function Prototypes */
void introscreen(void);
void printoutstats (void);
void rewritearmor(int plyrnum, int prevarmor, int newarmor);
void calcstats(void);
void update_game(void);
void display_level(void);
void display_ending(void);
void ask_for_play_again(void);
void blit_ending_box(void);
void check_for_special_events (int tnknum); /* huge function :) */
void check_multi_tele_brd_events (int tnkis);
void check_the_13th_events (int tnkis);
int  check_space_for_no_tanks (int tnkis, int x, int y);
int  two_players_playing(void);
void check_for_timed_events (void);
int  comptank_space_exists(void);
void special_comptank_init(int num,int x,int y,int amode,int ttype,int arm);
/***************************************************************************/
void introscreen(void)
/* This function draws the intro screen before the game begins */
{
  int low_memory=FALSE;

  /* See if there is enough memory for my pict */
   if((farcoreleft()) < 64000)
     {
      sound(200);delay(5);nosound();
      low_memory=TRUE;
      /* Low mem so lets make room by deleting Main Menu pict for a sec */
      farfree(pcx[TANKMENU].image);
     }

  /* open the pcx file image */
   if (loadPCX(pcx_file_names[TNKINTRO],&pcx[TNKINTRO]))
      {
	printf(" ERROR loading .DAT file #%d\n",TNKINTRO);
	sound(300);delay(100);nosound();delay(3000);
	pcx_files_loaded=FALSE;
	return;
      }

  /* Move the pcx intro image to the screen */
   putwindow(0,0,320,200,FP_OFF(pcx[TNKINTRO].image),FP_SEG(pcx[1].image));

  /* Wait for a keypress, joystick1 button press, or time delay */
   do
   {
    /* Read in the joystick input for Joystick A (0) */
     read_joystick (0, &joystick1);
   }
   while((!keypress()) && (!joystick1.buttonA) && (!joystick1.buttonB) );

  /* reset the keys array just incase there was a keypress */
   reset_keys_array();

  /* Release memory for PCX image */
  farfree(pcx[TNKINTRO].image);

  /* Now check if we have low memory, if so we need to reload Main Menu */
  if (low_memory)
   {
    loadPCX(pcx_file_names[TANKMENU],&pcx[TANKMENU]);
   }
}
/***************************************************************************/
void printoutstats (void)
/* This function prints out the current level if in FULLGAME mode and */
/* also prints out the initial starting armor score for each tank.    */
/* This function is usually invoked at the beginning of a level.      */
{
  int num1=0;   /* Tens position */
  int num2=0;   /* Ones position */
  int i;

  /* Print out the level if FULLGAME mode or TWOPLYR_FULLGAME */
 if ((gametype==FULLGAME) || (gametype==TWOPLYR_FULLGAME))
 {
      if (level < 10)
	 num2=level;
      else
	{
	  num1=level/10;   /* So level 10 would yield a 1 for num1    */
	  num2=level%10;   /*  and level 13 would yield a 3 for num2. */
	}

    /* Next, print out the level number using num1 and num2 */
      if (gametype==FULLGAME)
       {
	  color_blit(72,187,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
		  screen_buffer,character_picts[num1]);
	  color_blit(72+6,187,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
		  screen_buffer,character_picts[num2]);
	/* Print out the players current weapon */
	 display_weapon_and_ammo(0);
	/* Write out initial tank 0 and 1's armor ,input a high number  */
	/* (1000) to guarantee the armor will be printed due to a change.  */
	 rewritearmor (0,1000,tanks[0].armor);
       }
      else /* gametype must be TWOPLYR_FULLGAME to be in this loop */
       {
	  color_blit(154,185,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
		  screen_buffer,character_picts[num1]);
	  color_blit(154+6,185,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
		  screen_buffer,character_picts[num2]);
       /* Print out the players current weapon */
	   display_weapon_and_ammo(0);
	   display_weapon_and_ammo(1);
	/* Write out initial tank 0 and 1's armor ,input a high number  */
	/* (1000) to guarantee the armor will be printed due to a change.  */
	   rewritearmor (0,1000,tanks[0].armor);
	   rewritearmor (1,1000,tanks[1].armor);
       }
       /* Write out the weapon abbreviations the player has */
	//display_weapon_abbreviations();
 }

 else /* must be in MATCH or NULL_MODEM_MATCH gametype */
 {
       /* Print out the players current weapon */
	if (display_stats)
	 {
	   display_weapon_and_ammo(0);
	   display_weapon_and_ammo(1);
	 }
	/* Write out initial tank 0 and 1's armor ,input a high number  */
	/* (1000) to guarantee the armor will be printed due to a change.  */
	if (display_stats)
	 {
	  rewritearmor (0,1000,tanks[0].armor);
	  rewritearmor (1,1000,tanks[1].armor);
	 }
       /* Write out the weapon abbreviations the player has */
	//display_weapon_abbreviations();
	//display_weapon_abbreviations();
 }
}
/***************************************************************************/
void rewritearmor(int plyrnum, int prevarmor, int newarmor)
/* This function erases the old armor score if the player has lost or */
/* gained armor and writes out the new armor score in its place.      */
{
  int i;
  int x;           /* Coordinates to write the armor score */
  int displaycolor;  /* This variable stores the //outtext color */
  int currentarmor;  /* This variable stores what armor is being */
		     /* printed to the screen, the old armor score */
		     /* (being erased), or the new armor score. */
  int num1=0;        /* Tens position */
  int num2=0;        /* Ones position */

  /* If there has been damage or an increase in armor */
  if (newarmor != prevarmor)
   {
    /* Set the correct position  */
      if (plyrnum==0)
       {
	 if (gametype==FULLGAME)
	   x=72;
	 else
	   x=43;
       }
      else
       {
	 x=218;
       }

    /* Calculate the old numbers to erase */
    num1=prevarmor / 10;     /* Ten's position */
    num2=prevarmor % 10;     /* One's position */

    /* Erase old armor values, use BLACK */
    color_blit(x,173,CHARPICTWIDTH,CHARPICTHEIGHT,BLACK,
	      screen_buffer,character_picts[num1]);
    color_blit(x+6,173,CHARPICTWIDTH,CHARPICTHEIGHT,BLACK,
	      screen_buffer,character_picts[num2]);

    /* Calculate the new numbers to print out */
    num1=newarmor / 10;    /* Ten's position */
    num2=newarmor % 10;    /* One's position */

    /* Write out the new armor value (use standard OLIVEDRAB) */
    color_blit(x,173,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
	      screen_buffer,character_picts[num1]);
    color_blit(x+6,173,CHARPICTWIDTH,CHARPICTHEIGHT,OLIVEDRAB,
	      screen_buffer,character_picts[num2]);
   };
};
/***************************************************************************/
void calcstats(void)
/* This function is called once every cycle of the main game loop. */
/* It checks all tanks to see if a tank has lost or gained armor,  */
/* and it also checks to see if a tank has lost all its armor.     */
{
 int i,j;
 int weapon;


 for (i=0;i<MAXTANKS;i++)
  {
   if (tanks[i].tankstats[GAMESTATUS])  /* tank is playing this level */
   {
     if ((tanks[i].armor) != (tanks[i].previousarmor)) /* if armor + or - */
      {
	    /* print out new armor score to screen if a player*/
	if ((tanks[i].playertype==HUMAN)&&(display_stats))
	 {
	  rewritearmor(i,tanks[i].previousarmor,tanks[i].armor);
	 }
	    /* update previousarmor value */
	 tanks[i].previousarmor = tanks[i].armor;

	   /* Set explosion if tank is destroyed */
	  if (tanks[i].armor == 0)
	   {
	     if (soundon)
	      make_sound(BIG_EXPLOSION);

	     set_explosion(tanks[i].xcor,tanks[i].ycor,TANK_EXPLOSION);

	     draw_blownuptank(tanks[i].xcor,tanks[i].ycor,
			      tanks[i].colors[0]);
	   }
      } /* end if armor has changed. */

      /* Set weapon to current weapon tank is using */
      weapon = tanks[i].current_weapon;

      if (tanks[i].previous_ammo != tanks[i].weapon_ammo[weapon])
      {
	 if ((tanks[i].playertype==HUMAN) && (display_stats))
	  {
	   /* erase old ammo score */
	   display_ammo(i,weapon,tanks[i].previous_ammo,BLACK);
	   /* show new ammo score */
	   display_ammo(i,weapon,tanks[i].weapon_ammo[weapon],OLIVEDRAB);
	  }

	 /* update new ammo value */
	 tanks[i].previous_ammo=tanks[i].weapon_ammo[weapon];
      }

   } /* end if tank is playing this level */
  }
}
/***************************************************************************/
void update_game(void)
/* This function simply checks to see if any tanks have been destroyed */
/* and also checks to see if the user has cleared the current level.   */
{
  int i;
  int alive_count=0; /* Counts how many tanks are still alive */
  int enemy_count=0; /* computer tanks left */

  /* Check the tanks for recently destroyed tanks */
  for (i=0;i<MAXTANKS;i++)
  {
    if (tanks[i].tankstats[GAMESTATUS]==PLAYING)
     {
       if (tanks[i].armor < 1)  /* If a tank has been destroyed */
	{
	   if (gametype==FULLGAME)
	    {
	      if (i==0) /* player has died.....game over :( */
	       {
		tanks[i].tankstats[GAMESTATUS] = JUSTKILLED; /* set status */
		erase_a_tanks_projectiles(i);
		quit_game=TRUE;
	       }
	      else /* computer tank is wasted */
	       {
		erase_a_tanks_projectiles(i);
		reset_tank(i);           /* reset the tank */
		tanks[i].tankstats[GAMESTATUS] = JUSTKILLED; /* set status */
	       }
	    }
	   else if ((gametype==MATCH) || (gametype==NULL_MODEM_MATCH))
	    { /* GAME MUST BE OVER! */
	      erase_a_tanks_projectiles(i);
	      quit_game=TRUE;
	    }
	   else if (gametype==TWOPLYR_FULLGAME)
	    {
	      if (i<2) /* player has died.....game over :( */
	       {
		tanks[i].tankstats[GAMESTATUS]=JUSTKILLED; /* set status */
		erase_a_tanks_projectiles(i);
		/* see if other player is dead */
		if (tanks[!i].tankstats[GAMESTATUS]!=PLAYING)
		  quit_game=TRUE;
	       }
	      else /* computer tank is wasted */
	       {
		erase_a_tanks_projectiles(i);
		reset_tank(i);             /* reset the tank */
		tanks[i].tankstats[GAMESTATUS] = JUSTKILLED; /* set status */
	       }
	    }
	} /* end if (armor<1) */
       else
	 {
	  alive_count++; /* add 1 to the number of tanks alive */
	  if (i>1) enemy_count++; /* add 1 to the number of cmptr tanks */
	 }
     } /* end if tank is PLAYING */
  } /* end for loop */

   /* If game is one match, and not 2 tanks are left, then its over */
  if ((alive_count < 2)&&((gametype==MATCH)||(gametype==NULL_MODEM_MATCH)))
     {
      quit_game=TRUE;
     }

   /* If in FULLGAME mode, and player has eliminated all the other */
   /* tanks, update the level. Level 9 you must solve              */
   else if ( (enemy_count < 1) && (gametype==FULLGAME)
	     && (level!=9))
     {
       level++;
     }

   /* If in TWOPLYR_FULLGAME mode, and players have eliminated all */
   /* the other tanks, update the level.                           */
   else if ((enemy_count < 1) && (gametype==TWOPLYR_FULLGAME)
	     && (level!=9))
     {
       level++;
     }
   /* If in FULLMATCH mode, player has lost */
   /* NOTE:                                                       */
   /* Need to add inclusions if a computer can fight two computer */
   /* opponents, or if two Humans can fight each other            */


  /* FINALLY, update the game_loop_cntr */
   game_loop_cntr++;
}
/***************************************************************************/
void display_level(void)
/* This function is called between level changes.  It simply draws */
/* out to a black screen what level the user is about to play.     */
{
  int num1,num2;

    num1=level / 10;
    num2=level % 10;

    /* First, clear the screen */
    clear_screen(screen_buffer);

    /* Show the blank screen */
     putwindow(0,0,320,200,FP_OFF(screen_buffer),FP_SEG(screen_buffer));

    /* Short delay */
     delay (500);

    /* First print out big LEVEL */
    blit (100,50,105,29,screen_buffer,stat_picts[2]);

    /* Now print out the actual level */
    blit (125,90,19,29,screen_buffer,stat_picts[3+num1]);
    blit (150,90,19,29,screen_buffer,stat_picts[3+num2]);

   /* if level==5 and user didn't jump here, tell them about jump code */
   if ((level==5)&&(!level_5_jmp_entered))
    {
      out_text("TO ADVANCE STRAIGHT TO LEVEL 5",53,140,RED,screen_buffer);
      out_text("IN FUTURE GAMES PRESS THE J AND",53,152,RED,screen_buffer);
      out_text("M AND P KEYS SIMULTANEOUSLY",53,164,RED,screen_buffer);
      putpixel(242,171,RED,screen_buffer);
    }

   /* Move the buffer to the screen, so user can see it */
   putwindow(0,0,320,200,FP_OFF(screen_buffer),FP_SEG(screen_buffer));

   if ((level > 0) && (soundon))
    make_sound(LEVEL_CLEAR);

    /* Delay, so user can read the screen */
   delay (1750);

   /*  Extra delay if on level 5 to read jump info */
   if ((level==5)&&(!level_5_jmp_entered))
    delay(3000);
}
/***************************************************************************/
void display_ending(void)
/* This function prints out the winner of the game if the game has  */
/* ended.  In MATCH mode it displays the tank number of the winner. */
{
  int x1 = 105;
  int y1 = 70;
  int y2 = 85;
  int y3 = 100;

   delay (800); /* pause between end of game and display */

   if (gametype==FULLGAME)
     {
       if (tanks[0].armor <= 0)
	{
	 blit_ending_box();
	 /* Blit 'GAME OVER' */
	 blit (x1,y2,103,16,screen,stat_picts[25]);
	 /* play some defeat music */
	 if (soundon)
	   make_sound(SADMUSIC);

	 delay(1500);
	}
       else if ((tanks[0].armor > 0)&&(level>MAXLEVELS))
	{
	 blit_ending_box();
	 /* Blit 'You did it!'*/
	 blit (x1+5,y2,94,16,screen,stat_picts[26]);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(LEVEL_CLEAR);
	 delay(1400);
	 /* Draw the (you won/game over) box */
	 blit_ending_box();
	 /* Blit 'You beat' */
	 blit (x1+15,y1,69,14,screen,stat_picts[31]);
	 /* Blit 'Tankgame' */
	 blit (x1+13,y3,74,14,screen,stat_picts[32]);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(LEVEL_CLEAR);
	 delay(1400);
	 /* show the winning code! (if not on EASY) */
	 blit_ending_box();
	 out_text ("THE WINNING CODE IS",95,80,YELLOW,screen);
	 if (difficulty_is != EASYDIFF)
	   out_text (SECRETCODE,140,100,YELLOW,screen);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(WINMUSIC);
	 delay(4000);
	}
      }
   else if (gametype==TWOPLYR_FULLGAME)
     {
       if ((tanks[0].armor <= 0) && (tanks[1].armor <= 0))
	{
	 blit_ending_box();
	 /* Blit 'GAME OVER' */
	 blit (x1,y2,103,16,screen,stat_picts[25]);
	 /* play some defeat music */
	 if (soundon)
	   make_sound(SADMUSIC);

	 delay(1500);
	}
       else if (((tanks[0].armor > 0) || (tanks[1].armor > 0))&&
		  level>MAXLEVELS)
	{
	 blit_ending_box();
	 /* Blit 'You did it!'*/
	 blit (x1+5,y2,94,16,screen,stat_picts[26]);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(LEVEL_CLEAR);
	 delay(1400);
	 /* Draw the (you won/game over) box */
	 blit_ending_box();
	 /* Blit 'You beat' */
	 blit (x1+15,y1,69,14,screen,stat_picts[31]);
	 /* Blit 'Tankgame' */
	 blit (x1+13,y3,74,14,screen,stat_picts[32]);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(LEVEL_CLEAR);
	 delay(1400);
	 /* show the winning code! (if not on EASY) */
	 blit_ending_box();
	 out_text ("THE WINNING CODE IS",95,80,YELLOW,screen);
	 if (difficulty_is != EASYDIFF)
	  out_text (SECRETCODE,140,100,YELLOW,screen);
	 /* Play some happy music */
	 if (soundon)
	   make_sound(WINMUSIC);
	 delay(4000); /* delay so user can read secret code */
	}
      }
   else /* Game is in MATCH mode */
     {
       if ((tanks[0].armor > 0) && (tanks[1].armor <= 0))
	 {
	  blit_ending_box();
	  /* Blit 'Player'*/
	  blit (x1+8,y1,70,16,screen,stat_picts[28]);
	  /* Blit '1' */
	  blit (x1+80,y1,15,16,screen,stat_picts[29]);
	  /* Blit 'WINS' */
	  blit (x1+23,y3,47,16,screen,stat_picts[27]);
	  /* need music */
	  delay(2000);
	 }
       else if ((tanks[1].armor > 0) && (tanks[0].armor <= 0))
	 {
	  blit_ending_box();
	  /* Blit 'Player'*/
	  blit (x1+8,y1,70,16,screen,stat_picts[28]);
	  /* Blit '2' */
	  blit (x1+80,y1,15,16,screen,stat_picts[30]);
	  /* Blit 'WINS' */
	  blit (x1+23,y3,47,16,screen,stat_picts[27]);
	  delay(2000);
	 }
       else if ((tanks[0].armor) == (tanks[1].armor))
	  {
	    blit_ending_box();
	   /* Blit 'DRAW' */
	   blit (x1+22,y2,54,14,screen,stat_picts[33]);
	   delay(2000);
	  }
     }
}
/***************************************************************************/
void ask_for_play_again(void)
/* This function brings up a small box and asks if you want to play again */
{
  int quit_play_again_loop=FALSE;

  /* First reset the keys array */
  reset_keys_array();

  /* blit box on screen */
  blit_ending_box();

  /* show text */
  out_text (" PLAY AGAIN ",115,80,YELLOW,screen);
  out_text ("   Y OR N",115,92,YELLOW,screen);
  /* draw question mark */

  /* Now, wait for a keypress loop */
  do
  {
    if (keypress())
     {
	/* Check input */
       if (get_and_reset_key_state(KEY_Y))
	{
	  quit_play_again_loop=TRUE;
	  quit_game=FALSE; /* reset the quit_game flag */
	}
       else if (get_and_reset_key_state(KEY_N))
	{
	  quit_play_again_loop=TRUE;
	  quit_entire_program=TRUE; /* set quit entire program flag */
	}
       else
	{
	 reset_keys_array();

	  if (soundon) /* make error sound */
	   {sound(200);delay(20);nosound();}
	}
      }
  }
  while (!quit_play_again_loop);
}
/***************************************************************************/
void blit_ending_box(void)
/* This function simply draws the ending box in the middle of the screen */
{
   /* draw the (you won/game over) box */
   bar_blit(80,60,161,66,BLUE,screen);
   /* outline that box */
   rect(79,59,241,126,WHITE,screen);
   rect(78,58,242,127,WHITE,screen);
}
/***************************************************************************/
void check_for_special_events (int tnknum)
/* This function checks for special events like teleporting within */
/* certain boards and such.  Also power pill pickups.              */
/* This function is called from TANKIO.C */
/* Uses globals : level_9_booby_trap, boarnumber, */
{
 int i,j;
 int check_x,check_y;
 int dieroll;

 /* First check for teleports */
 switch (boardnumber)
 {
  case PALACE :
  {
    /* If the moved tank went into the teleport box on level 7 */
    if (search_tankbox(150,75,tanks[tnknum].xcor,tanks[tnknum].ycor))
      {
	 /* Choose a random corner to teleport to */
	 dieroll=((rand() % 4) +1);

	 /* Set the check_x & check_y stats for that corner*/
	  switch (dieroll)
	  {
	   case (1) :  /* up left corner */
	     check_x = 30; check_y = 17;  break;
	   case (2) :  /* up left corner */
	     check_x = 275; check_y = 17;  break;
	   case (3) :  /* up left corner */
	      check_x = 30; check_y = 151;  break;
	   case (4) :  /* up left corner */
	     check_x = 275; check_y = 151;  break;
	  };

	 /* Now see if that corner is empty */
	 if (check_space_for_no_tanks(tnknum,check_x,check_y))
	   {
	     /* set the new coordinates of the tank */
	     tanks[tnknum].xcor=check_x;
	     tanks[tnknum].ycor=check_y;
	     /* make a noise */
	      if (soundon)
		make_sound(TELEPORT_SND);
	   }
	} /* end if the moved tank is in the teleport area on level 7 */
   break;
  }/* end  case PALACE */

   case  SPLITEXIT :
   {
    if (tnknum > 1) /* a computer tank, so don't check it */
      return;

    /* If the moved tank went into the exit box to level 9 */
    if (search_tankbox(316,55,tanks[tnknum].xcor,tanks[tnknum].ycor))
      {
	/* make a noise */
	 if (soundon)
	   make_sound(LEVEL_WARP);

	/* DO EXIT SEQ IF 2 PLAYER FULL GAME */
	/* SET LEVEL TO 9 HERE */
	level=9;

       } /* end if the moved tank has teleported to level 9 */

    /* If the moved tank went into the exit box to level 11 */
    if (search_tankbox(316,130,tanks[tnknum].xcor,tanks[tnknum].ycor))
      {
	   /* make a noise */
	    if (soundon)
	       make_sound(LEVEL_WARP);

	   /* DO EXIT SEQ IF 2 PLAYER FULL GAME */
	   /* SET LEVEL TO 11 HERE */
	   level=11;

       } /* end if the moved tank has teleported to level 11 */

    break;
   }/* end  case SPLITEXIT */

 case SNAKEBRD :
  {
    if (tnknum > 1) /* a computer tank, so don't check it */
      return;
    /* If the moved tank went into the exit box to level 10 */
    if (search_tankbox(315,18,tanks[tnknum].xcor,tanks[tnknum].ycor))
      {
	/* make a noise */
	 if (soundon)
	   make_sound(LEVEL_WARP);

	/* DO EXIT SEQ IF 2 PLAYER FULL GAME */

	/* SET LEVEL TO 10 HERE */
	level=10;

	/* Set a tank to armor=1 so when we process update_game() it */
	/* doesn't think we just killed all the tanks on level 10. */
	tanks[2].armor=1; tanks[2].tankstats[GAMESTATUS] = PLAYING;
       } /* end if the moved tank has teleported to level 10 */

    /* CHECK to see if a player tank ran over the booby trap! */
    if (!level_9_booby_trap)
    {
     if (search_tankbox(100,18,tanks[tnknum].xcor,tanks[tnknum].ycor))
      {
	/* set trap to TRUE for triggered */
	level_9_booby_trap=TRUE;

	/* make a noise */
	 if (soundon)
	    make_sound (ALARM1);

	/* CLOSE DOOR - to make sure enemies are not there...move em! */
	 for (i=2;i<MAXTANKS;i++)
	   {
	     if ((tanks[i].xcor > 200) && (tanks[i].xcor < 214) &&
		(tanks[i].ycor < 32))
	       {
		 tanks[i].xcor=tanks[i].xcor+17; /* move tank to right */
	       }
	   }

	/* Ok, enemies clear..shut the door */
	 dual_color_blit(200,5,13,29,WALLCOLOR2,
		       WALLCOLOR3,screen_buffer,board_picts[15]);

	/* OPEN ENEMY TANK DOORS w/terminators */
	solid_rect (195,97,220,124,BLACK,screen_buffer); /* the room */
	bar_blit(195,99,3,25,WALLCOLOR3,screen_buffer); /* the 3d shade */
	solid_rect (225,97,250,124,BLACK,screen_buffer); /* the room */
	bar_blit(225,99,3,25,WALLCOLOR3,screen_buffer); /* the 3d shade */
	/* next 4 lines of code shave off new wall corners for 3d look */
	line (196,99,197,99,BLACK,screen_buffer);
	putpixel(197,100,BLACK,screen_buffer);
	line (226,99,227,99,BLACK,screen_buffer);
	putpixel(227,100,BLACK,screen_buffer);
	/* ALSO, OPEN HALLWAY UP THROUGH UPPER HORIZONTAL */
	solid_rect (180,31,201,71,BLACK,screen_buffer);
	bar_blit(180,34,3,34,WALLCOLOR3,screen_buffer);
	putpixel(180,33,WALLCOLOR3,screen_buffer); /* shave off corner */
	putpixel(182,34,BLACK,screen_buffer);   /* with 2 pixel plots */
	/* NOW SET THE TANKS THERE */
	/* Tanks are already there...just need to update their */
	/* heuristics...and there tanktype and colors */
	tanks[4].tanktype=tanks[5].tanktype=HUNTERKILLER;
	tanks[4].attackmode=tanks[5].attackmode=TERMINATOR;
	tanks[4].colors[0]=tanks[5].colors[0]=HKCOLOR;
	tanks[4].colors[1]=tanks[5].colors[1]=HKCOLOR_2;
	/* SET SOME PIERCING PILLS */
	set_power_up(20,145,PIERCING_PILL,12);
	set_power_up(30,150,PIERCING_PILL,12);
       } /* end if the moved tank has triggered the booby trap */
     }
   break;
  } /* end case SNAKEBRD */

  case MULTI_TELEPORT_BRD :
  {
    check_multi_tele_brd_events (tnknum);
    break;
  } /* end case MULTI_TELEPORT_BRD */

  case THE_13TH : /* the 13th and final level */
  {
    check_the_13th_events (tnknum);
    break;
  } /* end case MULTI_TELEPORT_BRD */

 };  /* end switch (boarnumber) */

  /* NOW, check for power up pill pickups */
  check_for_power_up_pickups();
}
/***************************************************************************/
void check_multi_tele_brd_events (int tnkis)
/* This function checks to see if any special events have occurred */
/* on the current level if the level is on MULTI_TELEPORT_BRD.     */
{
  int dieroll;
  int i,check_x,check_y;
  int x,y;
  int teleport=FALSE; /* set to TRUE if tank has entered a TP box */


   /* SEE if a tnkis has moved into any of the teleporters */
   /* stop looking if tank already has drove into one.     */
   for (x=116;((x<200)&&(!teleport));x+=40)
    for (y=71;((y<100)&&(!teleport));y+=18)
     {
      if (search_tankbox(x,y,tanks[tnkis].xcor,tanks[tnkis].ycor))
       {
	 teleport=TRUE;
       }
     }

    if (teleport) /* Tank has entered a TP cell, find a destination */
     {
	 /* Choose a random cell to teleport to */
	 dieroll=(rand() % 6);

	 /* Set the check_x & check_y stats for that corner*/
	  switch (dieroll)
	  {
	   case (0) :  /* up left cell */
	     check_x = 115; check_y = 65-5;  break;
	   case (1) :  /* up mid cell */
	     check_x = 155; check_y = 65-5;  break;
	   case (2) :  /* up right cell */
	      check_x = 195; check_y = 65-5;  break;
	   case (3) :  /* low left cell */
	     check_x = 115; check_y = 98+5;  break;
	   case (4) :  /* mid left cell */
	      check_x = 155; check_y = 98+5;  break;
	   case (5) :  /* low right cell */
	     check_x = 195; check_y = 98+5;  break;
	  };

       /* NOW see if the wanted destination cell is empty */
	 if (check_space_for_no_tanks(tnkis,check_x,check_y))
	   {
	     /* set the new coordinates of the tank */
	     tanks[tnkis].xcor=check_x;
	     tanks[tnkis].ycor=check_y;
	     /* if tank is cruising backwards, stop the tank */
	     if (tanks[tnkis].cruise==BACKWARDCRUISE)
	      {
	       tanks[tnkis].thrustcount=0;
	       tanks[tnkis].cruise=NOCRUISE;
	      }

	     if (dieroll<3) /* uppercell */
	       tanks[tnkis].pos=UP;
	     else /* lowercell */
	       tanks[tnkis].pos=DOWN;

	     /* make a noise */
	      if (soundon)
		make_sound(TELEPORT_SND);
	   }
     } /* end if (teleport) */
}
/***************************************************************************/
void check_the_13th_events (int tnkis)
/* This function checks if any special events have occurred on the */
/* 13th and final level.  */
{
  int check_x,check_y;
  int x,y;
  int teleport=FALSE; /* set to TRUE if tank has entered a TP box */

   /* SEE if a tnkis has moved into any of the teleporters */
   /* search the left one first */
   if (search_tankbox(3,84,tanks[tnkis].xcor,tanks[tnkis].ycor))
    {
      check_x=308; check_y=84;
      teleport=TRUE;
    }

   /* search the right teleport */
   else if (search_tankbox(314,84,tanks[tnkis].xcor,tanks[tnkis].ycor))
    {
      check_x=10; check_y=84;
      teleport=TRUE;
    }

     /* If the tank is in teleport postion, check the destination */
     if (teleport)
      {
       /* NOW see if the wanted destination pathway is empty */
	 if (check_space_for_no_tanks(tnkis,check_x,check_y))
	   {
	     /* erase the tank */
	     erasetank(tnkis);
	     /* set the new coordinates of the tank */
	     tanks[tnkis].xcor=check_x;
	     tanks[tnkis].ycor=check_y;
	     /* make a noise */
	      if (soundon)
		make_sound(TELEPORT_SND);
	   }
     } /* end if (teleport) */
}
/***************************************************************************/
int  check_space_for_no_tanks (int tnkis, int x, int y)
/* This function takes in as input a teleporting tank...and a set */
/* of (x,y) coordinates.....it then determines if the area of (x,y) */
/* is free from all other tanks in the game.....if so it returns TRUE. */
{
  int i;

	 /* Go through all the playing tanks */
	  for (i=0;i<MAXTANKS;i++)
	   {
	    /* Don't check for yourself and make sure others are playing */
	    if ((tnkis != i) && (tanks[i].tankstats[GAMESTATUS]==PLAYING))
	    {
	      /* Now see if the (x,y) space is occupied */
		 if (search_tankbox(x,y,tanks[i].xcor,
		      tanks[i].ycor))
		     {
		       /* not possible */
		       return FALSE;
		     }
	     }
	   } /* end for (i=0;i<MAXTANKS;i++) */

  /* If we got here...it must have been a legal space */
   return TRUE;
}
/***************************************************************************/
int two_players_playing(void)
/* This function simply returns TRUE if the game is TWOPLYR_FULLGAME */
/* and if both players have armor left......ie playing.              */
{
  if ( (gametype==TWOPLYR_FULLGAME) && (tanks[0].armor > 0) &&
       (tanks[1].armor > 0) )
	return TRUE;
  else
	return FALSE;
}
/***************************************************************************/
void check_for_timed_events ()
/* This function simply checks the global variable game_loop_cntr. */
/* It then might perform an action based on this variable's value,  */
/* and the current level. */
{
  int i; /* used for temp variable, to check for tank init space */
  int ttypeis,amodeis,xis,yis;

  /* make sure we need to check time */
  if (game_loop_cntr < 500)
   {
    switch (level)
     {
     case 6 : /* invisible tank level */
	      if (game_loop_cntr == 20)
		{
		 tanks[2].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		 make_sound(WIERDEVENT);
		}
	      else if (game_loop_cntr == 30)
		{
		 if (tanks[3].armor) /* if tank is initialized */
		  {
		   tanks[3].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		   make_sound(WIERDEVENT);
		  }
		}
	      break;
     case 10 : /* invisible tank level */
	      if (game_loop_cntr == 50)
		{
		 tanks[3].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		 make_sound(WIERDEVENT);
		}
	      else if (game_loop_cntr == 70)
		{
		 tanks[4].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		 make_sound(WIERDEVENT);
		}
	      break;
     case 11 : /* Tank yard level with hidden tanks in waiting */
	      if (game_loop_cntr == 100)
		{
		  make_sound(WIERDEVENT);
		  tanks[2].colors[0]=COMPUTERCOLOR;
		  tanks[2].colors[1]=COMPUTERCOLOR_2;
		  tanks[2].attackmode=NAIVE;
		  tanks[2].tanktype=SQRTANK;
		}
	      else if (game_loop_cntr == 200)
		{
		  make_sound(WIERDEVENT);
		  tanks[3].colors[0]=COMPUTERCOLOR;
		  tanks[3].colors[1]=COMPUTERCOLOR_2;
		  tanks[3].attackmode=NAIVE;
		  tanks[3].tanktype=SQRTANK;
		}
	      /* if (two players playing || difficulty_is == HARD) */
	      else if( (game_loop_cntr == 499)&&
			((two_players_playing()) ||
			 (difficulty_is==HARDDIFF)) )
		{
		  make_sound(WIERDEVENT);
		  if (tanks[4].armor > 0)
		   tanks[4].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		}
	      break;
     case 12 : /* Fortress w/terminators */
	     /* set a third tank to invis if (2 players || diff is HARD) */
	      if (game_loop_cntr == 350)
		{
		 if ((two_players_playing()) || (difficulty_is==HARDDIFF))
		  {
		   make_sound(WIERDEVENT);
		   tanks[4].tankstats[VISIBILITYSTATUS]=INVISIBLE;
		  }
		}
	      break;
     case 13 : /* THE 13TH LEVEL! */
	     if (game_loop_cntr==50)
	      {
	      /* if there is room for another foe */
	      /* check for an exact space to set a tank */
	      i=comptank_space_exists();

	      if ((i) && (foes_on_13th))
	       {
		 /* set some variables */
		 ttypeis=((rand() % 4) +1); /* set a tanktype 1-4 */
		 amodeis=((rand() % 4) +1); /* set a heuristic 1-4 */
		 /* now get some coordinates */
		 xis=((rand()%65)+100);
		 yis=((rand()%148)+9);

		/* if that spot is open */
		if (check_space_for_no_tanks(i,xis,yis))
		  {
		    special_comptank_init(i,xis,yis,amodeis,ttypeis,1);
		    foes_on_13th--; /* subtract one tank from the list */
		  }
	       }
	       game_loop_cntr=25; /* reset the counter to 25 */
	      } /* end if (game_loop_cntr==50); */
	      break;
     }  /* end switch (level) */
   } /* end if (game_loop_cntr < 500) */
}
/***************************************************************************/
int comptank_space_exists(void)
/* This function returns a number if there is a space in the tanks array */
/* that is not being used by a computer tank.  It returns that exact num. */
{
  int i;

  for (i=2;i<MAXTANKS;i++)
   {
     if (tanks[i].armor < 1)  /* tank space is available */
      {
	return i;
      }
   }

  /* if we got to here, then no spaces are available */
  return FALSE;
}
/***************************************************************************/
void special_comptank_init(int num,int x,int y,int amode,int ttype,int arm)
/* This function is called to specifically and specially initialize */
/* a computer tank.  It is invoked during the middle of a combat round. */
/* Returns FALSE if a tank cannot be started at (x,y) coordinates. */
{
     /* set the new coordinates of the tank */
     tanks[num].xcor=x;
     tanks[num].ycor=y;
     tanks[num].pos = LEFT;  /* just set it to left ? :) */

     tanks[num].playertype = COMPUTER;
     tanks[num].tanktype   = ttype;
     tanks[num].attackmode=amode;
     tanks[num].tanknumber = num;
     tanks[num].tankstats[GAMESTATUS] = PLAYING;
     tanks[num].armor   = arm + difficulty_is; /* so 2 for mediumdiff */
     tanks[num].previousarmor = arm + difficulty_is;

      if (ttype==CIRCLETANK)
       {
	tanks[num].colors[0]  = BLUE;
	tanks[num].colors[1]  = BLACK;//COMPUTERCOLOR_2;
       }
      else if (ttype==HUNTERKILLER)
       {
	tanks[num].colors[0]    = HKCOLOR;
	tanks[num].colors[1]    = HKCOLOR_2;
       }
      else if (ttype==GUNTANK)
       {
	tanks[num].colors[0]    = OFFGRAY;
	tanks[num].colors[1]    = 30; /* 30 is a gray color */
       }
      else
       {
	tanks[num].colors[0]    = COMPUTERCOLOR;
	tanks[num].colors[1]    = COMPUTERCOLOR_2;
       }

      tanks[num].projcolor    = COMPUTERPROJCOLOR;
      tanks[num].current_weapon = NORMAL_PROJ;

    /* make a noise */
     if (soundon)
	make_sound(WIERDEVENT);
}
/***************************************************************************/
#define _TANKFNCS
#endif