
/* Kirk Christiansen */
/* TANKGAME.C        */

/* MODE 13h VERSION  */
/* Tank Combat Game  */
/* VERSION : 5.0    */
/**************************************************************************/
#include <stdio.h>
#include <conio.h>       /* for getch()                */
#include <dos.h>         /* for delay  / and random    */
#include <stdlib.h>      /* for srand and rand         */
#include <alloc.h>
#include <time.h>
/* My modules created for tankgame */
#include "tankdoc.c"    /* This is all comments. It is the documentation */
#include "kcgrphcs.c"   /* mode 13h graphics lib */
#include "kctnkasm.h"   /* header file for my assembly functions */
#include "tankmenu.c"    /* my menu system */
#include "drawtank.c"    /* tank drawing functions     */
#include "tankfncs.c"    /* other functions            */
#include "tankvars.c"    /* global variables           */
#include "movetank.c"    /* tank movement functions    */
#include "tnkprojs.c"    /* tank projectile functions  */
#include "tankio.c"      /* contains keyboard handler  */
#include "boards.c"      /* board drawing functions    */
#include "setboard.c"    /* updating level functions   */
#include "tnkpicts.c"    /* bitmap function module     */
#include "powerups.c"    /* power-up pill functions    */
#include "kc_pcsnd.c"    /* sound & noise module       */
#include "weapons.c"    /* tank weapons functions     */
#include "miscfncs.c"   /* misc function calls        */
#include "effects.c"    /* special effects module     */
#include "ser_lib.c"    /* communications library     */
#include "kc_keybd.c"   /* my keyboard handler        */
#include "kc_jstck.c"   /* my joystick library        */
/***************************************************************************/
/************************* MAIN PROGRAM ************************************/
/***************************************************************************/
void main (int argc, char *argv[])
{
 int i; /* counter variable */
 unsigned int xmin,xmid,xmax; /* joystick auto-calibration variables */
 unsigned int ymin,ymid,ymax;
 int savelevel = 0;            /* If this doesn't match current level,   */
			       /* then the player has cleared the board. */

 /* First, check the command line parameters */
 if (!strcmp(argv[1],"n")) /* use ! here cuz strcmp returns 0 if true */
    report_status=FALSE;
 /* Lets check the argc (count) to avoid a compiler warning anyways */
  if (argc > 2)
    {sound(600);delay(30);nosound();}

 /* Display Startup */
 if (report_status)
  {
   printf("\nLoading Tankgame v5.0(b4).....1995Tri-Cat Software.\n");
   delay(200);  printf (" Checking Memory...\n"); delay(200);
  }

 /* Make sure Tankgame has enough memory for needed sprites, tell user */
 if ((farcoreleft())<MINIMUM_MEM_NEEDED)
   {
    printf (" Not enough DOS memory to run Tankgame. \n");
    printf (" You need %lu bytes.\n",MINIMUM_MEM_NEEDED);
    printf (" You only have %lu bytes free.\n",farcoreleft());
    exit(1);
   }
 else
   {
     if (report_status)
      {
       printf (" Memory Status is ");

       if ((farcoreleft()) > MINIMUM_MEM_NEEDED+100000)
	 printf("EXCELLENT.\n");
       else /* must be average or good */
	 printf("GOOD.\n");
       delay (200); /* small delay */
      }
   }

 /* Now, Report the memory */
 if (report_status)
   printf("Starting Tankgame with: %lu bytes free.\n", farcoreleft());

 /* Test CPU speed */
   get_computer_speed();

 /* Install my Keyboard handler */
  remove_old_keyboard_handler();

  if (report_status)
   printf("New keyboard handler is now installed.\n");

 /* Detect if joysticks are present */
  detect_joystick(0, &joystick1); /* check for joystick A */

 /* Enable only the A joystick (Player 1) THIS ISNT NEEDED ! */
  joystick1.mask &= 0x3; /* turns off bits 2+3 but leaves 0+1 on */

 /* Calibrate the A joystick if present */
 if (joystick1.mask != 0)
  {
   joystick1.is_present=TRUE;/* set first joystick to TRUE (plugged in) */
   calibrate_js_midpoint(0,&joystick1,&xmin,&ymin,&xmid,&ymid,&xmax,&ymax);
   /* set the thresholds */
   joystick1.upthreshold=ymax/3;
   joystick1.downthreshold=(ymax*2)/3;
   joystick1.leftthreshold=xmax/3;
   joystick1.rightthreshold=(xmax*2)/3;
  }
 else
   joystick1.is_present=FALSE;

 /* Report if joystick is present to user */
 if (report_status)
  {
     if (joystick1.is_present)
	printf ("JOYSTICK A is installed and pre-calibrated.\n");
     else
	printf ("No Joystick was detected. \n");
  }

 /* Get the current video mode the computer is in */
  oldmode = get_current_mode();

 /* Report current video mode */
 if (report_status)
   printf ("Current video mode is %dh.\n",oldmode);

 /* Set the pointer to video memory, Video memory begins at A000:0000 */
 screen = (char *)MK_FP(0xa000,0);

 /* Create & set a second pointer to an offscreen buffer */
 screen_buffer = (char far *) farcalloc(64000,sizeof(unsigned char));

 if (report_status)
  {
   printf("AFT SCRNBUFF, BFR LOAD DATFILES: %lu ", farcoreleft());
   printf("bytes free.\n");
  }

 /* BEFORE ANY CALLS TO MY GRAPHICS LIB, I GOTTA INITIALIZE mul320_tbl */
  initialize_mul320_table();

 /* Load in the Tankgame multi-purpose sprites from PCX graphic files */
  load_in_tankgame_sprites();

 if (report_status)
   printf("AFT LD TNKGM SPRITES: %lu bytes free.\n", farcoreleft());

 /* Load in the Menu files */
 load_in_menu_pcx_files();

  if (report_status)
   {
    printf("AFT LD MENU DATFILES: %lu bytes free.\n", farcoreleft());
    printf("\n            < Hit a Key > \n");
    while(!keypress());reset_keys_array(); /* wait for keypress */
   }

 /* Clear screen */
 clear_screen (screen);

 /* Clear the screen buffer */
 clear_screen (screen_buffer);

 /* Seed the random number generator */
 srand (time(NULL));

/**************** The Game Begins ****************************************/

  /* Initialize mode 13h */
   set_mode (0x13);

 /* Now, set the correct color palette for graphics */
  if (pcx_files_loaded==TRUE)
   {
    setpalette(pcx[0].palette);
   }
  else /* pcx files did not load */
   {
     /* free memory and exit */
     farfree(screen_buffer);
     set_mode(oldmode);
     printf (" ERROR loading .dat files. \n "); /* ie .pcx files */
     /* Restore old keyboard handler */
     restore_old_keyboard_handler();
     exit (1);
   }

    current_time = clock();   /* update game time */

 /* Display an introduction screen to user */
   introscreen();

/******** EXTREME OUTER LOOP (for program termination check ***************/
do
{
  /* Reset some of the game variables incase user has restarted again */
  reset_tanks(ALLTANKS);   /* Clears the whole tank array */
  reset_power_ups();       /* Resets the power-up pill stats */
  reset_explosions();      /* Resets all the explosions */
  level_9_booby_trap=FALSE; /* set to true when triggered */

 /* Fade screen (morph style) */
   morph_fade(screen,pcx[TANKMENU].image);

 /* Invoke menu system for user interaction on game setup */
   display_configuration_menu();

  /* Call the main setup function that sets up the entire game for play */
    set_up_tank_game();

/********** OUTSIDE LOOP - For Level Change Functions *********************/
   do
   {
    /* reset the game_loop_cntr to 0 */
    game_loop_cntr=0;

    savelevel=level;  /* Set the save level equal to current level */

    if ((gametype==FULLGAME) || (gametype==TWOPLYR_FULLGAME))
       set_up_level();  /* Program the level, & computer tanks */
    else  /* Must be Duel or Null Modem Match, so set up a single board */
       set_up_board();

    clear_screen(screen_buffer);

    drawboard(boardnumber,boardcolor); /* Draw the current board  */

    drawtanks();       /* Place the tanks onto the board */

    printoutstats();   /* Print out the level and armor, weapons  */

    /* Move the screen buffer to the screen with an assembly function */
    putwindow(0,0,320,200,FP_OFF(screen_buffer),FP_SEG(screen_buffer));

    /* Pause so the user can see the levels layout */
    delay(500);

/*************************************************************************/
/******************** Main Program Loop **********************************/
/*************************************************************************/
      do
      {
	/* Wait so game loop time will be consistant */
	while ((clock()-current_time) < TICKS_PER_LOOP);

	current_time = clock();   /* Update game time */

	getinput();    /* Get input from player(s) & from computer */

	adjustprojectiles();    /* Move the projectiles */

	processinput();         /* Carry out desired movements */

	check_for_timed_events();  /* checks time triggered activities */

	if (level==savelevel)
	  draw_board_items();  /* items that can be under tanks & projs */

	drawtanks();            /* Draw tanks at proper coordinates */

	drawprojectiles();      /* Draw the projectiles */

	drawboardoutline(boardcolor);  /* Draw it every loop, so piercing */
				       /* projectiles can't go through it */
	check_for_projhits();   /* Check for collisions on tanks */

	draw_explosions();  /* Draw explosions if they exist on game grid */

	calcstats();  /* See if there has been any armor damage to tanks */

	update_game(); /* Check for destroyed tnks, update game_loop_cntr */

	wait_for_vert_retrace(); /* Wait for the vertical retrace */

	/* Move the screen buffer to the screen with an assembly function */
	putwindow(0,0,320,200,FP_OFF(screen_buffer),FP_SEG(screen_buffer));

      }
      while ((!quit_game) && (savelevel==level));
/*************************************************************************/
/******************** End Main Program Loop ******************************/
/*************************************************************************/

   /* Finish displaying explosions if there are any */
   /* And pause between level changes here          */
    for (i=0;i<END_OF_LEVEL_PAUSE_COUNT;i++)
     {
      /* update current time */
       current_time = clock();
      /* Draw the explosions */
       draw_explosions();
      /* Move the screen buffer to the screen */
       putwindow(0,0,320,200,FP_OFF(screen_buffer),FP_SEG(screen_buffer));
      /* small delay so user can see explosions */
       delay(50);
     }

    /* Now, a final delay before starting next level or ending game */
     delay(125);

    /* Check to see if player has finished all levels */
     if ((level > MAXLEVELS) || (gametype==MATCH))
       quit_game=TRUE;

     if (!quit_game)
       select_random_fade(screen);  /* Do a random fade out */

   }
   while (!quit_game);
/**********************END OUTER LOOP ************************************/

 /* Before displaying ending make sure nightfight palette is off */
 if (nightfight==TRUE)
     setpalette(pcx[0].palette); /* set the default palette */

 /* Game is over, display some ending information like the winner */
  display_ending();

 /* Ask if user wants to play again */
  ask_for_play_again();
}
while (!quit_entire_program);
/************ END EXTREME OUTER LOOP **************************************/

 /* Fade out */
  select_random_fade(screen);

 /* Restore ISR's, close Com port, release memory, etc */
  do_quit_game_sequence(0);
}
/**************************************************************************/

