
/* Kirk Christiansen         */
/* TANKIO.C                  */

/* Input Device Handler File */
/* Created for Tank Game     */
/* Functions for user IO     */

#ifndef  _TANKIO
/*************************************************************************/
#include <conio.h>
#include <alloc.h>

#include "tankvars.c"
#include "kc_keybd.c"
#include "kc_jstck.c"   /* my joystick library */
#include "movetank.c"
#include "tnkprojs.c"   /* for projectile firing   */
#include "queueops.c"   /* to get moves from tank input buffer */
#include "naive.c"      /* naive heuristic functions */
#include "patrol.c"     /* patrol heuristic functions */
#include "naive2.c"
#include "termintr.c"
#include "tnkpicts.c"  /* for call to free up memory */
#include "ser_lib.c"
#include "miscfncs.c"
/*************************************************************************/
#define CRUISECOUNT 3     /* Number of consecutive times an arrow    */
			   /* key must be pressed before auto cruise. */
#define FIREKEY  SPACE      /* just set spacebar equal to FIREKEY */
#define SWITCHWEAPONKEY TAB /* set TAB equal to SWITCHWEAPONKEY */
/*************************************************************************/
/* Function Prototypes */
unsigned int get_keyboard_input(int playernum);
unsigned int get_joystick_input(int playernum);
void getinput(void);
void processinput (void);

/* local module global */
int turn_off_comp_tanks = FALSE; /* secret code flag to */
				 /* turn off computer tanks */

/*************************************************************************/
unsigned int get_keyboard_input(int playernum)
/* This function simply takes in as input the playernum and then */
/* gets the appropiate keyboard input based on the current keys  */
/* being used by the player.   It returns 0 if no input is found */
{
    if ((gametype==MATCH)||(gametype==TWOPLYR_FULLGAME))
     {
      /* The keyboard is divided!  So use correct keys */
       if (playernum==0) /* Player 1 who uses left side of keyboard */
	    { /* uses e,s,d,f for arrow keys, CAPS lock for fire & tab */
	      if (get_and_reset_key_state(KEY_E))
		return UPARROW;
	      else if (get_and_reset_key_state(KEY_S))
		return LEFTARROW;
	      else if (get_and_reset_key_state(KEY_F))
		return RIGHTARROW;
	      else if (get_and_reset_key_state(KEY_D))
		return DOWNARROW;
	      else if (get_and_reset_key_state(CAPSLOCK)) /* fire key */
		return SPACE;
	      else if (get_and_reset_key_state(TAB)) /* switch wpn key */
		return TAB;
	    }

	   else /* playernum==1, Player 2 who uses right side of keyboard */
	    {
	      if (get_and_reset_key_state(UPARROW))
		return UPARROW;
	      else if (get_and_reset_key_state(LEFTARROW))
		return LEFTARROW;
	      else if (get_and_reset_key_state(RIGHTARROW))
		return RIGHTARROW;
	      else if (get_and_reset_key_state(DOWNARROW))
		return DOWNARROW;
	      else if (get_and_reset_key_state(ENTER)) /* fire key */
		return SPACE;
	      else if (get_and_reset_key_state(SLASH)) /* switch wpn key */
		return TAB;
	    }
     } /* end if ((gametype==MATCH) || (gametype==TWOPLYR_FULLGAME)) */

    else    /* Null Modem Match, or Fullgame (ie Full keybrd) */
     {
	      if (get_and_reset_key_state(UPARROW))
		return UPARROW;
	      else if (get_and_reset_key_state(LEFTARROW))
		return LEFTARROW;
	      else if (get_and_reset_key_state(RIGHTARROW))
		return RIGHTARROW;
	      else if (get_and_reset_key_state(DOWNARROW))
		return DOWNARROW;
	      else if (get_and_reset_key_state(SPACE)) /* fire key */
		return SPACE;
	      else if (get_and_reset_key_state(TAB)) /* switch wpn key */
		return TAB;
     }


 /* else no key press found */
  return 0;
}
/***************************************************************************/
unsigned int get_joystick_input(int playernum)
/* This function simply takes in as input the playernum and then */
/* gets the appropiate joystick input based on the current input  */
/* being entered by the player.  It returns 0 if no input is found */
{

 /* CHECK to see if player inputed valid move last round...if so */
 /* don't let him again cuz joystick inputs way quicker than the */
 /* the keyboard who has its keys reset. */
 if (!tanks[playernum].lastinput)
   {
      /* Read in the joystick input for Joystick A (0) */
       read_joystick (0, &joystick1);

       if (joystick1.y_axis < joystick1.upthreshold)
	 return UPARROW;
       else if (joystick1.y_axis > joystick1.downthreshold)
	 return DOWNARROW;
       else if (joystick1.x_axis < joystick1.leftthreshold)
	 return LEFTARROW;
       else if (joystick1.x_axis > joystick1.rightthreshold)
	 return RIGHTARROW;
       else if (joystick1.buttonA)
	 return SPACE;
       else if (joystick1.buttonB)
	 return TAB;
  }

  /* Else no input found or input not taken in this round */
      return 0;
}
/***************************************************************************/
void getinput(void)
/* This function gets the next moves the tanks will do.   */
/* If the tank is HUMAN controlled, the function will     */
/* call to the function Keyboardevent to get the current  */
/* keyboard input from the HUMAN. If the tank is COMPUTER */
/* controlled, the function will call to a heuristic      */
/* input function within one of the heurisitic modules    */
{
  int i;
  char c;  /* used for sending through serial port */
  static int palette=0;

  /* First lets check for configuration toggle keys like speed */
  /* adjustment, sound toggle, ESC/quit key etc                */
     if (get_and_reset_key_state(ESC))
       {
	 quit_game=TRUE; return;
       }
     else if (get_and_reset_key_state(F1)) /* Toggle sound */
       soundon=!soundon;
     else if (get_and_reset_key_state(PAUSE)) /* Pause */
	do_pause_routine();
     /* Check for jump to level 5 code */
     else if ((get_and_reset_key_state(KEY_J)) &&
	      (get_and_reset_key_state(KEY_M)) &&
	      (get_and_reset_key_state(KEY_P)) &&
	      (level < 5) )
	       {
		level=5;  /* user has skipped to level 5 */
		level_5_jmp_entered=TRUE; /* set flag */
	       }

     /* CHECK TO SEE IF Secret Cheat Code has been entered */
     else if ((get_and_reset_key_state(KEY_B)) &&
	      (get_and_reset_key_state(KEY_O)) &&
	      (get_and_reset_key_state(KEY_Z)) &&
	      ((get_and_reset_key_state(KEY_E)) ||
	      (get_and_reset_key_state(KEY_Y))))
	    {
	     reset_keys_array(); /* to avoid any buffered codes */
	     sound(200);delay(15);nosound(); /* tell user code worked! */
	     secret_code_entered=!secret_code_entered;
	    }

     /* NOW SECRET CHEAT CODES */
     if (secret_code_entered)
      {
	if (get_and_reset_key_state(F2)) /* Toggle palette */
	 {
	  if (!palette)
	    palette=5;
	  else
	    {palette=0;sound(1000);delay(20);nosound();}

	    setpalette(pcx[palette].palette); /* set the palette */
	  }
	 else if (get_and_reset_key_state(F3))
	    {if(level<MAXLEVELS)level++;}
	 else if (get_and_reset_key_state(F4))
	    { /* NOTE THIS KEY WILL NOT WORK IF ON LEV 9 and all */
	      /* the enemies are dead...cuz you will go into level++ */
	      /* part of code in update_game() function immediately. */
	      if(level>1)level--;
	     }
	/* Check for armor boost cheat code */
	 else if (get_key_state(KEY_A) && get_key_state(KEY_R))
	    tanks[0].armor++;
	 else if (get_key_state(KEY_M) && get_key_state(KEY_O))
	    tanks[1].armor++;
	 else if (get_key_state(KEY_T) && get_key_state(KEY_C))
	    turn_off_comp_tanks=!turn_off_comp_tanks;
	 /* check for invisibility toggle code */
	 else if (get_key_state(KEY_I) && get_key_state(KEY_0))
	    {if(soundon){sound(700);delay(200);nosound();}
	    tanks[0].tankstats[VISIBILITYSTATUS]=  /* toggle visibilty */
	     !tanks[0].tankstats[VISIBILITYSTATUS];}
	 else if (get_key_state(KEY_I) && get_key_state(KEY_1))
	    {if(soundon){sound(700);delay(200);nosound();}
	     tanks[1].tankstats[VISIBILITYSTATUS]= /* toggle visibility */
	     !tanks[1].tankstats[VISIBILITYSTATUS];}
	 /* check for far jumps on levels */
	 else if (get_and_reset_key_state(PLUS))
	      level=MAXLEVELS;
	 else if (get_and_reset_key_state(MINUS))
	      level=1;
      }

  /* Now check the players/computers input */
  for (i=0; i<MAXTANKS; i++) /* Get the input from every tank */
  {
   if (tanks[i].armor > 0)  /* If tank is alive */
   {
     if (tanks[i].playertype == HUMAN)
       {
	 if (i==0) /* NOTE: this will be local player in Null Modem Match */
	  {
	    /* get a keystroke if available */
	    tanks[i].currentinput = get_keyboard_input(i);

	    /* maybe joystick is being used too */
	    if ((!tanks[i].currentinput)&&(joystick1.is_present))
	      tanks[i].currentinput = get_joystick_input(i);

	    if (gametype==NULL_MODEM_MATCH)
	     {
	       c = convert_scan_to_ascii (tanks[i].currentinput);
	       send_a_character(c);
	     }
	  }
	 else if (i==1) /* NOTE: this will be remote plyr in Nll Mdm Match */
	  {
	    if (gametype==NULL_MODEM_MATCH)
	     {
	       c = get_character_from_in_buffer();
	       tanks[i].currentinput =
		  convert_ascii_to_scan(c);
	     }
	    else /* game must be 2 player same keyboard,cuz its HUMAN */
	     {
	      /* get a keystroke if available */
	      tanks[i].currentinput = get_keyboard_input(i);
	     }
	  }
       }

     else  /* Else, tank is a computer */
      {
	/* Check to make sure the watching user doesn't want to quit, */
	/* toggle with the sound, pause the game, or adjust the speed */
	/* This is necessary if user is in MATCH mode and is watching */
	/* two computer tanks fight each other.                       */
	 //k=keyboardevent();

	/* NOW, GET COMPUTER'S MOVE. FIRST CHECK ITS BUFFER */
	if (tanks[i].inputbuffer[0])  /* a move is in the buffer */
	 {
	  /* Get the move from the buffer, and put it in current input */
	   remove_buffer(i);
	 }

	/*  Else (BUFFER EMPTY) */
	/*  So get the computer's input based on its heuristic */
       else
	 {
	  switch (tanks[i].attackmode)
	   {
	      case (RANDOM) :  randomattack(i);
			       break;
	      case (NAIVE)  :  naive_attack(i);
			       break;
	      case (NAIVE2) :  naive2_attack(i);
			       break;
	      case (TERMINATOR) :  terminator_attack(i);
				   break;
	      case (SMART)  : // smart_attack(i);
			       break;
	      case (DUMMY)  :  /* do nothing, just sit there */
			       break;
	      case (PATROL)  : patrol_attack(i);
			       break;
	      default       :  /* tank does nothing, no attack mode */
			       break;
	    }
	 }
      } /* else tank is a computer */
   } /* end if (tanks[i].armor > 0) */
 } /* end for (i=0;i<MAXTANKS;i++) */
}
/*************************************************************************/
void processinput (void)
/* This function simply takes the current input by a tank, and  */
/* carries out transferring that input into a move by the tank. */
{
 int i;
 int move_has_occurred = FALSE;

  for (i=0; i<MAXTANKS; i++)
 {
  if (tanks[i].armor > 0)
   {
    /* First check to see if Computer tanks are frozen due to */
    /* secret code entered in by player.                      */
    if ((tanks[i].playertype==COMPUTER) && (turn_off_comp_tanks))
       {
	 break;  /* if so, exit so comp tanks can do anything */
       }

    /* First, check to see if a direction was pushed */
      /* If UPARROW pushed and tank isn't already crusing forward */
    if ((tanks[i].currentinput == UPARROW) &&
       (tanks[i].cruise != FORWARDCRUISE))
     {
       if (tanks[i].cruise == BACKWARDCRUISE)
	   tanks[i].cruise = NOCRUISE;
       if (tanks[i].thrustcount < 0)
	   tanks[i].thrustcount = 0;

      tanks[i].thrustcount++; /* Consecutive times thrust has been pushed */

      /* Check to see if tank is ready for automatic cruise control */
      if (tanks[i].thrustcount == CRUISECOUNT)
	{
	  tanks[i].cruise = FORWARDCRUISE;
	  tanks[i].thrustcount = 0;
	  move_has_occurred = movetankforward(i);
	}
      else
	 move_has_occurred = movetankforward(i);
     }

   /* DOWN ARROW PRESSED */
   else if ((tanks[i].currentinput == DOWNARROW) &&
	   (tanks[i].cruise != BACKWARDCRUISE))
     {
       if (tanks[i].cruise == FORWARDCRUISE)
	   tanks[i].cruise = NOCRUISE;
       if (tanks[i].thrustcount > 0)
	   tanks[i].thrustcount = 0;

	tanks[i].thrustcount--;

	    /* Check to see if tank is ready for backward auto cruise */
	    if (tanks[i].thrustcount==(CRUISECOUNT * -1))
	       {
		tanks[i].cruise=BACKWARDCRUISE;
		tanks[i].thrustcount=0;
		move_has_occurred = movetankbackward(i);
	       }
	    else
	       move_has_occurred = movetankbackward(i);
     }

   /* LEFT ARROW PRESSED */
   else if (tanks[i].currentinput == LEFTARROW)
     {
      rotatetankleft(i);
      tanks[i].thrustcount = 0;

	     /* If it's a computer tank cruising, stop the cruise */
	     if ((tanks[i].cruise > NOCRUISE) &&
		(tanks[i].playertype==COMPUTER))
		 {
		    tanks[i].cruise = NOCRUISE;
		 }
     }

   /* RIGHT ARROW PRESSED */
   else if (tanks[i].currentinput == RIGHTARROW)
     {
      rotatetankright(i);
      tanks[i].thrustcount = 0;

	     /* If it's a computer tank cruising, stop the cruise */
	     if ((tanks[i].cruise > NOCRUISE) &&
		  (tanks[i].playertype==COMPUTER))
		 {
		  tanks[i].cruise = NOCRUISE;
		 }
     }

   /* Check to see if FIRE was pushed and was not pushed last round */
   /* This way you can have a constant stream of projectiles.       */
   else if ((tanks[i].currentinput == SPACE)
	     && (tanks[i].lastinput != SPACE))
     {
       fireprojectile(i);
     }

   /* Check to see if ESC was pushed. This is needed for NULLMODEMGAME*/
   else if (tanks[i].currentinput == ESC)
     {
       quit_game=TRUE;
       return;
     }

  /* Check for weapons change */
   else if (tanks[i].currentinput==TAB)
    {
      change_current_weapon(i);
    }


   /* ELSE, no significant key was pushed. Move tanks on cruise control */
   else
     {
       if (tanks[i].cruise==FORWARDCRUISE)
	  move_has_occurred = movetankforward(i);
       else if (tanks[i].cruise==BACKWARDCRUISE)
	  move_has_occurred = movetankbackward(i);

       if (!move_has_occurred)
	  tanks[i].cruise=NOCRUISE;
     }

   /* Save lastinput & restore currentinput to 0 */
     tanks[i].lastinput = tanks[i].currentinput;
     tanks[i].currentinput = 0;

   } /* end if (tanks[i].armor > 0) */


 /* Check for special events if a move has occurred before drawing */
 if (move_has_occurred)
   check_for_special_events(i);


 } /* end for (i=0;i<MAXTANKS;i++) */
}
/***************************************************************************/
#define _TANKIO
#endif
