
/* Kirk Christiansen */
/* TANKVARS.C        */

/* Global variables and constant declarations */
/* for the Tank Game.                         */

#ifndef _TANKVARS
/*************************************************************************/
#include "pcxfuncs.c"
#include "kc_jstck.c"
/*************************************************************************/
#define TS  6           /* Tank Size */
#define TICKS_PER_LOOP 1 /* Number of ticks per game loop */
#define MINIMUM_MEM_NEEDED 349000

#define PROJDAM     1    /* projectile armor damage   */
#define PROJRADIUS  1    /* projectile radius         */
#define STARTARMOR  10   /* starting armor for tanks  */
#define PLAYERTANK      SQRTANK  /* type of tank is square */
#define COMPUTERTANK    SQRTANK  /* type of tank is square */

/* define colors */
#define BLACK  0
#define BLUE   1
#define GREEN  2
#define CYAN   3
#define RED    4
#define MAGENTA    5
#define BROWN      6
#define LIGHTGRAY  7
#define DARKGRAY   8
#define LIGHTBLUE  9
#define LIGHTGREEN 10
#define LIGHTCYAN  11
#define LIGHTRED   12
#define LIGHTMAGENTA 13
#define YELLOW       14
#define WHITE        15

#define OFFGRAY     28
#define DARKGRAY2   29
#define MEDIUMRED   32
#define BLOODRED    42
#define DARKRED     47
#define FADEORANGE  59
#define LIGHTBROWN  60
#define DARKBROWN   63
#define WOOD        65
#define LIGHTYELLOW 70
#define OLIVEDRAB   77
#define OFFGREEN    78

#define LIMEGREEN  100
#define ELECTRICBLUE 143
#define RIVERBLUE  148
#define AQUA       130
#define DARKPURPLE 175

/* Misc colors */
/* NOTE: each tank consists of two colors */
#define NUMOFTNKCLRS             2    /* How many colors define a tank  */

#define PLAYERCOLOR        GREEN        /* Player's primary color   */
#define PLAYERCOLOR_2      LIGHTGREEN   /* Player's secondary color */

#define COMPUTERCOLOR      DARKGRAY
#define COMPUTERCOLOR_2    30

#define PLAYERCOLOR2     RED         /* 2nd Player tank color */
#define PLAYERCOLOR2_2   OFFGREEN    /* secondary color         */

#define HKCOLOR            OFFGRAY     /* Hunter Killer tank color */
#define HKCOLOR_2          DARKRED

#define CIRCLE_CLR         BLUE
#define CIRCLE_CLR2        BLACK
/* Obstacle colors */
#define WALLCOLOR          LIGHTGRAY
#define WALLCOLOR2         OLIVEDRAB
#define WALLCOLOR3         DARKGRAY2
#define BRICKWALLCOLOR     BROWN
#define BRICKWALLCOLOR2    WOOD
#define GROUNDCOLOR        BLACK      /* the base color of board  */
#define MOUNTAINCOLOR      WHITE


#define BRIDGE_COLOR_1     DARKBROWN
#define BRIDGE_COLOR_2     LIGHTBROWN

#define PLAYERPROJCOLOR    YELLOW
#define COMPUTERPROJCOLOR  LIGHTRED
#define PLAYERPROJCOLOR2   CYAN

#define PROJEXPLOSIONCOLOR1   LIGHTYELLOW
#define PROJEXPLOSIONCOLOR2   FADEORANGE
#define PROJEXPLOSIONCOLOR3   MEDIUMRED

#define POWER_UP_COLOR_1      LIGHTMAGENTA
#define POWER_UP_COLOR_2      BLOODRED

#define NIGHT_PCXPAL   5  /* what pcx file night fight's use */

#define TRUE          1
#define FALSE         0
#define PROJSPEED     5       /* The projectile move rate      */
#define TANKMOVERATE  3       /* The tank movement rate        */
#define MAXPROJS      5       /* Max # of projectiles per tank */
#define MAXTANKS      6       /* The maximum number of tanks in game */
#define MAXWEAPONS    3
			      /* that can be playing at one time */
#define MAXLEVELS        13    /* Number of levels in Tank Game       */
#define MAX_EXPLOSIONS    7   /* How many explosions can be at once */

/* ending code you get if you win */
#define SECRETCODE  "4UJOSE"

#define PROJ_EXIST_TIME    60   /* In ticks */
#define PROJ_EXPLOSION_EXIST_TIME  5  /* Ticks a proj explosion lasts */
#define TANK_EXPLOSION_EXIST_TIME  10  /* Ticks a tank explosion lasts */

#define MAX_POWER_UPS           7  /* Max amount of power-ups on a board */
#define MAX_OBSTACLE_CLRS       6  /* Used to define an array size       */
#define MAX_OBSTACLE_PROJ_CLRS  9  /* Used to define an array size       */

#define MAXBUFFERSIZE           25    /* Buffer size of stored moves   */
#define STATSARRAYSIZE           2    /* 0=gamestatus, 1=invisible/visible */
#define END_OF_LEVEL_PAUSE_COUNT  10   /* Approximately 5 seconds */

/* Starting locations, in terms of screen coordinates */
#define STARTXTANK0    20      /* start x coordinate of player tank   */
#define STARTYTANK0    84      /* start y coordinate of computer tank */

#define STARTXTANK1   299
#define STARTYTANK1    84

#define STARTXTANK2   300
#define STARTYTANK2    70

#define STARTXTANK3    20
#define STARTYTANK3    70

/* Starting facing positions of tanks */
#define STARTPOSTANK0   RIGHT
#define STARTPOSTANK1   LEFT
#define STARTPOSTANK2   LEFT
#define STARTPOSTANK3   RIGHT

/* Screen Constants */
#define SCREENWIDTH  320
#define SCREENHEIGHT 200
#define GAME_GRID_WIDTH   320
#define GAME_GRID_HEIGHT  167

/* Boundary squares on objects, used for collision detection */
#define TANKBOUND     6
#define PROJBOUND     1

#define POWER_UP_WIDTH   4
#define POWER_UP_HEIGHT  5
/* Searching constants */
#define SCANDISTANCE   200    /* How far a tank will scan for an enemy  */
#define SCANRATE      10      /* Pixel skip step amount during scaning  */

/* Distance measurements, for tank searching functions */
#define ADJACENTDISTANCE  30
#define CLOSEDISTANCE     75
#define MEDIUMDISTANCE    100

/* Efficiency constants --- to avoid division */
#define TSDIV2     3     /* Tank size divided by 2 */
#define TSDIV3     2
#define TSDIV5     1
#define TSDIV10    1
#define TSMUL1_5   9     /* Tank size multiplied by 1.5  */
#define TSMUL_P75  4     /* Tank size multiplied by .75  */
#define TSMUL_P7   4     /* Tank size mulitplied by .7   */
#define TSMUL2    12
#define TSMIN1     5     /* Tank size minus 1 */

/* File stuff */
#define NUMPCXFILES     8     /* Intro,mytanks,menu,misc(2),kcpict,tankbars */
#define NUMTANKPICTS    73    /* 3 types * 3 colors * 8 positions + circle*/
#define NUMSTATPICTS    34    /* 10 numbers,2'level's, 1'armor',3 statbxes */
			      /* 6 mini-abbrev blocks,3 wpn texts,2blanks*/
			      /* 9 ending picts (gmovr/plyr1/2/wins etc)*/
#define NUMOFCHARPICTS  36       /* 10 numbers, 26 letters */
#define NUMOFMENUPICTS  44

#define CHARPICTWIDTH    7
#define CHARPICTHEIGHT   9
#define NUMOFMISCPICTS   5  /* 2 explosions,3Pwr-upPills,*/
#define NUMOFPROJPICTS   12  /* 4 yellow, 4 lightred, 4 cyan*/
#define NUMOFMINIBOARDPICTS 9
#define NUMOFBOARDPICTS  20  /* 9 citys,1 pillar,1 bridge,river,2tele-boxs
			       ground,tree,deadtree,bush, 2 arrows */

#define SMALLBLDGWIDTH   20
#define SMALLBLDGHEIGHT  15
#define MIDBLDGWIDTH     35
#define MIDBLDGHEIGHT    30
#define BIGBLDGWIDTH     60
#define BIGBLDGHEIGHT    40
#define LONGBLDGWIDTH    30
#define LONGBLDGHEIGHT   55
#define BRIDGEWIDTH      49
#define BRIDGEHEIGHT     29
#define RIVERWIDTH       50
#define RIVERHEIGHT      72
#define TELE_GROUND_WIDTH  26
#define TELE_GROUND_HEIGHT 20
#define TREE_WIDTH  19
#define TREE_HEIGHT 21
#define DEADTREE_WIDTH  19
#define DEADTREE_HEIGHT 20
#define BUSH_WIDTH  14
#define BUSH_HEIGHT  9

#define SQRTANK_X_OFFSET   7
#define SQRTANK_Y_OFFSET   8
#define SQRTANKPICT_WIDTH    15
#define SQRTANKPICT_HEIGHT   15


/* Enumerated Types, used primarly for code readability  */
enum playertype {NOONE, HUMAN, COMPUTER};

enum tanktype {NOTANK, SQRTANK, HUNTERKILLER, GUNTANK, CIRCLETANK,
		DUMMYTANK};

enum filename_type {MYTANKS, TNKINTRO, TANKMSC1, TANKMSC2, TANKMENU,
		    KCPICT, TNKCTRLS, TANKBARS};

enum movetype {NOMOVE, UP,UPRIGHT,RIGHT,DOWNRIGHT,DOWN,
	       DOWNLEFT,LEFT,UPLEFT,FIRE};

enum lookmodes  {NOLOOK,SCAN,SEARCH};

enum distancetypes {ADJACENT, CLOSE, MEDIUM, FAR};

enum boardtypes {VOID,CITY,PILLARS,FORTRESS,CITY2,RIVER,PALACE,
		 RANDOM_BOARD,MULTI_TELEPORT_BRD,MOUNTAINS,SPLITEXIT,
		 SNAKEBRD,TANK_YARD,THE_13TH};

enum terraintype {EMPTY, SPOTTED, DENSE};

enum cruisetype  {NOCRUISE, FORWARDCRUISE, BACKWARDCRUISE};

/*3 Heurisitics */
enum attacktype {NOATTACK,RANDOM,NAIVE,NAIVE2,PATROL,TERMINATOR,
		 SMART,DUMMY};

/* Tank stats array subscript meanings */
enum tankstatstype  {GAMESTATUS, VISIBILITYSTATUS};
/* Possible game status for a tank */
enum gamestatustype {NOTPLAYING, PLAYING, JUSTKILLED};
/* Possible visibility status for a tank */
enum invisibletypes {VISIBLE, INVISIBLE};
/* Current type of game running in Tankgame */
enum gametype {NOGAME, FULLGAME, MATCH, TWOPLYR_FULLGAME, NULL_MODEM_MATCH};

enum explosion_types {PROJ_EXPLOSION, TANK_EXPLOSION};

 /* Parameter type for clearing the tank array during game. */
enum clear_size {ALLTANKS, NOTANKS/* not used */, COMPTANKS};

enum weapon_types {NORMAL_PROJ, REBOUNDING_PROJ, PIERCING_PROJ,
		   LASER/* not used ,Land Mine?*/};

enum power_up_types {NO_POWER_UP, ARMOR_PILL, REBOUND_PILL, PIERCING_PILL};

enum pc_status_types {SENDER, RECEIVER};  /* for communications */

enum difficulty_types {EASYDIFF,MEDIUMDIFF,HARDDIFF};

enum cpu_speed_types {SLOWSPEED, FASTSPEED};

struct explosion_type
	       {
		 int x;
		 int y;
		 int type;
		 unsigned int time;
	       };

struct proj_type
	       {
		int dir;
		int x;
		int y;
		int type;
		unsigned int time;
	       };

struct power_up_type
	       {
		int x;
		int y;
		int type;
		int value;
	       };

/*************************************************************************/
/********************* MAIN STRUCTURE FOR TANKS **************************/
/*************************************************************************/
struct tank
{
 int xcor;                  /* current x coordinate */
 int ycor;                  /* current y coordinate */
 int pos;                   /* position of tank (up,down,left,or right) */
 int cruise;                /* current cruise (forwards or backwards)   */
 int playertype;            /* computer or human    */
 int tanktype;              /* square, triangular   */
 int tanknumber;            /* Stores the tank's position in the array */
 int tankside;              /* tank can be on one of two sides (0 or 1) */
 int tankstats[STATSARRAYSIZE];/* Array that stores important info */
 int previousarmor;         /* armor left during previous loop */
 int armor;                 /* current armor        */
 int colors[NUMOFTNKCLRS];  /* colors of the tank, [0] is primary color */
 int projcolor;             /* projectile color     */
 enum attacktype attackmode;/* attack heuristic of the CPU tanks      */
 unsigned currentinput;     /* current move entered                   */
 unsigned lastinput;        /* the last entered move by tank          */
 int thrustcount;           /* number of consecutive thrust inputs    */
 int current_weapon;        /* current selected weapon                */
 int previous_ammo;          /* ammo of current weapon from previous loop */
 int weapon_ammo[MAXWEAPONS]; /*0=normproj, 1=rebnding proj, 2=?      */
			    /* weapon's ammo the tank posesses        */
 unsigned inputbuffer[MAXBUFFERSIZE]; /* computer's future moves */
 struct proj_type projs[MAXPROJS];
			   /* MAXPROJS = maximum projectiles at once */
};			    /* storing direction and x,y coordinates  */

/*************************************************************************/
/***************  GLOBAL VARIABLES ***************************************/
/*************************************************************************/
int soundon=TRUE;     /* Sound is either on(TRUE), or off(FALSE)   */
int boardnumber=VOID; /* Current board pattern                     */
int boardcolor=LIGHTGRAY;  /* Default/Current board color          */
int gametype=NOGAME;  /* Current type of game being played         */
int level = 0;        /* Current game level                        */
int pcx_files_loaded=TRUE;  /* FALSE if errors occur during loading */
int oldmode;          /* Original video mode user was in before     */
int current_com_port=1;  /* current open port used for communications  */
int current_local_pc_is=SENDER; /* either sender or receiver */
int quit_game   = FALSE;          /* Quit game flag variable */
int quit_entire_program = FALSE;
int report_status = TRUE;  /* for memory report upon game start */
int secret_code_entered = FALSE; /* flag needed to be on for cheating */
int level_5_jmp_entered = FALSE; /* flag set if user jmps to level 5 */
int foes_on_13th = 0; /* the num of extra foes on 13th level */
int level_9_booby_trap=FALSE; /* set to true when triggered */

/* Duel option global flags */
int powerups_on=TRUE;
int nightfight=FALSE;
int display_stats=TRUE;
int difficulty_is=MEDIUMDIFF;

/* input device globals */
struct joystick_type joystick1,joystick2;

/* time vars */
unsigned long current_time;

/* gameloop cntr (used for tanks that stall before going invisible/etc) */
unsigned long game_loop_cntr=0;

/* cpu speed variable */
unsigned long cpu_speed;

char far *screen;         /* Video Screen pointer      */
char far *screen_buffer;  /* Off-screen buffer pointer */

struct tank tanks[MAXTANKS];  /* This is the tank array structure       */
/*************************************************************************/
/* GLOBAL DATA STRUCTURES */

/* Define an array of all colors in which one can not go through */
char obstacle_color_array[MAX_OBSTACLE_CLRS]=
					  {
					   WALLCOLOR,WALLCOLOR2,
					   WALLCOLOR3,
					   BRICKWALLCOLOR,BRICKWALLCOLOR2,
					   MOUNTAINCOLOR
					  };

/* A more in-depth array of colors that one cannot pass through */
char obstacle_proj_color_array[MAX_OBSTACLE_PROJ_CLRS]=
				    {
				     WALLCOLOR,WALLCOLOR2,WALLCOLOR3,
				     BRICKWALLCOLOR,BRICKWALLCOLOR2,
				     MOUNTAINCOLOR,PLAYERPROJCOLOR,
				     COMPUTERPROJCOLOR,PLAYERPROJCOLOR2
				    };

/* Explosion array which keeps track of explosions on the screen */
struct explosion_type explosion_array[MAX_EXPLOSIONS]={0};


/* Power-up array for storing power pill locations and data */
struct power_up_type power_ups[MAX_POWER_UPS]={0};

/*************************************************************************/

/*  PCX file structure */
/* NOTE: pcx_struct is defined in module "pcxfuncs.c".  */
struct pcx_struct pcx[NUMPCXFILES];

/* structure for pcx file names */
char *pcx_file_names[NUMPCXFILES]= {  /* these are info-hiding names */
				     "tnkgm0.dat", /* mytanks.pcx  */
				     "tnkgm1.dat", /* tnkintro.pcx */
				     "tnkgm2.dat", /* tankmsc1.pcx */
				     "tnkgm3.dat", /* tankmsc2.pcx */
				     "tnkgm4.dat", /* tankmenu.pcx */
				     "tnkgm5.dat", /* kcpict.pcx   */
				     "tnkgm6.dat", /* tnkctrls.pcx */
				     "tnkgm7.dat"  /* tankbars.pcx */
				   };
/* Array for tank bitmaps/sprites */
unsigned char far *tank_picts[NUMTANKPICTS];
/* Array for statistics bitmaps/sprites */
unsigned char far *stat_picts[NUMSTATPICTS];
/* Array for characters&numbers */
unsigned char far *character_picts[NUMOFCHARPICTS];     /* 0-9, A-Z */
/* Sprite array for misc pictures */
unsigned char far *misc_picts[NUMOFMISCPICTS];
/* Sprite array for board pictures */
unsigned char far *board_picts[NUMOFBOARDPICTS];
/* Sprite array for mini board pictures */
unsigned char far *mini_board_picts[NUMOFMINIBOARDPICTS];
/* Sprite array for proj pictures */
unsigned char far *proj_picts[NUMOFPROJPICTS];
/* Sprite array for menu pictures */
unsigned char far *menu_picts[NUMOFMENUPICTS+1]; /* +1 cuz we start with */
						 /* button number 1.     */
/*************************************************************************/
#define _TANKVARS
#endif
