
/* Kirk Christiansen */
/* TERMINTR.C	*/

/* This module contains heuristics       */
/* for the terminator-style of attack.   */
/* This style contains several new       */
/* improvements, such as, obstacle       */
/* negotiation and an evaluation         */
/* structure that stores probablilities. */

#ifndef _TERMINTR
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>    /* random functions */
#include "tankvars.c"
#include "tanksrch.c"
/************************************************************************/
/* Function Prototypes */
void terminator_attack(int attacker);
void terminator_hunt_for_tank(int attackeris);
/************************************************************************/
void terminator_attack(int attacker)
/* This attack style mimics a terminator. I.e.. it locates     */
/* the enemy tank's x,y coordinates by means of radar or such, */
/* then it comes after that tank, with no fear of pain, pity,  */
/* or remorse.  It doesn't care if it has greater armor damage */
/* than its opponents. The biggest improvement in this heuristic */
/* is that it can negotiate obstacles with a high sucess rate    */
/* It does this by first examining an obstacles width and depth  */
/* It also determines what the shortest path is to take around   */
/* the obstacle. All this is calculated with a small evaluation  */
/* structure.  This is done by calling several functions, and    */
/* which ever move looks the best with each function call, well  */
/* then it its probablilty of getting done is increased.         */
{
 int tankfoundleft = FALSE;    /* is there a tank to the left?    */
 int tankfoundright = FALSE;   /* is there a tank to the right?   */
 int tankfound = FALSE;        /* is there a tank straight ahead? */

 int obstacle  = FALSE;
 int movepathblocked = FALSE;
 int enemy=-1;
 int distance=0;
 int direction;

 /* The tank must now check in 3 directions. */
 /* First, the tank scans straight ahead.    */
 direction = tanks[attacker].pos;
 scan_for_tank (attacker, direction, &tankfound,&enemy, &distance);

 /* If no tank was found, the tank will check to its right hand side */
 /* and if no tank there, it will scan to its left hand side.        */
 if (!tankfound)
  {
   direction = right_of(direction);
   scan_for_tank (attacker, direction, &tankfoundright,&enemy, &distance);

     /* If enemy is to the right, rotate towards it */
     if (tankfoundright)
       {
	add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	add_buffer(attacker,RIGHTARROW);  /* or Turn right */
	return;
       }
     else /* Check the left side of tank for enemy */
       {
	 direction = opposite_of(direction); /* from right to left */
	 scan_for_tank (attacker, direction, &tankfoundleft,
			&enemy, &distance);
	 if (tankfoundleft)
	   {
	    add_buffer(attacker,LEFTARROW);  /* Turn left */
	    add_buffer(attacker,LEFTARROW);  /* Turn left */
	    return;
	   }
	 else    /* no tanks in all 3 directions, so search */
	  {
	   terminator_hunt_for_tank(attacker);
	  }
       }
  }


 /* If a tank is found, calculate some moves */

 else if (tankfound)  /* straight ahead */
  {
    /* Check to see if obstacle is between the two tanks */
    obstacle = check_projpath_for_obstacle (tanks[attacker].xcor,
	       tanks[attacker].ycor,tanks[enemy].xcor,tanks[enemy].ycor,
	       tanks[attacker].pos);

    if (!obstacle)   /* No obstacle in firing or movement path */
     {
       if (distance != ADJACENT)
	 {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	   add_buffer(attacker,UPARROW);   /* move at tank */
	  }
	 else /* tanks are ADJACENT */
	  {
	   add_buffer(attacker,SPACE);  /* fire */
	   add_buffer(attacker,SPACE);  /* fire */
	  }

     } /* end if (!obstacle) */

     /* else if (an obstacle is in path between tanks) */
     else if (obstacle)
     {
       /* See if obstacle is in tank's move path */
       movepathblocked = check_movepath_for_obstacle(tanks[attacker].xcor,
			       tanks[attacker].ycor, tanks[attacker].pos);

       if (movepathblocked)
       {
	 /* Go around the obstacle, using a high percentage algorithm */
	 negotiate_obstacle (attacker);
       }
       else /* obstacle is far from tank, so tank can move forwards */
       {
	 add_buffer(attacker,UPARROW);
	 add_buffer(attacker,UPARROW);
       }
     }
  } /* end if (tankfound) */
}
/************************************************************************/
void terminator_hunt_for_tank(int attackeris)
/* This function is the code for what the terminator controlled */
/* tank does if it has no enemy in sight.  It calculates a move */
/* by means of an evaluation structure.                         */
{
  int x,y;
  int enemy_x, enemy_y;       /* Enemy's x and y coordinates */
  int obstacle_up=FALSE;      /* Obstacle  UP from tank?     */
  int obstacle_down=FALSE;    /* Obstacle DOWN from tank?    */
  int obstacle_left=FALSE;    /* Obstacle LEFT of tank?      */
  int obstacle_right=FALSE;   /* Obstacle RIGHT of tank?     */
  struct ways_you_can_move  shouldgo;  /* Stores the probablilities for */
				       /* a tank moving a certain way.  */

  /* Set shouldgo probabilities to 0 apiece */
  shouldgo.up=0; shouldgo.down=0; shouldgo.left=0; shouldgo.right=0;
  /* Set x and y to the looking tanks coordinates */
  x=tanks[attackeris].xcor; y=tanks[attackeris].ycor;
  /* Get an enemy tank's x and y coordinates */
  locate_enemy(&enemy_x, &enemy_y,attackeris);

  /* Determine which way to go */
  /* First, by which way the enemy is from the tank */
  if ((enemy_x < x) && (enemy_y < y))      /* Enemy is UP/LEFT from tank */
  {
   shouldgo.up+=1; shouldgo.left+=1;
  }
  else if ((enemy_x == x)&&(enemy_y <= y))  /* Enemy is straight UP */
  {
   shouldgo.up+=2;
  }
  else if ((enemy_x > x) && (enemy_y < y))  /* Enemy is to UP/RIGHT  */
  {
   shouldgo.up+=1; shouldgo.right+=1;
  }
  else if ((enemy_x <= x) && (enemy_y == y))  /* Enemy is directly LEFT */
  {
   shouldgo.left+=2;
  }
  else if ((enemy_x >= x) && (enemy_y == y))  /* Enemy is directly RIGHT */
  {
   shouldgo.right+=2;
  }
  else if ((enemy_x < x) && (enemy_y > y))  /* Enemy is to DOWN/LEFT */
  {
   shouldgo.down+=1; shouldgo.left+=1;
  }
  else if ((enemy_x == x)&&(enemy_y >= y))  /* Enemy is straight DOWN */
  {
   shouldgo.down+=2;
  }
  else if ((enemy_x > x) && (enemy_y > y))  /* Enemy is to DOWN/RIGHT */
  {
   shouldgo.down+=1; shouldgo.right+=1;
  }

  /* Now, see if there are any obstacles in path of promising ways */
  /* If not, upgrade their probabilities.                          */
  if (shouldgo.up)
   {
     obstacle_up=check_movepath_for_obstacle (x,y,UP);
     if (!obstacle_up)
	  shouldgo.up+=1;
   }
  if (shouldgo.down)
   {
     obstacle_down=check_movepath_for_obstacle (x,y,DOWN);
     if (!obstacle_down)
	  shouldgo.down+=1;
   }
  if (shouldgo.left)
   {
     obstacle_left=check_movepath_for_obstacle (x,y,LEFT);
     if (!obstacle_left)
	  shouldgo.left+=1;
   }
  if (shouldgo.right)
   {
     obstacle_right=check_movepath_for_obstacle (x,y,RIGHT);
     if (!obstacle_right)
	  shouldgo.right+=1;
   }

  /* Now pick the best probability way */
  if (shouldgo.up==3)
    {
     turn_tank_up(attackeris);
    }
  else if (shouldgo.down==3)
    {
     turn_tank_down(attackeris);
    }
  else if (shouldgo.left==3)
    {
     turn_tank_left(attackeris);
    }
  else if (shouldgo.right==3)
    {
     turn_tank_right(attackeris);
    }

  else if ((shouldgo.up==2) && (!obstacle_up))
    {
     turn_tank_up(attackeris);
    }
  else if ((shouldgo.down==2) && (!obstacle_down))
    {
     turn_tank_down(attackeris);
    }
  else if ((shouldgo.left==2) && (!obstacle_left))
    {
     turn_tank_left(attackeris);
    }
  else if ((shouldgo.right==2) && (!obstacle_right))
    {
     turn_tank_right(attackeris);
    }
  else  /* Probabilities must be just at 1, so that means         */
	/* there must be an obstacle in the way of the enemy tank */
    {
      if ((shouldgo.up) && (obstacle_up))
	 negotiate_obstacle(attackeris);
      else if ((shouldgo.down) && (obstacle_down))
	 negotiate_obstacle(attackeris);
      else if ((shouldgo.left) && (obstacle_left))
	 negotiate_obstacle(attackeris);
      else if ((shouldgo.right) && (obstacle_right))
	 negotiate_obstacle(attackeris);

     return;
    }

  add_buffer(attackeris,UPARROW);   /* Move forward towards    */
				    /* enemy after the turn    */
				    /* or obstacle negotiation */
}
/************************************************************************/
#define _TERMINTR
#endif
