
/* Kirk Christiansen */
/* TNKPICTS.C        */

/* This module contains code for loading in picture */
/* bitmaps and storing these bitmaps in memory.     */

#ifndef _TNKPICTS
/**************************************************************************/
#include <stdio.h>
#include "tankvars.c"
/**************************************************************************/
/* Function Prototypes */
void load_in_tankgame_sprites (void);
void load_in_tank_bitmaps (void);
void load_in_stat_bitmaps (void);
void load_in_character_pict_bitmaps (void);
void load_in_misc_picts_bitmaps (void);
void load_in_board_picts_bitmaps (void);
void load_in_mini_board_picts_bitmaps (void);
void release_tankgame_sprites (void);
void release_menu_files_allocated_memory (void);
/**************************************************************************/
void load_in_tankgame_sprites (void)
/* This function simply loads into memory the picture files desired */
{
 int i;

  /* Load in the primary file for the tankgame misc sprites */
  if (loadPCX(pcx_file_names[MYTANKS],&pcx[MYTANKS]))
     {
	printf(" ERROR loading .DAT file #%d\n",MYTANKS);
	sound(300);delay(100);nosound();delay(1250);
	pcx_files_loaded=FALSE;
	return;
     }

   /* Now slice & dice the input pcx picture files into separate */
   /* bitmap arrays.  Call functions to do this now.             */
   load_in_tank_bitmaps ();
   load_in_stat_bitmaps ();
   load_in_character_pict_bitmaps ();
   load_in_misc_picts_bitmaps ();
   load_in_board_picts_bitmaps ();

   farfree(pcx[MYTANKS].image);
}
/**************************************************************************/
void load_in_tank_bitmaps (void)
{
  int i;

   /* Load in the first half of the tank picture array */
   /* That is...the green and gray Square tanks */
    for (i=0;i<16; i++)
  {
   /* Allocate memory for each individual picture */
   tank_picts[i]=(char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*17)+1),1,15,15,pcx[MYTANKS].image,tank_picts[i]);
  }

 /* Now, Load in the second half of the tank picture array */
 /* the red Square tanks, and the Offgray HUNTERKILLERS */
 for (i=0;i<16; i++)
  {
   /* Allocate memory for each individual picture */
   tank_picts[i+16]=
	    (char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*17)+1),18,15,15,
	      pcx[MYTANKS].image,tank_picts[i+16]);
  }

 /* NOW load in the secondary file for the tank sprites */
  if (loadPCX(pcx_file_names[TANKMSC2],&pcx[TANKMSC2]))
     {
	printf(" ERROR loading .DAT file #%d\n",TANKMSC2);
	sound(300);delay(100);nosound();delay(3000);
	pcx_files_loaded=FALSE;
	return;
     }

 /* Load in the green + red HUNTERKILLERS */
 for (i=0;i<16; i++)
  {
   /* Allocate memory for each individual picture */
   tank_picts[i+32]=
	    (char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*17)+1),139,15,15,pcx[TANKMSC2].image,tank_picts[i+32]);
  }

 /* Load in the green + red GUNTANKS */
 for (i=0;i<16; i++)
  {
   /* Allocate memory for each individual picture */
   tank_picts[i+48]=
	    (char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*17)+1),156,15,15,pcx[TANKMSC2].image,tank_picts[i+48]);
  }

 /* Load in the Offgray GUNTANKS */
 for (i=0;i<8; i++)
  {
   /* Allocate memory for each individual picture */
   tank_picts[i+64]=
	    (char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*17)+1),173,15,15,pcx[TANKMSC2].image,tank_picts[i+64]);
  }

 /* Load in the Circle floater */
   tank_picts[72]= (char far *)farcalloc(15*15,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(137,173,15,15,pcx[TANKMSC2].image,tank_picts[72]);

 /* Release the secondary tank sprite PCX file image */
 farfree(pcx[TANKMSC2].image);

}
/**************************************************************************/
void load_in_stat_bitmaps (void)
{
 int i;

  /* Load first two small stat picts in 'Level' & 'Armor' */
  for (i=0;i<2; i++)
   {
      /* Allocate memory for each text picture */
      stat_picts[i]=(char far *)farcalloc(33*9,sizeof(unsigned char));
      /* Cut & Paste each picture into its own array cell */
      move_image(((i*34)+1),35,33,9,pcx[MYTANKS].image,stat_picts[i]);
   }
  /* Now, load in BIG 'LEVEL' */
   stat_picts[2]=(char far *)farcalloc(105*29,sizeof(unsigned char));
   move_image(1,69,105,29,pcx[MYTANKS].image,stat_picts[2]);

  /* and the BIG number array */
   for (i=0;i<10;i++)
    {
     stat_picts[3+i]=(char far *)farcalloc(19*29,sizeof(unsigned char));
     /* Cut & Paste each BIG number picture into its own array cell */
     move_image((i*20+107),69,19,29,pcx[MYTANKS].image,stat_picts[3+i]);
    }

  /* Now load in the stat boxes that go on the bottom of the screen */
  /* Load in the PCX file for the status bars */
     if (loadPCX(pcx_file_names[TANKBARS],&pcx[TANKBARS]))
	{
	  printf(" ERROR loading .DAT file #%d\n",TANKBARS);
	  sound(300);delay(100);nosound();delay(3000);
	  pcx_files_loaded=FALSE;
	  return;
	}

  /* Now allocate mem for them */
  /* load in the FULLGAME stat bar */
    stat_picts[13]=(char far *)farcalloc(320*30,sizeof(unsigned char));
    move_image(0,0,320,30,pcx[TANKBARS].image,stat_picts[13]);
  /* load in dual/match statbar */
    stat_picts[14]=(char far *)farcalloc(320*30,sizeof(unsigned char));
    move_image(0,35,320,30,pcx[TANKBARS].image,stat_picts[14]);
  /* load in TWOPLYR_FULLGAME statbar */
    stat_picts[15]=(char far *)farcalloc(320*30,sizeof(unsigned char));
    move_image(0,75,320,30,pcx[TANKBARS].image,stat_picts[15]);

  /* Now load in the 6 mini weapon abbreviation blocks (in blue&black) */
  for (i=0;i<6;i++)
   {
    stat_picts[16+i]=(char far *)farcalloc(9*8,sizeof(unsigned char));
    move_image(1+(i*10),110,9,8,pcx[TANKBARS].image,stat_picts[16+i]);
   }

  /* Load in the weapon text sprites ('Normal', 'Piercers', 'Rebounders' */
   stat_picts[22]=(char far *)farcalloc(36*7,sizeof(unsigned char));
   move_image(1,119,36,7,pcx[TANKBARS].image,stat_picts[22]);
  /* 'Piercers' */
   stat_picts[23]=(char far *)farcalloc(43*7,sizeof(unsigned char));
   move_image(1,127,43,7,pcx[TANKBARS].image,stat_picts[23]);
  /* 'Rebounders' */
   stat_picts[24]=(char far *)farcalloc(59*7,sizeof(unsigned char));
   move_image(1,135,59,7,pcx[TANKBARS].image,stat_picts[24]);

  /* Load in 'GAME OVER' pict */
   stat_picts[25]=(char far *)farcalloc(103*16,sizeof(unsigned char));
   move_image(61,105,103,16,pcx[TANKBARS].image,stat_picts[25]);
  /* Load in 'YOU DID IT' pict */
   stat_picts[26]=(char far *)farcalloc(94*16,sizeof(unsigned char));
   move_image(165,105,94,16,pcx[TANKBARS].image,stat_picts[26]);
  /* Load in 'WINS' pict */
   stat_picts[27]=(char far *)farcalloc(47*16,sizeof(unsigned char));
   move_image(261,105,47,16,pcx[TANKBARS].image,stat_picts[27]);
  /* Load in 'PLAYER' pict */
   stat_picts[28]=(char far *)farcalloc(70*16,sizeof(unsigned char));
   move_image(214,122,70,16,pcx[TANKBARS].image,stat_picts[28]);
  /* Load in '1' pict */
   stat_picts[29]=(char far *)farcalloc(15*16,sizeof(unsigned char));
   move_image(285,122,15,16,pcx[TANKBARS].image,stat_picts[29]);
  /* Load in '2' pict */
   stat_picts[30]=(char far *)farcalloc(15*16,sizeof(unsigned char));
   move_image(301,122,15,16,pcx[TANKBARS].image,stat_picts[30]);
  /* Load in 'You beat' pict */
   stat_picts[31]=(char far *)farcalloc(69*14,sizeof(unsigned char));
   move_image(182,143,69,14,pcx[TANKBARS].image,stat_picts[31]);
  /* Load in 'Tankgame' pict */
   stat_picts[32]=(char far *)farcalloc(74*14,sizeof(unsigned char));
   move_image(245,164,74,14,pcx[TANKBARS].image,stat_picts[32]);
  /* Load in 'DRAW' pict */
   stat_picts[33]=(char far *)farcalloc(54*14,sizeof(unsigned char));
   move_image(252,143,54,14,pcx[TANKBARS].image,stat_picts[33]);

  /* Now release TANKBARS .image memory */
   farfree(pcx[TANKBARS].image);
}
/**************************************************************************/
void load_in_character_pict_bitmaps (void)
/* The letters and numbers used in Tankgame */
{
 int i;

  for (i=0;i<NUMOFCHARPICTS;i++)
   {
    /* Allocate memory for each number picture */
   character_picts[i]=(char far *)farcalloc(7*9,sizeof(unsigned char));
   /* Cut & Paste each picture into its own array cell */
   move_image(((i*8)+1),45,7,9,pcx[MYTANKS].image,character_picts[i]);
   }
}
/**************************************************************************/
void load_in_misc_picts_bitmaps (void)
{
 int i;
 /* First, load in the small explosion picture */
     /* Allocate memory for the picture */
   misc_picts[0]=(char far *)farcalloc(7*7,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(1,55,7,7,pcx[MYTANKS].image,misc_picts[0]);

 /* Now load in the big explosion picture */
    /* Allocate memory for the big explosion picture */
    misc_picts[1]=(char far *)farcalloc(15*13,sizeof(unsigned char));
    /* Paste it into its own array cell */
    move_image(9,55,15,13,pcx[MYTANKS].image,misc_picts[1]);

 /* Now load in the POWER-UP pictures */
    for (i=2;i<NUMOFMISCPICTS;i++)
    {
     /* Allocate memory for the big explosion picture */
     misc_picts[i]=(char far *)farcalloc(10*13,sizeof(unsigned char));
     /* Paste it into its own array cell */
     move_image(81+(11*(i-2)),55,10,13,pcx[MYTANKS].image,misc_picts[i]);
    }

 /* Now load in the PROJ pictures */
    for (i=0;i<NUMOFPROJPICTS;i++)
    {
     /* Allocate memory for the big explosion picture */
     proj_picts[i]=(char far *)farcalloc(3*3,sizeof(unsigned char));
     /* Paste it into its own array cell */
     move_image((205+(i*4)),55,3,3,pcx[MYTANKS].image,proj_picts[i]);
    }
}
/**************************************************************************/
void load_in_board_picts_bitmaps (void)
{
 /* First load in the city building picts */
     /* Allocate memory for the 1st city building picture */
   board_picts[0]=(char far *)farcalloc(60*40,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(1,133,60,40,pcx[MYTANKS].image,board_picts[0]);
     /* Allocate memory for the 2nd city building picture */
   board_picts[1]=(char far *)farcalloc(35*30,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(63,133,35,30,pcx[MYTANKS].image,board_picts[1]);
 /* Load in pillar picture */
     /* Allocate memory for the pillar picture */
   board_picts[2]=(char far *)farcalloc(20*15,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(100,133,20,15,pcx[MYTANKS].image,board_picts[2]);
 /* Load in long bldg picture */
     /* Allocate memory for the bldg picture */
   board_picts[3]=(char far *)farcalloc(30*55,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(122,133,30,55,pcx[MYTANKS].image,board_picts[3]);
 /* Load in the bridge picture */
     /* Allocate memory for the bridge picture */
   board_picts[4]=(char far *)farcalloc(BRIDGEWIDTH*BRIDGEHEIGHT,
					sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(154,133,BRIDGEWIDTH,BRIDGEHEIGHT,
	      pcx[MYTANKS].image,board_picts[4]);

 /* Load in the river picture */
     /* Allocate memory for the river picture */
   board_picts[5]=(char far *)farcalloc(RIVERWIDTH*RIVERHEIGHT,
				       sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(205,99,RIVERWIDTH,RIVERHEIGHT,
	     pcx[MYTANKS].image,board_picts[5]);

 /* Load in the teleport ground picture */
     /* Allocate memory for the picture */
   board_picts[6]=(char far *)farcalloc(TELE_GROUND_WIDTH*TELE_GROUND_HEIGHT,
				       sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(63,165,TELE_GROUND_WIDTH,TELE_GROUND_HEIGHT,
	      pcx[MYTANKS].image,board_picts[6]);

 /* Load in the tree picture */
     /* Allocate memory for the picture */
   board_picts[7]=(char far *)farcalloc(TREE_WIDTH*TREE_HEIGHT,
				       sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(257,99,TREE_WIDTH,TREE_HEIGHT,
	      pcx[MYTANKS].image,board_picts[7]);

 /* Load in the dead tree picture */
     /* Allocate memory for the picture */
   board_picts[8]=(char far *)farcalloc(DEADTREE_WIDTH*DEADTREE_HEIGHT,
				       sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(257,122,DEADTREE_WIDTH,DEADTREE_HEIGHT,
	      pcx[MYTANKS].image,board_picts[8]);

 /* Load in the bush picture */
     /* Allocate memory for the picture */
   board_picts[9]=(char far *)farcalloc(BUSH_WIDTH*BUSH_HEIGHT,
				       sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(278,99,BUSH_WIDTH,BUSH_HEIGHT,
	      pcx[MYTANKS].image,board_picts[9]);

 /* NOW load in the secondary file for the board bitmaps */
  if (loadPCX(pcx_file_names[TANKMSC2],&pcx[TANKMSC2]))
     {
	printf(" ERROR loading .DAT file #%d\n",TANKMSC2);
	sound(300);delay(100);nosound();delay(3000);
	pcx_files_loaded=FALSE;
	return;
     }
 /* Load in the Arrow picts 'To level 9 & 11' */
  /* Allocate memory for the picture 'To Level 11'*/
   board_picts[10]=(char far *)farcalloc(51*24,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(66,93,51,24,pcx[TANKMSC2].image,board_picts[10]);
  /* Allocate memory for the picture 'To Level 9'*/
   board_picts[11]=(char far *)farcalloc(51*24,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(118,93,51,24,pcx[TANKMSC2].image,board_picts[11]);

  /* Allocate memory for the picture OliveDrab&Wood City bldg */
   board_picts[12]=(char far *)farcalloc(29*24,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(170,93,29,24,pcx[TANKMSC2].image,board_picts[12]);

  /* Allocate memory for the picture OliveDrab&Wood City bldg */
   board_picts[13]=(char far *)farcalloc(17*13,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(200,93,17,13,pcx[TANKMSC2].image,board_picts[13]);

  /* Allocate memory for the level 10 teleport box */
   board_picts[14]=(char far *)farcalloc(27*20,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(66,118,27,20,pcx[TANKMSC2].image,board_picts[14]);

  /* Allocate memory for the closing door */
   board_picts[15]=(char far *)farcalloc(13*29,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(218,93,13,29,pcx[TANKMSC2].image,board_picts[15]);

  /* Allocate memory for the horizontal medium-long block */
   board_picts[16]=(char far *)farcalloc(33*13,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(232,93,33,13,pcx[TANKMSC2].image,board_picts[16]);

  /* Allocate memory for the vertical medium-long block */
   board_picts[17]=(char far *)farcalloc(13*34,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(266,93,13,34,pcx[TANKMSC2].image,board_picts[17]);

  /* Allocate memory for the really long horizontal olivedrab block */
   board_picts[18]=(char far *)farcalloc(100*11,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(137,127,100,11,pcx[TANKMSC2].image,board_picts[18]);

  /* Allocate memory for the really long vertical lightgray block */
   board_picts[19]=(char far *)farcalloc(20*77,sizeof(unsigned char));
   /* Cut & Paste the picture into its own array cell */
   move_image(280,93,20,77,pcx[TANKMSC2].image,board_picts[19]);

 /* Release the secondary board bitmap PCX file image */
 farfree(pcx[TANKMSC2].image);
}
/**************************************************************************/
void load_in_mini_board_picts_bitmaps (void)
/* Loads in the small board pictures for the Select board menu */
{
 int x,y,i=0;

 /* First load in correct pcx file*/
  if (loadPCX(pcx_file_names[TANKMSC2],&pcx[TANKMSC2]))
    {
      printf(" ERROR loading .DAT file #%d\n",TANKMSC2);
      sound(300);delay(100);nosound();delay(3000);
      pcx_files_loaded=FALSE;
      return;
    }

 /* Load in the first 8 mini boards */
 for (y=1;y<92;y+=46)
  for (x=1;x<200;x+=65)
    {
     mini_board_picts[i]=(char far *)farcalloc(64*45,sizeof(unsigned char));
     move_image(x,y,64,45,pcx[TANKMSC2].image,mini_board_picts[i]);
     i++;
    }

 /* Make sure i=8 */
  i=8;

 /* Load in the last board which on its own line */
   mini_board_picts[i]=(char far *)farcalloc(64*45,sizeof(unsigned char));
   move_image(1,93,64,45,pcx[TANKMSC2].image,mini_board_picts[i]);

 /* release the 64k pcx image */
 farfree(pcx[TANKMSC2].image);
}
/**************************************************************************/
void load_in_menu_pcx_files (void)
{
 int i,x,y;

 /* First load in correct pcx file*/
  if (loadPCX(pcx_file_names[TANKMSC1],&pcx[TANKMSC1]))
    {
      printf(" ERROR loading .DAT file #%d\n",TANKMSC1);
      sound(300);delay(100);nosound();delay(3000);
      pcx_files_loaded=FALSE;
      return;
    }

/* Now load the buttons from the PCX file */
 i=1; /* first button */

 /* Load the first 4 buttons for gametypes */
 for (x=1;x<150;x+=49)
  {
     menu_picts[i]=(char far *)farcalloc(48*18,sizeof(unsigned char));
     move_image(x,1,48,18,pcx[TANKMSC1].image,menu_picts[i]);
     i++;
  }

 i=5; /* make sure i=5 */

 /* Load the next 9 bigger buttons */
   for (y=20;y<76;y+=19)
    for (x=1;x<110;x+=54)
    {
     menu_picts[i]=(char far *)farcalloc(53*18,sizeof(unsigned char));
     move_image(x,y,53,18,pcx[TANKMSC1].image,menu_picts[i]);
     i++;
    }

 /* Load the 5 big left side config buttons */
  i=14; /* make sure i=14 */

  for (y=1;y<175;y+=35)
   {
     menu_picts[i]=(char far *)farcalloc(100*34,sizeof(unsigned char));
     move_image(218,y,100,34,pcx[TANKMSC1].image,menu_picts[i]);
     i++;
   }

 /* Load in 'Use Arrow keys to move' pict */
   menu_picts[19]=(char far *)farcalloc(153*14,sizeof(unsigned char));
   move_image(1,77,153,14,pcx[TANKMSC1].image,menu_picts[19]);
 /* Load in 'Hit ENTER to Select' pict */
   menu_picts[20]=(char far *)farcalloc(130*12,sizeof(unsigned char));
   move_image(1,93,130,12,pcx[TANKMSC1].image,menu_picts[20]);
 /* Load in 'Misc' pict */
   menu_picts[21]=(char far *)farcalloc(30*12,sizeof(unsigned char));
   move_image(132,93,30,12,pcx[TANKMSC1].image,menu_picts[21]);
 /* Load in 'Tankgame Information' pict */
   menu_picts[22]=(char far *)farcalloc(149*15,sizeof(unsigned char));
   move_image(1,106,149,15,pcx[TANKMSC1].image,menu_picts[22]);
 /* Load in Tank Sprite pict */
   menu_picts[23]=(char far *)farcalloc(87*37,sizeof(unsigned char));
   move_image(1,122,87,37,pcx[TANKMSC1].image,menu_picts[23]);
 /* Load in the key ESC */
     menu_picts[24]=(char far *)farcalloc(25*24,sizeof(unsigned char));
     move_image(89,122,25,24,pcx[TANKMSC1].image,menu_picts[24]);
 /* Load in the key F1 */
     menu_picts[25]=(char far *)farcalloc(25*24,sizeof(unsigned char));
     move_image(115,122,25,24,pcx[TANKMSC1].image,menu_picts[25]);

 /* LOAD in the Duel menu options */
 /* Load in the 'NO' sprite */
     menu_picts[26]=(char far *)farcalloc(25*11,sizeof(unsigned char));
     move_image(116,147,25,11,pcx[TANKMSC1].image,menu_picts[26]);
 /* Load in the 'YES' sprite */
     menu_picts[27]=(char far *)farcalloc(25*11,sizeof(unsigned char));
     move_image(89,147,25,11,pcx[TANKMSC1].image,menu_picts[27]);
 /* Load in the 'DUEL OPTIONS' sprite */
     menu_picts[28]=(char far *)farcalloc(175*20,sizeof(unsigned char));
     move_image(1,160,175,20,pcx[TANKMSC1].image,menu_picts[28]);
 /* Load in the 'Invisible Tanks' sprite */
     menu_picts[29]=(char far *)farcalloc(96*14,sizeof(unsigned char));
     move_image(1,181,96,14,pcx[TANKMSC1].image,menu_picts[29]);
 /* Load in the 'Power Ups' sprite */
     menu_picts[30]=(char far *)farcalloc(69*14,sizeof(unsigned char));
     move_image(98,181,69,14,pcx[TANKMSC1].image,menu_picts[30]);
 /* Load in the 'Night Fight' sprite */
     menu_picts[31]=(char far *)farcalloc(68*14,sizeof(unsigned char));
     move_image(168,181,68,14,pcx[TANKMSC1].image,menu_picts[31]);
 /* Load in the 'Display Stats' sprite */
     menu_picts[32]=(char far *)farcalloc(81*14,sizeof(unsigned char));
     move_image(237,181,81,14,pcx[TANKMSC1].image,menu_picts[32]);

  /* Load in 'Easy','Medium','Hard' Picts */
     for (i=0;i<3;i++)
      {
       menu_picts[33+i]=(char far *)farcalloc(66*14,sizeof(unsigned char));
       move_image(151,106+(i*15),66,14,pcx[TANKMSC1].image,menu_picts[33+i]);
      }

  farfree(pcx[TANKMSC1].image);

/* Load in TankBars PCX file to get some more menu sprites */
  if (loadPCX(pcx_file_names[TANKBARS],&pcx[TANKBARS]))
    {
      printf(" ERROR loading .DAT file #%d\n",TANKBARS);
      sound(300);delay(100);nosound();delay(3000);
      pcx_files_loaded=FALSE;
      return;
    }

  /* Load in 'SET DIFFICULTY' PICT */
     menu_picts[36]=(char far *)farcalloc(180*20,sizeof(unsigned char));
     move_image(1,143,180,20,pcx[TANKBARS].image,menu_picts[36]);

  /* Load in 'CHOOSE YOUR' PICT */
     menu_picts[37]=(char far *)farcalloc(176*20,sizeof(unsigned char));
     move_image(1,164,176,20,pcx[TANKBARS].image,menu_picts[37]);

  /* Load in 'Player 1' PICT */
     menu_picts[38]=(char far *)farcalloc(61*15,sizeof(unsigned char));
     move_image(1,185,61,15,pcx[TANKBARS].image,menu_picts[38]);

  /* Load in 'Player 1' PICT */
     menu_picts[39]=(char far *)farcalloc(63*15,sizeof(unsigned char));
     move_image(63,185,63,15,pcx[TANKBARS].image,menu_picts[39]);

  /* Load in 'TANK' PICT */
     menu_picts[40]=(char far *)farcalloc(66*20,sizeof(unsigned char));
     move_image(178,164,66,20,pcx[TANKBARS].image,menu_picts[40]);

  /* Load in 'GAME SOUND' PICT */
     menu_picts[41]=(char far *)farcalloc(152*20,sizeof(unsigned char));
     move_image(61,122,152,20,pcx[TANKBARS].image,menu_picts[41]);

  /* Load in 'SOUND :' PICT */
     menu_picts[42]=(char far *)farcalloc(75*15,sizeof(unsigned char));
     move_image(127,185,75,15,pcx[TANKBARS].image,menu_picts[42]);

  /* Load in 'ON' PICT */
     menu_picts[43]=(char far *)farcalloc(25*14,sizeof(unsigned char));
     move_image(203,185,25,14,pcx[TANKBARS].image,menu_picts[43]);

  /* Load in 'OFF' PICT */
     menu_picts[44]=(char far *)farcalloc(33*14,sizeof(unsigned char));
     move_image(229,185,33,14,pcx[TANKBARS].image,menu_picts[44]);

 /* release the 64k pcx images */
  farfree(pcx[TANKBARS].image);

 /* LOAD IN THE MINI BOARD PICTS FOR THE SELECT BOARD SCREEN */
 load_in_mini_board_picts_bitmaps();

/* Lastly, Load in the big menu PCX file */
  if (loadPCX(pcx_file_names[TANKMENU],&pcx[TANKMENU]))
    {
      printf(" ERROR loading .DAT file #%d\n",TANKMENU);
      sound(300);delay(100);nosound();delay(3000);
      pcx_files_loaded=FALSE;
      return;
    }
}
/**************************************************************************/
void release_menu_files_allocated_memory (void)
{
  int i;

 /* Release the memory from the Menu PCX file image, I had to wait to */
 /* release this cuz it was constantly blitted during the menu operation */
   farfree(pcx[TANKMENU].image);

   /* Now deallocate all the individual button,etc sprites */
   for (i=1;i<NUMOFMENUPICTS+1; i++) /* +1 used cuz I started at 1 */
      farfree(menu_picts[i]);        /* for button number 1.       */

   /* Now deallocate all the mini board sprites */
   for (i=0;i<NUMOFMINIBOARDPICTS; i++)
      farfree(mini_board_picts[i]);
}
/**************************************************************************/
void release_tankgame_sprites (void)
/* This function simply releases any previous allocated memory */
/* from the pcx graphic bitmaps.  NOTE: This function just deallocates */
/* all possible sprites.  Some may have already been deallocated.      */
/* This does not include the menu sprites.                             */
{
 int i;

   for (i=0;i<NUMPCXFILES;i++)
     farfree(pcx[i].image);

    for (i=0;i<NUMSTATPICTS; i++)
     farfree(stat_picts[i]);

   for (i=0;i<NUMTANKPICTS; i++)
     farfree(tank_picts[i]);

   for (i=0;i<NUMOFCHARPICTS; i++)
     farfree(character_picts[i]);

   for (i=0;i<NUMOFMISCPICTS; i++)
     farfree(misc_picts[i]);

   for (i=0;i<NUMOFPROJPICTS; i++)
     farfree(proj_picts[i]);

   for (i=0;i<NUMOFBOARDPICTS; i++)
     farfree(board_picts[i]);
}
/**************************************************************************/
#define _TNKPICTS
#endif