
/* Kirk Christiansen        */
/* TANKPROJS.C              */

/* This file contains the   */
/* functions for projectile */
/* movement and collision   */
/* detection.               */

#ifndef _TNKPROJS
/**************************************************************************/
#include <dos.h>          /* for sound functions */
#include "kcgrphcs.c"
#include "tankvars.c"
#include "drawtank.c"
#include "weapons.c"
#include "miscfncs.c"  /* set expl*/
#include "kc_pcsnd.c"
/**************************************************************************/
int  isinvalidcolor(int color);
void fireprojectile (int tanknum);
void draw_a_projectile(int x, int y,int typeis, int color);
void drawprojectiles (void);
void adjustprojectiles(void);
int  checkforhit(int prjx, int prjy, int tankx, int tanky);
void check_for_projhits(void);
void erase_a_tanks_projectiles(int tanknum);
void get_new_proj_coordinates (int x, int y, int dir, int *newx, int *newy);
void clear_a_proj_path (int x1, int y1, int x2, int y2, int dir);
int  check_for_valid_rebound (int x, int y, int dir);
int  reflection_direction (int x, int y, int dir);
/**************************************************************************/
int isinvalidcolor(int color)
{
 int i;

 /* Check to see if inputed color is an obstacle color */
 for (i=0;i<MAX_OBSTACLE_PROJ_CLRS;i++)
  {
   if (color==obstacle_proj_color_array[i])
     return TRUE;
  };

 /* need to add situation where projectile hits projectile */
 return FALSE;
}
/***************************************************************************/
void fireprojectile (int tanknum)
{
 int prjx,prjy;   /* projectile's x and y coordinates */
 int x,y,i;
 int weapon;
 int fired=FALSE; /* when (fired == TRUE) a projectile has been launched */

 /* Set weapon equal to the current weapon the tank is using */
 weapon=tanks[tanknum].current_weapon;

 switch (tanks[tanknum].pos)   /* direction tank is facing */
   {  /* if tank is pointing up, then x doesn't change, but y does */
    case UP:    prjx = (tanks[tanknum].xcor);
		prjy = (tanks[tanknum].ycor - (TSMUL1_5));
		break;
    case UPRIGHT:prjx = (tanks[tanknum].xcor)+(TSMUL1_5);
		prjy = (tanks[tanknum].ycor - (TSMUL1_5));
		break;
    case RIGHT: prjx = (tanks[tanknum].xcor + (TSMUL1_5));
		prjy = (tanks[tanknum].ycor);
		break;
    case DOWNRIGHT:prjx = (tanks[tanknum].xcor)+(TSMUL1_5);
		  prjy = (tanks[tanknum].ycor + (TSMUL1_5));
		  break;
    case DOWN:  prjx = (tanks[tanknum].xcor);
		prjy = (tanks[tanknum].ycor + (TSMUL1_5));
		break;
    case DOWNLEFT:prjx = (tanks[tanknum].xcor)-(TSMUL1_5);
		  prjy = (tanks[tanknum].ycor + (TSMUL1_5));
		  break;
    case LEFT:  prjx = (tanks[tanknum].xcor - (TSMUL1_5));
		prjy = (tanks[tanknum].ycor);
		break;
    case UPLEFT:prjx = (tanks[tanknum].xcor)-(TSMUL1_5);
		prjy = (tanks[tanknum].ycor - (TSMUL1_5));
		break;
   };


  /* Check the radius to see if firing of projectile is blocked */
  for (x=(prjx-(PROJBOUND));x<=(prjx+(PROJBOUND));x++)
   {
    for (y=(prjy-(PROJBOUND)); y<=(prjy+(PROJBOUND)) ;y++)
    {
     if (isinvalidcolor(getpixel(x,y,screen_buffer)))
	 {
	    if (soundon)
	     make_sound(ILLEGAL_MOVE);

	    /* Check to see if an invisible tank fired */
	    if (tanks[tanknum].tankstats[VISIBILITYSTATUS]==INVISIBLE)
	      drawtank(tanknum,tanks[tanknum].colors[0]);

	    return;  /* exit the function */
	 }
     }
    }

     /* Now, Is there an open space to store the launched projectile? */
  for (i=0; ((i<MAXPROJS) && (!fired)); i++)
   {
	 if ((tanks[tanknum].projs[i].dir) == 0) /* Is empty space open for  */
	{             		            /* projectile's information */
	  tanks[tanknum].projs[i].dir = tanks[tanknum].pos;  /* set the       */
	  tanks[tanknum].projs[i].x = prjx;         /* projectile's  */
	  tanks[tanknum].projs[i].y = prjy;         /* direction and x,y */

	  /* Now, set the proj_type according to weapon tank posesses */
	  tanks[tanknum].projs[i].type = weapon;

	  /* Set the time the projectile started */
	  tanks[tanknum].projs[i].time = current_time;

	  /* Decrement the ammo counter */
	  if (weapon != NORMAL_PROJ)
	   tanks[tanknum].weapon_ammo[weapon]--;

	  /* Check to see if tank ran out of ammo */
	  if ((tanks[tanknum].weapon_ammo[weapon]==0) &&
	     (weapon != NORMAL_PROJ))
	   {
	     /* auto select a new weapon */
	     change_current_weapon(tanknum);

	     /* erase abbreviation */
	     display_abbreviation(tanknum,weapon,BLACK);
	   }

	  fired=TRUE; /* terminates loop */
	 }
    }

 /* If the tank is invisible, show him to the opponent */
 if (tanks[tanknum].tankstats[VISIBILITYSTATUS]==INVISIBLE)
    drawtank(tanknum,tanks[tanknum].colors[0]);
}
/***************************************************************************/
void draw_a_projectile(int x, int y, int typeis, int color)
/* This function simply draws a projectile at the inputed */
/* x,y coordinates.  It draws it with a radius of 2.      */
{
   if (color==BLACK)
    bar_blit(x-1,y-1,3,3,BLACK,screen_buffer);
   else if (color==YELLOW)
    fullblit(x-1,y-1,3,3,screen_buffer,proj_picts[typeis]);
   else if (color==LIGHTRED)
    fullblit(x-1,y-1,3,3,screen_buffer,proj_picts[typeis+4]);
   else if (color==CYAN)
    fullblit(x-1,y-1,3,3,screen_buffer,proj_picts[typeis+8]);
}
/***************************************************************************/
void drawprojectiles (void)
/*  This function draws the current inputed tank's projectiles  */
/* on the screen, it checks to see what color the projectiles   */
/* will be when they are placed on to the screen.               */
{
 int i,j;
 int pcolor;  /* projectile color */

 for (i=0; i<MAXTANKS; i++)
 {
   if (tanks[i].armor > 0)  /* the tank is alive */
    {
       /* set the correct projectile color */
	 pcolor=tanks[i].projcolor;

       /* See if the tank has fired some projectiles */
      for (j=0;j<MAXPROJS;j++)
       {
	if (tanks[i].projs[j].dir)    /* a projectile exists */
	  {
	   draw_a_projectile((tanks[i].projs[j].x),
			     (tanks[i].projs[j].y),
			     (tanks[i].projs[j].type),
			     pcolor);
	  }
       }
    } /* if (tanks[i].armor > 0) */
 } /* end for (i=0;i<MAXTANKS;i++) */
}
/**************************************************************************/
void adjustprojectiles(void)
/* This function adjusts the current existing projectiles */
/* by erasing them and redrawing them if they can move or */
/* destroying them if they can not.                       */
{
 int num,j,x,y;
 int projgone;    /* boolean variable that tells whether a projectile */
		  /* has crashed.                                     */
 int pcolor;      /* color of the projectile */
 int newx, newy;  /* the coordinates to which the projectile will move */


  for (num=0;num<MAXTANKS;num++)
  {
   if (tanks[num].tankstats[GAMESTATUS])  /* tank is alive or just killed */
     {
      /* set the appropiate projectile color */
      pcolor = tanks[num].projcolor;

      for (j=0;j<MAXPROJS;j++)
       {
	 projgone=FALSE;

	  if (tanks[num].projs[j].dir)    /* a projectile exists */
	    {
	     /* erase the projectile */
	     draw_a_projectile((tanks[num].projs[j].x),
			       (tanks[num].projs[j].y),
			       (tanks[num].projs[j].type),
				BLACK);


	    if ((current_time - tanks[num].projs[j].time)
		 <  PROJ_EXIST_TIME) /* if it still has time left */
	     {
	     /* calculate the new coordinates */
	     get_new_proj_coordinates (tanks[num].projs[j].x,
				       tanks[num].projs[j].y,
				       tanks[num].projs[j].dir,
				       &newx,&newy);

	     /* check the new coordinates by using PROJBOUND */
	     for (x=(newx-(PROJBOUND));((x <= (newx+(PROJBOUND))) &&
		 (!projgone));x+=PROJBOUND)
	       {

		for (y=(newy-(PROJBOUND));((y <= (newy+(PROJBOUND))) &&
		    (!projgone));y+=PROJBOUND)
		 {

		  if (isinvalidcolor(getpixel(x,y,screen_buffer)))
		   {
		     /* If its a rebounder, calculate new direction */
		    if (tanks[num].projs[j].type==REBOUNDING_PROJ)
		     {
		      /* Calculate the angle of reflection */
		       tanks[num].projs[j].dir=
			  reflection_direction(tanks[num].projs[j].x,
					       tanks[num].projs[j].y,
					       tanks[num].projs[j].dir);
		      /* Calculate the newx & newy coordinates */
		       get_new_proj_coordinates (tanks[num].projs[j].x,
						 tanks[num].projs[j].y,
						 tanks[num].projs[j].dir,
						 &newx, &newy);

		      /* set projgone (projectile gone) to TRUE, so */
		      /* we don't continue checking invalid coordinates */
		      projgone = TRUE;
		     }

		    /* If it's a piercer shell, set time so it moves */
		    /* one more space to pierce into a wall.         */
		    else if (tanks[num].projs[j].type==PIERCING_PROJ)
		     {
		       clear_a_proj_path (newx,newy,tanks[num].projs[j].x,
					 tanks[num].projs[j].y,
					 tanks[num].projs[j].dir);

		      /* Now set the time over limit. This will let the */
		      /* proj be drawn one more step, because draw_projs */
		      /* comes after adjust_projs().                     */
		      tanks[num].projs[j].time -= PROJ_EXIST_TIME;

		      /* sound a crashed projectile */
		       if (soundon)
			make_sound(CRASHED_PROJ);

		      /* set projgone (projectile gone) to TRUE, so */
		      /* we don't continue checking invalid coordinates */
		      projgone = TRUE;
		     }

		    else  /* not a rebounder, it blows up */
		     {
		      /* sound a crashed projectile */
		      if (soundon)
		       make_sound(CRASHED_PROJ);

		      /* projectile crashed so reset it */
		      tanks[num].projs[j].dir=0;
		      tanks[num].projs[j].x=0;
		      tanks[num].projs[j].y=0;
		      tanks[num].projs[j].type=0;
		      tanks[num].projs[j].time=0;

		      /* set projgone (projectile gone) to TRUE */
		      projgone = TRUE;
		     }
		   }
		 }
	       }

	      if (tanks[num].projs[j].dir) /* projectile did not crash so */
		{                          /* save its new coordinates */
		 tanks[num].projs[j].x=newx;
		 tanks[num].projs[j].y=newy;
		 draw_a_projectile((tanks[num].projs[j].x),
				   (tanks[num].projs[j].y),
				   (tanks[num].projs[j].type),
				   pcolor);

		  /* decrement the time */
		 // don't need this anymore since I'm using real time;
		}
	   } /* end if it still has time left */

	   else  /* NO TIME LEFT, so reset it. */
	   {
	      tanks[num].projs[j].dir=0;
	      tanks[num].projs[j].x=0;
	      tanks[num].projs[j].y=0;
	      tanks[num].projs[j].type=0;
	      tanks[num].projs[j].time=0;
	   }

	       } /* end if a projectile exists */
       } /* end for loop */
     } /* end if tank is alive or just killed */
  } /* end for (num=0;num<MAXTANKS;num++) */
}
/**************************************************************************/
int checkforhit(int prjx, int prjy, int tankx, int tanky)
/*  This function takes in as input a projectile's coordinates,    */
/* the tank that fired it, and the tank it is aimed at. It checks  */
/* to see if it scores a hit on the target tank at (tankx,tanky)   */
{
 int x,y;

 for (y = (prjy - PROJRADIUS); y <= (prjy + PROJRADIUS); y+=PROJRADIUS)
   for (x = (prjx - PROJRADIUS); x <= (prjx + PROJRADIUS); x+=PROJRADIUS)
	  {
	     if ( (((tankx-TS) <= x) &&  (x <= (tankx+TS))) &&
		  (((tanky-TS) <= y) &&  (y <= (tanky+TS))) )
		 {
		  /* the projectile has struck the tank */
		   return 1;
		 }
	  }

 return 0;  /* a miss */
}
/**************************************************************************/
void check_for_projhits(void)
/*  This function checks atank's (attacking tank) projectiles to see */
/* if they have recorded a hit to ttank's (target tank) body.        */
{
 int atank,ttank,i,j;
 int projectilehit = FALSE; /* TRUE means a hit was scored */


 for (atank=0;atank<MAXTANKS;atank++) /* Go through all the attackers */
 {
  if (tanks[atank].tankstats[GAMESTATUS]) /*if they are alive or just dead */
  {
   for (i=0;i<MAXPROJS;i++) /* Go through all their projectiles */
   {
     if (tanks[atank].projs[i].dir)   /* if a projectile exists */
      {
	for (ttank=0;ttank<MAXTANKS;ttank++) /* check all target tanks */
	{
	 if ((ttank!=atank)&&(tanks[ttank].tankstats[GAMESTATUS]==PLAYING))
	  { /* you can't shoot yourself...yet, or a dead tank */
	   projectilehit= checkforhit(tanks[atank].projs[i].x,
				      tanks[atank].projs[i].y,
				      tanks[ttank].xcor,tanks[ttank].ycor);

	     if (projectilehit)
	      {
		tanks[ttank].armor -= 1;/* record the damage to target tank */

		  if (soundon)
		   make_sound(SMALL_EXPLOSION);

		  /* set an explosion in the array */
		  set_explosion (tanks[atank].projs[i].x,
				 tanks[atank].projs[i].y,
				 PROJ_EXPLOSION);

		  /* erase projectile */
		  draw_a_projectile(tanks[atank].projs[i].x,
				   tanks[atank].projs[i].y,
				   (tanks[atank].projs[i].type),
				   BLACK);

		  // IS THIS NEEDED???????????
		  /* draw the tank again */
		  drawtank(ttank,tanks[ttank].colors[0]);

		  /* reset the projectile */
		  tanks[atank].projs[i].dir=0;
		  tanks[atank].projs[i].x=0;
		  tanks[atank].projs[i].y=0;
		  tanks[atank].projs[i].type=0;
		  tanks[atank].projs[i].time=0;

		  /* exit the function, NOTE: if two projectiles can hit */
		  /* a tank at once (e.g bouncing projectiles) then this */
		  /* return will need to be eliminated.                  */
		  return;
	      }  /* end if projectile hit something */
	 } /* end if (attank!=ttank) && (tanks[ttank].tankstatus==ALIVE)*/
	} /* end for (ttank=0;ttank<MAXTANKS;ttank++) */
      } /* end if a projectile exists */
   } /* end for (i=0;i<MAXPROJS;i++) */
  } /* end if the attack tank is alive or just dead */
 } /* end for (attank=0;atank<MAXTANKS;a++) */
}
/**************************************************************************/
void erase_a_tanks_projectiles(int tanknum)
/* This function erases a tank's(tankis) projectiles when it has */
/* been killed or something drastic has happened to it.          */
{
  int i;

  for (i=0;i<MAXPROJS;i++)
   {
     /* If a projectile exists, erase it */
     if (tanks[tanknum].projs[i].dir)
	{
	  draw_a_projectile(tanks[tanknum].projs[i].x,
			    tanks[tanknum].projs[i].y,
			    tanks[tanknum].projs[i].type,
			    BLACK);
	}
   }
}
/**************************************************************************/
void get_new_proj_coordinates (int x, int y, int dir, int *newx, int *newy)
/* This function takes in an x,y pair of coordinates and calculates */
/* a new set of coordinates based on a projectiles direction.       */
{
  switch (dir)
    {
     case UP: /* projectile moving upwards */
	  *newx = x; /* don't adjust x  */
	  *newy = y - PROJSPEED;
	  break;
     case UPRIGHT: /* projectile moving to  up and right */
	  *newx = x + PROJSPEED;
	  *newy = y - PROJSPEED;
	  break;
     case RIGHT: /* projectile moving to the right */
	  *newx = x + PROJSPEED;
	  *newy = y; /* don't adjust y */
	  break;
     case DOWNRIGHT:
	  *newx = x + PROJSPEED;
	  *newy = y + PROJSPEED;
	  break;
     case DOWN:
	  *newx = x; /* don't adjust x  */
	  *newy = y + PROJSPEED;
	  break;
     case DOWNLEFT:
	  *newx = x - PROJSPEED;
	  *newy = y + PROJSPEED;
	  break;
     case LEFT:
	  *newx = x - PROJSPEED;
	  *newy = y; /* don't adjust y */
	  break;
     case UPLEFT:
	  *newx = x -PROJSPEED;
	  *newy = y - PROJSPEED;
	  break;
    };
}
/**************************************************************************/
void clear_a_proj_path (int x1, int y1, int x2, int y2, int dir)
/* This function takes in two sets of coordinates and clears a path */
/* between them. It is primarly used for piercing projectiles which */
/* eat up walls when they strike them.                              */
{
  switch (dir)
  {
    case UP:
    case DOWN:  x1=x2;
	       thickline (x1,y1,x2,y2,BLACK,screen_buffer);
	       break;
    case RIGHT:
    case LEFT:  y1=y2;
	       thickline (x1,y1,x2,y2,BLACK,screen_buffer);
	       break;

    case UPRIGHT:
    case DOWNRIGHT:
    case DOWNLEFT:
    case UPLEFT:
		 line (x1,y1,x2,y2,BLACK,screen_buffer);
		 line (x1,y1-1,x2,y2-1,BLACK,screen_buffer);
		 line (x1,y1+1,x2,y2+1,BLACK,screen_buffer);
		 line (x1,y1-2,x2,y2-2,BLACK,screen_buffer);
		 line (x1,y1+2,x2,y2+2,BLACK,screen_buffer);
		 break;
  };
}
/**************************************************************************/
int check_for_valid_rebound (int xcor, int ycor, int dir)
{
 int x,y;
 int check_x = x;
 int check_y = y;

 /* Calculate new coordinates based on old x,y and dir (direction) */
 get_new_proj_coordinates (xcor,ycor,dir,&check_x,&check_y);

 /* Check the new coordinates */
  for (x=(check_x-(PROJBOUND));(x <= (check_x+(PROJBOUND)));x+=PROJBOUND)
    {
     for (y=(check_y-(PROJBOUND));(y <= (check_y+(PROJBOUND)));y+=PROJBOUND)
       {
	 if (isinvalidcolor(getpixel(x,y,screen_buffer)))
	   return FALSE;
       }
    }
 /* The testing phase passed without a return so it must be a safe place */
 return TRUE;
}
/**************************************************************************/
int reflection_direction (int x, int y, int dir)
{
  int check_dir_1;
  int check_dir_2;

  switch (dir)
  {
    case UP        : return DOWN;
    case RIGHT     : return LEFT;
    case DOWN      : return UP;
    case LEFT      : return RIGHT;


    /* If projectile is on an angle, you must check two coordinate */
    /* sets, because it can rebound two different ways, if both of */
    /* those ways are invalid, you must reflect it straight back   */
    /* the way it came.                                            */

    case UPRIGHT   : check_dir_1=DOWNRIGHT;
		     check_dir_2=UPLEFT;
		     break;

    case DOWNRIGHT : check_dir_1=UPRIGHT;
		     check_dir_2=DOWNLEFT;
		     break;

    case DOWNLEFT  : check_dir_1=UPLEFT;
		     check_dir_2=DOWNRIGHT;
		     break;

    case UPLEFT    : check_dir_1=DOWNLEFT;
		     check_dir_2=UPRIGHT;
		     break;

  }; /* end switch (dir) */

    /* Now check the two possible rebound directions */
    if (check_for_valid_rebound(x,y,check_dir_1))
       {
	return check_dir_1;
       }
    else
     {
       if (check_for_valid_rebound(x,y,check_dir_2))
	 {
	  return check_dir_2;
	 }

       /* None of the two are valid, so just rebound straight back */
       else
	  return opposite_of(dir);
     }
}
/**************************************************************************/
#define _TNKPROJS
#endif