
/* Kirk Christiansen */
/* WEAPONS.C         */

/* Tank Weapon Functions        */
/* Functions that control the   */
/* change of user selected tank */
/* weapons.                     */
/************************************************************************/
#ifndef _WEAPONS.C
/************************************************************************/
#include "tankvars.c"
#include "kcgrphcs.c"
/************************************************************************/
/* Function Prototypes */
void display_weapon_and_ammo (int plyrnum);
void display_weapon (int plyris, int weapon, int clr);
void display_ammo (int plyris, int weapon, int ammo, int clr);
void change_current_weapon(int tanknum);
void change_to_a_specific_weapon(int weapon,int tanknum);
void dehighlight_weapon (int plyris,int weapon);
void highlight_weapon (int plyris,int weapon);
void display_weapon_abbreviations(int plyris);
void display_abbreviation (int plyris, int num, int clr);
/************************************************************************/
void display_weapon_and_ammo (int plyrnum)
/* This function displays the current selected weapon that the player */
/* is using, and the amount of ammo it has left                       */
{
 int i;
 int weapon = tanks[plyrnum].current_weapon;

 /* First, print out "WEAPON" */
 // out_text("WEAPON",70,178,GREEN,screen_buffer);

 /* Lets now blit all the abbrevs for the weapons the tank has */
 /* First make sure N is dsplyed for normal shells, this is always TRUE */
   display_abbreviation (plyrnum, NORMAL_PROJ, OLIVEDRAB);
 /* Now display other weapon abbreviations if it has em */
  for (i=0;i<MAXWEAPONS;i++)
   {
    if (tanks[plyrnum].weapon_ammo[i])
       display_abbreviation (plyrnum, i, OLIVEDRAB);
   }

 /* Next, print out the name of the current selected weapon */
   display_weapon (plyrnum, weapon, OLIVEDRAB);

 /* Print out the highlighted box for the weapon */
   highlight_weapon (plyrnum, weapon);
   /* Next, print out "AMMO" */
   //out_text("AMMO",70,188,GREEN,screen_buffer);

   /* Finally, print out the number of ammo */
  display_ammo(plyrnum,weapon,tanks[plyrnum].weapon_ammo[weapon],OLIVEDRAB);
}
/***************************************************************************/
void display_weapon (int plyris, int weapon, int clr)
{
  int xoff,yoff;

  /* exit if not wanted */
  if (!display_stats)
    return;

   switch (gametype)
   {
    case FULLGAME : xoff=177;yoff=174; break;
    default  : /* anything but FULLGAME */
			    if (plyris==0)
			      xoff=47;
			    else
			      xoff=222;

			    yoff=188; break;
   }


   if (clr==BLACK)
    {
      /* erase the old weapon & return */
	bar_blit(xoff,yoff,59,7,BLACK,screen_buffer);
      return;
    }

   switch (weapon)
   {
     case NORMAL_PROJ:
	      fullblit(xoff,yoff,36,7,screen_buffer,stat_picts[22]);
	      break;
     case PIERCING_PROJ:
	      fullblit(xoff,yoff,43,7,screen_buffer,stat_picts[23]);
	      break;
     case REBOUNDING_PROJ:
	      fullblit(xoff,yoff,59,7,screen_buffer,stat_picts[24]);
	      break;
   };
}
/***************************************************************************/
void display_ammo (int plyris, int weapon, int ammo, int clr)
{
 int num1=0;
 int num2=0;
 int xoff,yoff;


  /* exit if not wanted */
  if (!display_stats)
    return;

   /* Set the appropiate offset values for text writing */
   switch (gametype)
   {
    case FULLGAME : xoff=176;
		    yoff=187;
		    break;
    default  : /* anything but FULLGAME */
			    if (plyris==0)
			      xoff=102;
			    else
			      xoff=277;

			    yoff=173;
			    break;
   }

  /* Now get the correct numbers to blit, and blit em */
  if (weapon != NORMAL_PROJ)
    {
       if (ammo < 10)
	  num2=ammo;
       else
	 {
	  num1=ammo / 10;
	  num2=ammo % 10;
	 }

	 /* Next, print out the ammo number using num1 and num2 */
	 color_blit(xoff,yoff,CHARPICTWIDTH,CHARPICTHEIGHT,clr,
		    screen_buffer,character_picts[num1]);
	 color_blit(xoff+6,yoff,CHARPICTWIDTH,CHARPICTHEIGHT,clr,
		    screen_buffer,character_picts[num2]);
    }

  else /* weapon must not be NORMAL_PROJ */
    {
     /* do nothing? */
      //out_text("UNLIMITED",xoff,yoff,clr,screen_buffer);
    }
}
/***************************************************************************/
void change_current_weapon(int tanknum)
{
 int check_weapon;
 int count=0;

  /* Assign the current weapon to check_weapon */
  check_weapon=tanks[tanknum].current_weapon;


  for (count=0;count<MAXWEAPONS;count++)
   {
    /* Increment check_weapon because you want to see if the next one */
    /* is available. (i.e  if it has ammo available for it).          */
    check_weapon++;

    /* Make sure you don't go past last available weapon. If so reset */
    /* to original starting weapon and start checking there.          */
    if (check_weapon == MAXWEAPONS)
      check_weapon=NORMAL_PROJ;

    /* if ammo exists for next weapon, or you want to use default weapon */
    if ((tanks[tanknum].weapon_ammo[check_weapon]) ||
	(check_weapon==NORMAL_PROJ))
     {
      /* If tank is the HUMAN user, show the stats to screen */
      if (tanks[tanknum].playertype==HUMAN)
       {
	/* erase old weapon */
	display_weapon (tanknum,tanks[tanknum].current_weapon,BLACK);
	/* erase old ammo */
	display_ammo (tanknum,tanks[tanknum].current_weapon,
		      tanks[tanknum].previous_ammo,
		      BLACK);
	/* dehighlight the weapon */
	dehighlight_weapon (tanknum,tanks[tanknum].current_weapon);

	/* reset weapon & previous ammo */
	tanks[tanknum].current_weapon=check_weapon;
	tanks[tanknum].previous_ammo=
	       tanks[tanknum].weapon_ammo[check_weapon];

	/* show new weapon  */
	display_weapon (tanknum,tanks[tanknum].current_weapon,OLIVEDRAB);
	/* show new ammo */
	display_ammo (tanknum,tanks[tanknum].current_weapon,
		      tanks[tanknum].weapon_ammo[check_weapon],
		      OLIVEDRAB);
	/* highlight the weapon */
	highlight_weapon (tanknum,check_weapon);

       }
      else /* tank is a computer so just set the new weapon */
       {
	/* Reset weapon and previous_ammo */
	tanks[tanknum].current_weapon=check_weapon;
	tanks[tanknum].previous_ammo=
	       tanks[tanknum].weapon_ammo[check_weapon];
       }

      return;
     }

   } /* end for loop */
}
/***************************************************************************/
void change_to_a_specific_weapon(int weapon,int tanknum)
{

     /* If tank is the HUMAN user, show the stats to screen */
      if (tanks[tanknum].playertype==HUMAN)
       {
	/* erase old weapon */
	display_weapon (tanknum,tanks[tanknum].current_weapon,BLACK);
	/* erase old ammo */
	display_ammo (tanknum,tanks[tanknum].current_weapon,
		      tanks[tanknum].previous_ammo,
		      BLACK);
	/* dehighlight the weapon */
	dehighlight_weapon (tanknum,tanks[tanknum].current_weapon);

	/* reset weapon & previous ammo */
	tanks[tanknum].current_weapon=weapon;
	tanks[tanknum].previous_ammo=
	       tanks[tanknum].weapon_ammo[weapon];

	/* show new weapon  */
	display_weapon (tanknum,tanks[tanknum].current_weapon,OLIVEDRAB);
	/* show new ammo */
	display_ammo (tanknum,tanks[tanknum].current_weapon,
		      tanks[tanknum].weapon_ammo[weapon],
		      OLIVEDRAB);
	/* highlight the weapon */
	highlight_weapon (tanknum,weapon);
       }

      else /* tank is a computer so just set the new weapon */
       {
	/* Reset weapon and previous_ammo */
	tanks[tanknum].current_weapon=weapon;
	tanks[tanknum].previous_ammo=
	       tanks[tanknum].weapon_ammo[weapon];
       }
}
/***************************************************************************/
void dehighlight_weapon (int plyris,int weapon)
{
 int xoff,yoff;

  /* exit if not wanted */
  if (!display_stats)
    return;

  switch (weapon)
  {
   case NORMAL_PROJ : yoff=172;
		      break;
   case REBOUNDING_PROJ : yoff=181;
			  break;
   case PIERCING_PROJ   : yoff=190;
			  break;
  };

  switch (gametype)
  {
   case FULLGAME : xoff=275; break;
   default :/* either TWOPLYR_FULLGAME,MATCH, or NULL_MODEM_MATCH */
	    if (plyris==0)
		 xoff=130;
	    else
		 xoff=305;
	    break;
  }

  /* Now use fullblit, so it blits black pixels too */
  fullblit (xoff,yoff,9,8,screen_buffer,stat_picts[19+weapon]);

 // display_abbreviation(weapon,WEAPON_ABRV_COLOR);
}
/************************************************************************/
void highlight_weapon (int plyris,int weapon)
{
 int xoff,yoff;

  /* exit if not wanted */
  if (!display_stats)
    return;

  switch (weapon)
  {
   case NORMAL_PROJ : yoff=172;
		      break;
   case REBOUNDING_PROJ : yoff=181;
			  break;
   case PIERCING_PROJ   : yoff=190;
			  break;
  };

  switch (gametype)
  {
   case FULLGAME : xoff=275; break;
   default :/* either TWOPLYR_FULLGAME,MATCH, or NULL_MODEM_MATCH */
	      if (plyris==0)
		     xoff=130;
	      else xoff=305;
	      break;
  }

  blit (xoff,yoff,9,8,screen_buffer,stat_picts[16+weapon]);

 // display_abbreviation(weapon,WEAPON_ABRV_COLOR);
}
/************************************************************************/
void display_weapon_abbreviations(int plyris)
/* This function prints out the one letter abbreviations of certain */
/* special weapons a player might of gathered.                      */
{
 int i;

  /* exit if not wanted */
  if (!display_stats)
    return;


 for (i=0;i<MAXWEAPONS;i++)
  {
    if ((tanks[plyris].weapon_ammo[i]) || (i==NORMAL_PROJ))
      display_abbreviation (plyris,i,!BLACK);
  }

  highlight_weapon(plyris,tanks[plyris].current_weapon);
}
/************************************************************************/
void display_abbreviation (int plyris, int num, int clr)
{
  int xoff,yoff;

  /* exit if not wanted */
  if (!display_stats)
    return;

   switch (gametype)
   {
    case FULLGAME : xoff=275;
		    break;
    default  : /* anything but FULLGAME */
			    if (plyris==0)
			      xoff=130;
			    else
			      xoff=305;
			    break;
   }

  switch (num)
  {
    case NORMAL_PROJ : yoff=172;break;
    case REBOUNDING_PROJ : yoff=181;break;
    case PIERCING_PROJ :   yoff=190;break;
  };

  if (clr!=BLACK) /* erase not wanted */
     { /* use 19+num cuz 19,20,21 are dehighlited abbrevs, and I'm */
       /* using the fact that NORMAL_PROJ=0 in enum type etc..*/
       blit (xoff,yoff,9,8,screen_buffer,stat_picts[19+num]);
     }
  else /* we want to erase the abbreviation, barblit will include black */
    bar_blit(xoff,yoff,9,8,BLACK,screen_buffer);
}
/************************************************************************/
#define _WEAPONS.C
#endif